!
!  SUBROUTINES FOR COMMUNICATING WITH GENERIC I/O ROUTINES
!

SUBROUTINE csv_file_read( file_name, values, mm, nn, istat )


!*****************************************************************************
!
!	CSV_FILE_READ reads data from a CSV file
!
!  Modified:
!
!    28 AUGUST 2012
!
!  Author:
!
!    Justin van de Ven
!
!  Parameters:
!
!	file_name	: full name of file to be saved
!	values		: matrix to read data into
!	mm			: number of rows in matrix
!	nn			: number of cols in matrix
!
!*****************************************************************************

!DEC$ ATTRIBUTES DLLEXPORT::csv_file_read

USE io_working

IMPLICIT NONE


integer(4), intent(in) :: mm, nn
integer(4), intent(out) :: istat
real(8), intent(out) :: values(mm,nn)
character(*) :: file_name

! local variables
integer(4) :: istat1, file_id, ii


!*****************************************************************************
!	begin code
!*****************************************************************************


!*****************************************************************************
!	open file to read from
!*****************************************************************************
istat = 0
call get_unit( file_id )
open ( unit = file_id, file = file_name, iostat = istat1 )
if ( istat1.ne.0 ) then

	istat = istat1
else
	!*****************************************************************************
	!	read data from file
	!*****************************************************************************
	do ii = 1, mm

		if ( istat.eq.0 ) then

			read ( file_id, *, IOSTAT=istat1 ) values(ii,:)
			istat = istat1
		end if
	end do
	close ( file_id )
end if


END SUBROUTINE csv_file_read


SUBROUTINE csv_file_write( file_name, values, mm, nn )

!*****************************************************************************
!
!	CSV_FILE_WRITE writes data to CSV file
!
!  Modified:
!
!    28 AUGUST 2012
!
!  Author:
!
!    Justin van de Ven
!
!  Parameters:
!
!	file_name	: full name of file to be saved
!	values		: matrix to read data into
!	mm			: number of rows in matrix
!	nn			: number of cols in matrix
!
!*****************************************************************************

!DEC$ ATTRIBUTES DLLEXPORT::csv_file_write

USE io_working

IMPLICIT NONE


integer(4), intent(in) :: mm, nn
real(8), intent(in) :: values(mm,nn)
character(*) :: file_name

! local variables
integer(4) :: file_id, istat, ii, jj
character(nn*16) :: output_line


!*****************************************************************************
!	begin code
!*****************************************************************************


!*****************************************************************************
!	open file to write to
!*****************************************************************************
call get_unit( file_id )
open ( unit = file_id, file = file_name, status = 'replace', iostat = istat )
if ( istat.ne.0 ) then

	print *, 'error writing CSV file'
	print *, 'could not open "' // trim ( file_name ) // '".'
	pause 'press enter to exit'
	continue
	stop
end if


!*****************************************************************************
!	write output to file
!*****************************************************************************
do ii = 1, mm

	output_line = '  '
	do jj = 1, nn

		call csv_data_append( values(ii,jj), output_line )
	end do
	write ( file_id, '(a)' ) trim ( output_line )
end do
close ( file_id )


END SUBROUTINE csv_file_write


SUBROUTINE excel_comms( OpenFileName, SaveFileName, ReadFlag, nn, SheetName, RowStart, ColStart, RowNumb, ColNumb, maxRow, maxCol, xx )

!**************************************************************
!
!	Subroutine to load in non-graduate parameters
!
!**************************************************************

!DEC$ ATTRIBUTES DLLEXPORT::excel_comms

USE io_working
USE FCppStaticComs

IMPLICIT none


character(2000), intent(in) :: OpenFileName, SaveFileName
character(256), intent(in) :: SheetName(nn)
integer(4), intent(in) :: nn, ReadFlag, maxRow, maxCol
integer(4), intent(in) :: RowStart(nn), ColStart(nn), RowNumb(nn), ColNumb(nn)
real(8) :: xx(maxRow,maxCol,nn)


!*************************************************************
!	begin code
!*************************************************************


!*************************************************************
!	initialise storeage arrays and indices
!*************************************************************
if ( ReadFlag.eq.1 ) then
	
	xx = 0.0
end if
call FCppComms( OpenFileName, SaveFileName, ReadFlag, nn, SheetName, RowStart, ColStart, RowNumb, ColNumb, maxRow, maxCol, xx )


END SUBROUTINE excel_comms

	
SUBROUTINE JobFileReplicate(RootSimName, NoReps, StartNo, StartIndex)

!*************************************************
!
!	subroutine to evaluate means and variances of 
!	replicated simulated statistics
!
!*************************************************

!DEC$ ATTRIBUTES DLLEXPORT::JobFileReplicate

USE io_working
USE FCppStaticComs
USE ifport
USE dfwin

IMPLICIT none


character(256), intent(in) :: RootSimName
integer(4), intent(in) :: NoReps, StartNo, StartIndex

! local variables
integer(4) :: istat
character(2000) :: DirName


!*************************************************
!	begin code
!*************************************************
istat = getcwd(DirName)
call FCppJobFileRepn( RootSimName, DirName, NoReps, StartNo, StartIndex )


END SUBROUTINE JobFileReplicate



