!
!  SUBROUTINES FOR COMMUNICATING WITH SPECIFIC EXCEL FILES
!

SUBROUTINE load_EstmnStats( model_dir, inFileName, job_name, m_samp, VCV_samp, I_samp_inv, MaxMoments, nn_est1, VCV_est1, J_est1 )

!**************************************************************
!
!	Subroutine to load in estimation statistics
!
!**************************************************************

!DEC$ ATTRIBUTES DLLEXPORT::load_EstmnStats

!USE io_working
USE FCppStaticComs


IMPLICIT none

character(2000), intent(in) :: model_dir
character(256), intent(in) :: inFileName, job_name
integer(4), intent(in) :: MaxMoments, nn_est1
real(8) :: m_samp(MaxMoments), VCV_samp(MaxMoments, MaxMoments), I_samp_inv(MaxMoments, MaxMoments)
real(8) :: VCV_est1(nn_est1,nn_est1), J_est1(nn_est1)


!*************************************************************
!	begin code
!*************************************************************
m_samp = 0.0
VCV_samp = 0.0
I_samp_inv = 0.0_8
VCV_est1 = 0.0
J_est1 = 0.0
call FCppEstimation( model_dir, inFileName, job_name, m_samp, VCV_samp, I_samp_inv, MaxMoments, nn_est1, VCV_est1, J_est1 )


END SUBROUTINE load_EstmnStats


SUBROUTINE JobFilesSave( short_run_flag, grid_load, grid_ref, grid_dir, job_dir, model_dir, base_dir, inFileName, job_name )

!**************************************************************
!
!	Subroutine to save all model parameter files
!
!**************************************************************

!DEC$ ATTRIBUTES DLLEXPORT::JobFilesSave

USE ifport
USE io_working

IMPLICIT none


integer(4), intent(in) :: short_run_flag, grid_load
character(256) :: inFileName, job_name, grid_ref
character(2000) :: grid_dir, job_dir, model_dir, base_dir

! local variables
logical :: chk_flag
character(2000) :: inFileName1, outfilename, anal_dir
integer(4) :: new_run, istat


!*************************************************************
!	begin code
!*************************************************************
if ( short_run_flag.eq.0 ) then

	job_dir = trim(model_dir) // '\simulations\' // trim(job_name)
	if ( grid_load.eq.0 ) then
				
		grid_dir = trim(job_dir) // '\grids'
	else
				
		grid_dir = trim(model_dir) // '\simulations\' // trim(grid_ref) // '\grids'
	end if
	anal_dir = trim(job_dir) // '\anals'
	chk_flag = changedirqq(job_dir)
	if ( chk_flag.eq..true. ) then
	! directory exists - not a new run so no need to create folders and copy files

		new_run = 0_4
	else
	! file doesn't exist - need to create folders and copy files

		new_run = 1_4
	end if
	if ( new_run.eq.1 ) then
	! create job directories, and file names

		istat = makedirqq(job_dir)
		istat = makedirqq(grid_dir)
		istat = makedirqq(anal_dir)
		istat = changedirqq(job_dir)
		istat = getcwd(job_dir)

		inFileName1 = trim(model_dir) // '\' // trim(inFileName)
		outFileName = trim(job_dir) // '\Job File.xls'
		call file_copy( trim(inFileName1), trim(outFileName) )

		inFileName1 = trim(base_dir) // '\DEMOGRAPHIC PARAMETERS.xls'
		outFileName = trim(job_dir) // '\DEMOGRAPHIC PARAMETERS.xls'
		call file_copy( trim(inFileName1), trim(outFileName) )

		inFileName1 = trim(base_dir) // '\EMPLOYEE WAGE PARAMETERS.xls'
		outFileName = trim(job_dir) // '\EMPLOYEE WAGE PARAMETERS.xls'
		call file_copy( trim(inFileName1), trim(outFileName) )

		inFileName1 = trim(base_dir) // '\FINANCIAL PARAMETERS.xls'
		outFileName = trim(job_dir) // '\FINANCIAL PARAMETERS.xls'
		call file_copy( trim(inFileName1), trim(outFileName) )

		inFileName1 = trim(base_dir) // '\HEALTH PARAMETERS.xls'
		outFileName = trim(job_dir) // '\HEALTH PARAMETERS.xls'
		call file_copy( trim(inFileName1), trim(outFileName) )

		inFileName1 = trim(base_dir) // '\MIGRATION PARAMETERS.xls'
		outFileName = trim(job_dir) // '\MIGRATION PARAMETERS.xls'
		call file_copy( trim(inFileName1), trim(outFileName) )

		inFileName1 = trim(base_dir) // '\NONGRADUATE PARAMETERS.xls'
		outFileName = trim(job_dir) // '\NONGRADUATE PARAMETERS.xls'
		call file_copy( trim(inFileName1), trim(outFileName) )

		inFileName1 = trim(base_dir) // '\PUBLIC EMPLOYEE PARAMETERS.xls'
		outFileName = trim(job_dir) // '\PUBLIC EMPLOYEE PARAMETERS.xls'
		call file_copy( trim(inFileName1), trim(outFileName) )

		inFileName1 = trim(base_dir) // '\SELF EMPLOYMENT PARAMETERS.xls'
		outFileName = trim(job_dir) // '\SELF EMPLOYMENT PARAMETERS.xls'
		call file_copy( trim(inFileName1), trim(outFileName) )
	elseif ( short_run_flag.eq.1 ) then

		grid_dir = trim(model_dir) // '\simulations\' // trim(inFileName1) // '\grids'
	else

		! always save job file
		inFileName1 = trim(model_dir) // '\' // trim(inFileName)
		outFileName = trim(job_dir) // '\Job File.xls'
		call file_copy( trim(inFileName1), trim(outFileName) )

		istat = changedirqq(job_dir)
		istat = getcwd(job_dir)
	end if
end if


END SUBROUTINE JobFilesSave


SUBROUTINE excel_param_write( xArr, mm, NoColumns, NoColumns_continuous, FileName, outputFileName, SheetName, open_file, close_file, excelapp, workbooks, workbook, worksheets, worksheet )

!**************************************************************
!
!	Subroutine to write model parameters into analysis sheet
!
!**************************************************************

!DEC$ ATTRIBUTES DLLEXPORT::excel_param_write

USE IFCOM
USE IFWINTY
USE IFAUTO
USE KERNEL32
USE excelComms
USE io_working

IMPLICIT none

integer(ptr) :: excelapp, workbooks, workbook, worksheets, worksheet
integer(4), intent(in) :: mm, NoColumns, NoColumns_continuous, open_file, close_file
real(8), intent(in) :: xArr(mm,NoColumns)
character(2000), intent(in) :: FileName, outputFileName
character(256), intent(in) :: SheetName

! local variables
integer(4) :: ii
character(256) :: outsheet_temp

integer(4) :: look_forward, ColHere
character(10) :: str_row
character(56) :: str1, str2
integer(b32) :: status
character(2) :: office_version_string
integer(ptr) :: bstr3
type (variant) :: vBSTR3



!*************************************************************
!	begin code
!*************************************************************
status = 0
if ( open_file.eq.1 ) then
! need to open Excel file - otherwise it is already open


	!*************************************************************
	!	open excel file
	!*************************************************************
	excelapp = 0
	workbooks = 0
	workbook = 0
	worksheets = 0
	worksheet = 0

	! Create an Excel object.  Try Office 2016 (16), 2013 (15),  2010 (14 and 13), then 2007 (12).
	CALL COMINITIALIZE(status)
	do  ii= 16, 3, -1 !Original lowest was version 12 for supporting 16384 columns, below that version max is 256
		
		write(office_version_string, '(i2)') ii
		call comCreateObject("Excel.Application." // office_version_string, excelapp, status)
		if (excelapp /= 0) exit ! If successful, exit the loop
	end do
	if ( status.ne.0 ) go to 100

	! Turn off Excel applicatin window and warning messages
	call $Application_SetVisible(excelapp, .false.)
	call $Application_SetDisplayAlerts(excelapp, .false., $STATUS = status)

	! Get the WORKBOOKS object
	workbooks = $Application_GetWorkbooks(excelapp, $STATUS = status)
	if ( status.ne.0 ) go to 100

	! Open the specified spreadsheet file
	workbook = Workbooks_Open(workbooks, FileName, $STATUS = status)
	if ( status.ne.0 ) go to 100

	! Get the worksheets object
	worksheets = $Workbook_GetWorksheets(workbook, $STATUS = status)
	if ( status.ne.0 ) go to 100
end if
		
! Get the worksheet
worksheet = $Workbook_GetActiveSheet(workbook, status)
if ( status.ne.0 ) go to 100
outSheet_temp = $Worksheet_GetName(worksheet, status)
look_forward = 0
do while ( SheetName.ne.outSheet_temp )

	if ( look_forward.eq.0 ) then

		worksheet = $worksheet_getprevious(worksheet, status)
		if ( (status.ne.0).or.(worksheet.eq.0) ) then

			look_forward = 1
			worksheet = $Workbook_GetActiveSheet(workbook, status)
		end if
	end if
	if ( look_forward.eq.1 ) then

		worksheet = $worksheet_getnext(worksheet, status)
		if ( status.ne.0 ) go to 100
	end if
	outSheet_temp = $worksheet_GetName(worksheet, status)
end do	


if ( (mm+1).lt.10 ) then

	write( str_row, '(i1)' ) (mm+1)
elseif ( (mm+1).lt.100 ) then

	write( str_row, '(i2)' ) (mm+1)
elseif ( (mm+1).lt.1000 ) then

	write( str_row, '(i3)' ) (mm+1)
else

	write( str_row, '(i4)' ) (mm+1)
end if
do ii = 1, NoColumns
	! loop over columns

	if ( ii.le.NoColumns_continuous ) then

		ColHere = 70 + ii
	else

		ColHere = 70 + NoColumns_continuous + (ii - NoColumns_continuous) * 2
	end if
	if ( ColHere.le.90_4 ) then

		str1 = char(ColHere) // "2"
		str2 = char(ColHere) // trim(str_row)
	else

		str1 = char( 64+((ColHere-65)/26) ) // char( ColHere-26*((ColHere-65)/26) ) // "2"
		str2 = char( 64+((ColHere-65)/26) ) // char( ColHere-26*((ColHere-65)/26) ) // trim(str_row)
	end if
	call ExcelArray(str1, str2, xArr(:,ii), size(xArr, DIM=1), 1, worksheet, 1) 
end do
if ( close_file.eq.1 ) then
	! save and close the workbook

	if ( FileName.eq.outputFileName ) then

		call $Workbook_Save(workbook)
	else

		CALL VariantInit(vBSTR3)
		vBSTR3%VT = VT_BSTR
		bstr3 = ConvertStringToBSTR(outputFileName)
		vBSTR3%VU%PTR_VAL = bstr3
		call $Workbook_SaveAs(workbook, vBSTR3)
	end if
	call ExcelClose( worksheet, worksheets, workbook, workbooks, excelapp )
end if


!*******************************************************
!   error reporting
!*******************************************************
100		if ( status.ne.0 ) then

	print *, 'problem communicating with Excel when loading in data from main job file'
	pause 'press enter to exit'
	stop
end if


END SUBROUTINE excel_param_write
