MODULE FCppStaticComs

!*******************************************************************
!	
!	MODULE contains tools for communicating with C++
!
!*******************************************************************

IMPLICIT none


CONTAINS

	
SUBROUTINE IntToChar( IntStore, len_store, text, len_text )


!*******************************************************************
!	
!	MODULE contains tools for communicating with C++
!
!*******************************************************************

IMPLICIT none


integer(4), intent(in) :: len_store, len_text
integer(4) :: IntStore(len_store)
character(len_text) :: text

! local variables
integer(4) :: ii


!*******************************************************************
!	begin code
!*******************************************************************
ii = 1
do  ii= 1, min(len_store, len_text)
	
	if (IntStore(ii).eq.0) then
		
		! text = text(1:ii-1) // char(0)
		exit ! If successful, exit the loop
	else
		if ( ii.eq.1 ) then
			
			text = achar(IntStore(ii))
		else
			
			text = text(1:ii-1) // achar(IntStore(ii))
		end if
	end if
end do
continue


END SUBROUTINE IntToChar

	
SUBROUTINE CharToInt( clen, text, IntStore )


!*******************************************************************
!	
!	MODULE contains tools for communicating with C++
!
!*******************************************************************

IMPLICIT none


integer(4), intent(in) :: clen
integer(4) :: IntStore(clen)
character :: text(clen)

! local variables
integer(4) :: ii


!*******************************************************************
!	begin code
!*******************************************************************
ii = 1
do while ( ii.le.clen ) 

	IntStore(ii) = iachar(text(ii))
	ii = ii + 1
end do
continue


END SUBROUTINE CharToInt

	
SUBROUTINE FCppComms( OpenFileName, SaveFileName, ReadFlag, nn, SheetName, RowStart, ColStart, RowNumb, ColNumb, maxRow, maxCol, xx )

!************************************************************************************
!	
!	MODULE contains generic routine for communicating with existing Excel file via C#
!
!	OpenFileName =	complete path for file to open
!	SaveFileName =	complete path for saving file
!	ReadFlag =		1 if routine is to read in data from existing Excel file and do not save file
!					otherwise write to file, and save file as SaveFileName
!	nn =			number of contiguous arrays to read in / write to
!	SheetName =		array of sheet names for each contiguous read/write
!	RowStart =		array of first row to write to / read from for each contiguous
!	ColStart =		array of first column to write to / read from for each contiguous
!	RowNumb =		array of number of rows in contiguous read/write
!	ColNumb =		array of number of cols in contiguous read/write
!	maxRow =		maximum number of rows in all contiguous regions
!	maxCol =		maximum number of columns in all contiguous regions
!	xx =			(maxRow, maxCol, nn) storage array for read/writes
!	
!************************************************************************************

use, intrinsic :: ISO_C_BINDING

IMPLICIT none

character(2000), intent(in) :: OpenFileName, SaveFileName
character(256), intent(in) :: SheetName(nn)
integer(4), intent(in) :: nn, ReadFlag, maxRow, maxCol
integer(4), intent(in) :: RowStart(nn), ColStart(nn), RowNumb(nn), ColNumb(nn)
real(8) :: xx(maxRow,maxCol,nn)

! local variables
character(2000) :: OpenFileName1, SaveFileName1
integer(4) :: chrlen(2), int_vec(4)
integer(4) :: iSheetName(256,nn)
integer(4) :: ii, clen


!*************************************************************
!	begin code
!*************************************************************

! C++ interfaces
INTERFACE

	SUBROUTINE CppCSComms( chrlen, OpenFileName, SaveFileName, int_vec, iSheetName, RowStart, ColStart, RowNumb, ColNumb, xx ) bind(C, name="CppCSComms")

		use, intrinsic :: ISO_C_BINDING
		integer(c_int) :: chrlen(2)
		character(kind=c_char) :: OpenFileName(chrlen(1))
		character(kind=c_char) :: SaveFileName(chrlen(2))
		integer(c_long) :: int_vec(4)
		integer(c_long), dimension(256*int_vec(1)) :: iSheetName
		integer(c_long) :: RowStart(int_vec(1))
		integer(c_long) :: ColStart(int_vec(1))
		integer(c_long) :: RowNumb(int_vec(1))
		integer(c_long) :: ColNumb(int_vec(1))
		real(c_double) :: xx(int_vec(1)*int_vec(2)*int_vec(3))
	END SUBROUTINE CppCSComms

END INTERFACE	

! run checks to ensure references can be managed
do ii = 1, nn
	
	if ( RowStart(ii).lt.1 ) then

		print *, "Attempt to reference Excel row index less than 1"
		pause "press enter to exit"
		stop
	end if
	if ( ColStart(ii).lt.1 ) then

		print *, "Attempt to reference Excel column index less than 1"
		pause "press enter to exit"
		stop
	end if
	if ( (RowStart(ii)+RowNumb(ii)-1).gt.1000000 ) then

		print *, "Attempt to reference Excel row index greater than 1000000"
		pause "press enter to exit"
		stop
	end if
	if ( (ColStart(ii)+ColNumb(ii)-1).gt.16000 ) then

		print *, "Attempt to reference Excel column index greater than 16000"
		pause "press enter to exit"
		stop
	end if
	if ( RowNumb(ii).lt.1 ) then

		print *, "Attempt to reference fewer than 1 Excel rows"
		pause "press enter to exit"
		stop
	end if
	if ( ColNumb(ii).lt.1 ) then

		print *, "Attempt to reference feser than 1 Excel columns"
		pause "press enter to exit"
		stop
	end if
end do

! preliminary variables
int_vec(1) = nn			!number of contiguous regions to read/write
int_vec(2) = maxRow		!maximum number of rows to read
int_vec(3) = maxCol		!maximum number of cols to read
int_vec(4) = ReadFlag	!flag=1 if read (otherwise write)

iSheetName = 0
do ii = 1, nn
	
	clen = len_trim(SheetName(ii))
	call CharToInt( clen, SheetName(ii), iSheetName(1:clen,ii) )
end do

! call C++ wrapper
OpenFileName1 = trim(OpenFileName) // char(0)
SaveFileName1 = trim(SaveFileName) // char(0)
chrlen(1) = Len(trim(OpenFileName1), kind=c_int)
chrlen(2) = Len(trim(SaveFileName1), kind=c_int)
call CppCSComms( chrlen, trim(OpenFileName1), trim(SaveFileName1), int_vec, iSheetName, RowStart, ColStart, RowNumb, ColNumb, xx )
continue


END SUBROUTINE FCppComms

	
SUBROUTINE FCppJobFileRepn( RootSimName, DirName, NoReps, StartNo, StartIndex )

!************************************************************************************
!	
!	MODULE contains generic routine for communicating with existing Excel file via C#
!
!	OpenFileName =	complete path for file to open
!	SaveFileName =	complete path for saving file
!	ReadFlag =		1 if routine is to read in data from existing Excel file and do not save file
!					otherwise write to file, and save file as SaveFileName
!	nn =			number of contiguous arrays to read in / write to
!	SheetName =		array of sheet names for each contiguous read/write
!	RowStart =		array of first row to write to / read from for each contiguous
!	ColStart =		array of first column to write to / read from for each contiguous
!	RowNumb =		array of number of rows in contiguous read/write
!	ColNumb =		array of number of cols in contiguous read/write
!	maxRow =		maximum number of rows in all contiguous regions
!	maxCol =		maximum number of columns in all contiguous regions
!	xx =			(maxRow, maxCol, nn) storage array for read/writes
!	
!************************************************************************************

use, intrinsic :: ISO_C_BINDING

IMPLICIT none


character(256), intent(in) :: RootSimName
character(2000), intent(in) :: DirName
integer(4), intent(in) :: NoReps, StartNo, StartIndex

! local variables
character(256) :: RootSimName1
character(2000) :: DirName1
integer(4) :: chrlen(2), int_vec(3)


!*************************************************************
!	begin code
!*************************************************************

! C++ interfaces
INTERFACE

	SUBROUTINE CppCSJobFileRepn( chrlen, RootSimName, DirName, int_vec ) bind(C, name="CppCSJobFileRepn")

		use, intrinsic :: ISO_C_BINDING
		integer(c_int) :: chrlen(2)
		character(kind=c_char) :: DirName(chrlen(1))
		character(kind=c_char) :: RootSimName(chrlen(2))
		integer(c_long) :: int_vec(3)
	END SUBROUTINE CppCSJobFileRepn

END INTERFACE	

! preliminary variables
int_vec(1) = NoReps
int_vec(2) = StartNo
int_vec(3) = StartIndex

! call C++ wrapper
DirName1 = trim(DirName) // char(0)
chrlen(1) = Len(trim(DirName1), kind=c_int)
RootSimName1 = trim(RootSimName) // char(0)
chrlen(2) = Len(trim(RootSimName1), kind=c_int)
call CppCSJobFileRepn( chrlen, trim(RootSimName1), trim(DirName1), int_vec )
continue


END SUBROUTINE FCppJobFileRepn


END MODULE FCppStaticComs