MODULE io_working


IMPLICIT NONE

CONTAINS


SUBROUTINE file_copy(file_in, file_out)

!*************************************************************
!
!	subroutine to copy and paste files
!
!*************************************************************

USE kernel32

IMPLICIT NONE


character(*), intent(in) :: file_in, file_out

! local
integer(4) :: istat
logical :: flag


!*************************************************************
!	begin code
!*************************************************************
inquire( file = trim(file_in), exist = flag)
if ( flag ) then

	istat = SetFileAttributes (trim(file_out)//""C, FILE_ATTRIBUTE_NORMAL)
	istat = CopyFile (trim(file_in)//""C, trim(file_out)//""C, FALSE)
	if ( istat.eq.0 ) then

		print *, 'problem encountered when copying file'
		pause 'press enter to exit'
		stop
	end if
else

	print *, 'cannot find file requested for transfer'
	pause 'press enter to exit'
	stop
end if


END SUBROUTINE file_copy


SUBROUTINE csv_data_append ( datum, output_line )

!*****************************************************************************
!
! CSV_DATA_APPEND appends new variable onto the end of a CSV record.
!
!  Parameters:
!
!    Input, real ( kind = 8 ) datum, the value to be appended
!
!    Input/output, character ( len = * ) output_line, the CSV record.
!
!*****************************************************************************

IMPLICIT none


real(8), intent(in) :: datum
character(*) :: output_line

! local variables
integer(4) :: ii, intgr, numb_len
character(5) :: fmat


!******************************************************************************
!	begin code
!******************************************************************************

! locate last used location in RECORD.
ii = len_trim ( output_line )

! append comma
if ( ii.gt.0 ) then
	! not first record

	ii = ii + 1
	output_line(ii:ii) = ','
end if

! append value after identifying relevant detail
if ( datum.eq.0.0 ) then
	! record as 0

	ii = ii + 1
	output_line(ii:ii) = '0'
elseif ( datum.lt.(2.0**32) ) then

	if ( datum.eq.real(int(datum)) ) then
		! record as integer

		intgr = int( datum )
		call i4_width ( numb_len, intgr )
		write ( fmat, '(a,i2,a)' ) '(i', numb_len, ')'
		write ( output_line(ii+1:ii+numb_len), fmat ) intgr
	elseif ( (abs(datum).lt.0.2).or.(abs(datum).ge.10000.0) ) then
		! want to allow for high degree of accuracy in relation to Value function

		write ( output_line(ii+1:ii+15), '(g15.7)' ) datum
	else

		write ( output_line(ii+1:ii+13), '(g13.5)' ) datum
	end if
else

	if ( (abs(datum).lt.0.2).or.(abs(datum).ge.10000.0) ) then
		! want to allow for high degree of accuracy in relation to Value function

		write ( output_line(ii+1:ii+15), '(g15.7)' ) datum
	else

		write ( output_line(ii+1:ii+13), '(g13.5)' ) datum
	end if
end if


END SUBROUTINE csv_data_append


SUBROUTINE csv_file_read2( file_name, values, mm, nn, istat )


!*****************************************************************************
!
!	CSV_FILE_READ reads data from a CSV file
!
!  Modified:
!
!    28 AUGUST 2012
!
!  Author:
!
!    Justin van de Ven
!
!  Parameters:
!
!	file_name	: full name of file to be saved
!	values		: matrix to read data into
!	mm			: number of rows in matrix
!	nn			: number of cols in matrix
!
!*****************************************************************************

IMPLICIT NONE


integer(4), intent(in) :: mm, nn
integer(4), intent(out) :: istat
real(8), intent(out) :: values(mm,nn)
character(*) :: file_name

! local variables
integer(4) :: istat1, file_id, ii


!*****************************************************************************
!	begin code
!*****************************************************************************


!*****************************************************************************
!	open file to read from
!*****************************************************************************
istat = 0
values = 0.0
call get_unit( file_id )
open ( unit = file_id, file = file_name, iostat = istat1 )
if ( istat1.ne.0 ) then

	istat = istat1
else
	!*****************************************************************************
	!	read data from file
	!*****************************************************************************
	do ii = 1, mm

		if ( istat.eq.0 ) then

			read ( file_id, *, IOSTAT=istat1 ) values(ii,:)
			istat = istat1
		end if
	end do
	close ( file_id )
end if


END SUBROUTINE csv_file_read2


subroutine csv_record_append_i4 ( i4, record )

!*****************************************************************************
!
!! CSV_RECORD_APPEND_I4 appends an I4 to a CSV record.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license.
!
!  Modified:
!
!    24 November 2008
!
!  Author:
!
!    John Burkardt
!
!  Parameters:
!
!    Input, integer ( kind = 4 ) I4, the integer to be appended
!
!    Input/output, character ( len = * ) RECORD, the CSV record.
!
!*****************************************************************************

implicit none

character ( len = 5 )  fmat
integer   ( kind = 4 ) i
integer   ( kind = 4 ) i4
integer   ( kind = 4 ) i4_len
character ( len = * )  record
!
!  Locate last used location in RECORD.
!
i = len_trim ( record )
!
!  Append comma.
!
if ( 0 < i ) then
	i = i + 1
	record(i:i) = ','
end if
!
!  Determine "width" of I4.
!
call i4_width ( i4_len, i4 )
!
!  Create format for I4.
!
write ( fmat, '(a,i2,a)' ) '(i', i4_len, ')'
!
!  Write I4 to RECORD.
!
write ( record(i+1:i+i4_len), fmat ) i4

return
end subroutine csv_record_append_i4


subroutine csv_record_append_r4 ( r4, record )

!*****************************************************************************
!
!! CSV_RECORD_APPEND_R4 appends an R4 to a CSV record.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license.
!
!  Modified:
!
!    29 November 2008
!
!  Author:
!
!    John Burkardt
!
!  Parameters:
!
!    Input, real ( kind = 8 ) R4, the value to be appended
!
!    Input/output, character ( len = * ) RECORD, the CSV record.
!
!*****************************************************************************

implicit none

character ( len = 5 )  fmat
integer   ( kind = 4 ) i
integer   ( kind = 4 ) i4
integer   ( kind = 4 ) i4_len
real      ( kind = 4 ) r4
character ( len = * )  record
!
!  Locate last used location in RECORD.
!
i = len_trim ( record )
!
!  Append comma.
!
if ( 0 < i ) then
	i = i + 1
	record(i:i) = ','
end if

if ( r4 == 0.0E+00 ) then
	i = i + 1
	record(i:i) = '0'
else if ( r4 == real ( int ( r4 ), kind = 4 ) ) then
	i4 = int ( r4 )
	call i4_width ( i4_len, i4 )
	write ( fmat, '(a,i2,a)' ) '(i', i4_len, ')'
	write ( record(i+1:i+i4_len), fmat ) i4
else
	write ( record(i+1:i+14), '(g14.6)' ) r4
end if

return
end subroutine csv_record_append_r4


subroutine csv_record_append_r8 ( r8, output_line )

!*****************************************************************************
!
! CSV_RECORD_APPEND_R8 appends an R8 to a CSV record.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license.
!
!  Modified:
!
!    29 November 2008
!
!  Author:
!
!    John Burkardt
!
!  Parameters:
!
!    Input, real ( kind = 8 ) R8, the value to be appended
!
!    Input/output, character ( len = * ) RECORD, the CSV record.
!
!*****************************************************************************

implicit none

character ( len = 5 )  fmat
integer   ( kind = 4 ) i
integer   ( kind = 4 ) i4
integer   ( kind = 4 ) i4_len
real      ( kind = 8 ) r8
character ( len = * )  output_line
!
!  Locate last used location in RECORD.
!
i = len_trim ( output_line )
!
!  Append comma.
!
if ( 0 < i ) then
	i = i + 1
	output_line(i:i) = ','
end if

if ( r8 == 0.0E+00 ) then
	i = i + 1
	output_line(i:i) = '0'
else if ( r8 == real ( int ( r8 ), kind = 8 ) ) then
	i4 = int ( r8 )
	call i4_width ( i4_len, i4 )
	write ( fmat, '(a,i2,a)' ) '(i', i4_len, ')'
	write ( output_line(i+1:i+i4_len), fmat ) i4
else
	write ( output_line(i+1:i+17), '(g15.7)' ) r8
end if

return
end subroutine csv_record_append_r8


subroutine csv_record_append_s ( s, record )

!*****************************************************************************
!
! CSV_RECORD_APPEND_S appends a string to a CSV record.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license.
!
!  Modified:
!
!    26 November 2008
!
!  Author:
!
!    John Burkardt
!
!  Parameters:
!
!    Input, character ( len = * ) S, the string to be appended
!
!    Input/output, character ( len = * ) RECORD, the CSV record.
!
!*****************************************************************************

implicit none

integer   ( kind = 4 ) i
character ( len = * )  record
character ( len = * )  s
integer   ( kind = 4 ) s_len
!
!  Locate last used location in RECORD.
!
i = len_trim ( record )
!
!  Append a comma.
!
if ( 0 < i ) then
	i = i + 1
	record(i:i) = ','
end if
!
!  Prepend a quote.
!
i = i + 1
record(I:i) = '"'
!
!  Write S to RECORD.
!
s_len = len_trim ( s )
record(i+1:i+s_len) = s(1:s_len)
i = i + s_len
!
!  Postpend a quote
!
i = i + 1
record(i:i) = '"'

return
end subroutine csv_record_append_s


subroutine get_unit ( iunit )

!*****************************************************************************
!
! GET_UNIT returns a free FORTRAN unit number.
!
!  Discussion:
!
!    A "free" FORTRAN unit number is an integer between 1 and 99 which
!    is not currently associated with an I/O device.  A free FORTRAN unit
!    number is needed in order to open a file with the OPEN command.
!
!    If IUNIT = 0, then no free FORTRAN unit could be found, although
!    all 99 units were checked (except for units 5, 6 and 9, which
!    are commonly reserved for console I/O).
!
!    Otherwise, IUNIT is an integer between 1 and 99, representing a
!    free FORTRAN unit.  Note that GET_UNIT assumes that units 5 and 6
!    are special, and will never return those values.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license.
!
!  Modified:
!
!    26 October 2008
!
!  Author:
!
!    John Burkardt
!
!  Parameters:
!
!    Output, integer ( kind = 4 ) IUNIT, the free unit number.
!
!*****************************************************************************

implicit none

integer ( kind = 4 ) i
integer ( kind = 4 ) ios
integer ( kind = 4 ) iunit
logical              lopen

iunit = 0

do i = 1, 99

	if ( i /= 5 .and. i /= 6 .and. i /= 9 ) then

		inquire ( unit = i, opened = lopen, iostat = ios )

		if ( ios == 0 ) then
			if ( .not. lopen ) then
				iunit = i
				return
			end if
		end if

	end if

end do

return
end subroutine get_unit


SUBROUTINE i4_width( numb_width, ii )

!*************************************************************
!
! I4_WIDTH returns the "width" of an I4.
!
!  Discussion:
!
!    The width of an integer is the number of characters necessary to print it.
!
!    The width of an integer can be useful when setting the appropriate output
!    format for a vector or array of values.
!
!    An I4 is an integer ( kind = 4 ) value.
!
!  Example:
!
!		II  I4_WIDTH
!    -----  -------
!    -1234    5
!     -123    4
!      -12    3
!       -1    2
!        0    1
!        1    1
!       12    2
!      123    3
!     1234    4
!    12345    5
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license.
!
!  Modified:
!
!    30 April 2015
!
!  Author:
!
!    John Burkardt
!	 Justin van de Ven (modified)
!
!  Parameters:
!
!    Input, integer ( kind = 4 ) II, the number whose width is desired.
!
!    Output, integer ( kind = 4 ) numb_width, the number of characters
!    necessary to represent the integer in base 10, including a negative
!    sign if necessary.
!
!*************************************************************

IMPLICIT none


integer(4), intent(in) :: ii
integer(4), intent(out) :: numb_width

! local variables
integer(4) :: ii_abs, kk


!*************************************************************
!	begin code
!*************************************************************
if ( ii.eq.0 ) then

	numb_width = 1
else

	ii_abs = abs(ii)
	numb_width = 0
	kk = 1
	do while ( ii_abs.ge.kk )

		numb_width = numb_width + 1
		kk = kk * 10
	end do
	if ( ii.lt.0 ) then

		numb_width = numb_width + 1
	end if
end if


END SUBROUTINE i4_width


SUBROUTINE report_error( istat )

!********************************************************
!
!   subroutine to report errors from csv_file_read2 routine
!
!********************************************************

IMPLICIT NONE


integer(4) :: istat

print *, 'problem reading model parameters from csv file'
pause 'press enter to exit'
stop
istat = istat


END SUBROUTINE report_error


END MODULE io_working
