﻿using System;
using System.Runtime.InteropServices;
using Excel = Microsoft.Office.Interop.Excel;
using System.Windows.Forms;
/*
using System.Collections.Generic;
using System.IO;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Globalization; // CultureInfo
using System.Diagnostics;
*/
public class AnalCSharp
{
	static string GetColumnName(int index)
	{

		//ROUTINE TO RETURN EXCEL COLUMN LETTER FROM COLUMN INDEX

		const string letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

		var value = "";
		if (index < 1)
		{
			throw new System.ArgumentException("Cannot call to a cell column under 1", "original");
		}
		if (index > 16000)
		{
			throw new System.ArgumentException("Cannot call to a cell column beyond 16000", "original");
		}
		int index1 = index;
		if (index1 > (26 * 27))
		{
			//Three letter reference
			int ref3 = ((index - 1) / 26 / 26) - 1;
			value += letters[ref3];
			index1 = index1 - 26 * 26 * (ref3 + 1);
		}
		if (index1 > 26)
		{
			//Two letter reference
			int ref2 = ((index1 - 1) / 26) - 1;
			value += letters[ref2];
			index1 = index1 - 26 * (ref2 + 1);
		}
		//Single letter reference
		value += letters[index1 - 1];

		return value;
	}

	static void StringToInt(string text, int[] text_store)
	{

		//ROUTINE TO CONVERT string VARIABLE INTO INT VECTOR FOR PORTING TO OTHER LANGUAGES

		int wrk = Math.Min(text.Length, text_store.Length);

		for (int ii = 0; ii < wrk; ii++)
		{
			text_store[ii] = Convert.ToInt32(text[ii]);
		}
		return;
	}

    static string IntToString(int clen, int[] IntText)
    {

		//ROUTINE TO INT ARRAY INTO STRING VARIABLE FOR PORTING FROM OTHER LANGUAGES
		string temp_sep = null;
		string[] text_temp = new string[clen];
		for (int ii = 0; ii < clen; ii++)
        {
			text_temp[ii] = Convert.ToString(Convert.ToChar(IntText[ii]));
        }
		string text = String.Join(temp_sep, text_temp);
		return text;
    }

	public void CSExcelComms(string OpenFileName, string SaveFileName, int[] int_vec, int[] iSheetName, int[] RowStart, int[] ColStart, int[] RowNumb, int[] ColNumb, double[] xx)
	{
        /* 

			ROUTINE TO COMMUNICATE WITH EXISTING EXCEL WORKSHEET

			Routine designed to read/write multiple contiguous regions within existing Excel file

			OpenFileName = path-name of existing excel file to communicate with
			SaveFileName = path-name to save Excel file to once opperations are complete, if directed to write data
			int_vec = vector of (integer) controls:
            iSheetName = array defining sheet names to communicate with (which are transfered in integer form)
            RowStart = array of starting rows for read/write
            ColStart = array of starting columns for read/write
            RowNumb = array of number of rows for read/write
            ColNumb = array of number of columns for read/write
			xx = array that stores read/write values for communication

		*/

        //Declare COM variables
        Excel.Application oXL = null;
		Excel.Workbooks oWBs = null;
		Excel.Workbook oWB = null;
		Excel.Sheets oSheets = null;
		Excel.Worksheet oSheet = null;
		Excel.Range oRng = null;
		//NOTE: these variables declared to avoid double-dot-calling expressions
		// as these prevent COM clean-up, complicating process closure.

		try
		{
            //Define working variables for analysis
			string SheetName;
            object CellRef1;
            object CellRef2;
            object[,] value;
            int[] iSheetArray;
			int tt1;
			int tt2;

            int nn = int_vec[0];        //number of contiguous regions for read/write
			int maxRow = int_vec[1];    //maximum number of rows for read/write
            int maxCol = int_vec[2];    //maximum number of cols for read/write
            int ReadFlag = int_vec[3];  //flag=1 if save

			//Start Excel and get Application object.
			oXL = new Excel.Application();
			oXL.Visible = false;
			oXL.DisplayAlerts = false;

			//Get a new workbook.
			oWBs = oXL.Workbooks;
			oWB = oWBs.Open(OpenFileName, 0, false, 5, "", "", true, Excel.XlPlatform.xlWindows, "", true, false, 0, false, true, false);
            oSheets = oWB.Sheets;

            for (int ss = 0; ss < nn; ss++)
            {
				//Get target worksheet
				tt1 = 0;
				tt2 = 256;
				do
				{
					if (iSheetName[256 * ss + tt1] > 0) tt1 = tt1 + 1;
					else tt2 = tt1;
				}
				while (tt1 < tt2);
                iSheetArray = new int[tt1];
                for (int rr = 0; rr < tt1; rr++) iSheetArray[rr] = iSheetName[256 * ss + rr];
				SheetName = IntToString(tt1, iSheetArray);
                if (oSheet != null) Marshal.ReleaseComObject(oSheet);
                oSheet = (Excel.Worksheet)oSheets[SheetName];
                oSheet.Activate();

                if (RowNumb[ss] * ColNumb[ss] == 1)
                {
                    CellRef1 = GetColumnName(ColStart[ss]);
                    CellRef1 = CellRef1 + RowStart[ss].ToString();
                    if (oRng != null) Marshal.ReleaseComObject(oRng);
                    oRng = oSheet.get_Range(CellRef1);

					if (ReadFlag == 1)
					{
						if (oRng.Value != null) xx[0] = (double)(oRng.Value);
					}
					else oRng.Value = xx[maxRow * maxCol * ss];
                }
                else
                {
                    CellRef1 = GetColumnName(ColStart[ss]);
                    CellRef1 = CellRef1 + RowStart[ss].ToString();
                    CellRef2 = GetColumnName(ColStart[ss] + ColNumb[ss] - 1);
                    CellRef2 = CellRef2 + (RowStart[ss] + RowNumb[ss] - 1).ToString();
                    if (oRng != null) Marshal.ReleaseComObject(oRng);
                    oRng = oSheet.get_Range(CellRef1, CellRef2);

                    if (ReadFlag == 1)
                    {
                        //Read from Excel file
                        value = oRng.Value;
                        for (int ii = 0; ii < RowNumb[ss]; ii++)
                        {
                            for (int jj = 0; jj < ColNumb[ss]; jj++)
                            {
                                if (value[ii + 1, jj + 1] != null) xx[ii + maxRow * jj + maxRow*maxCol * ss] = (double)value[ii + 1, jj + 1];
                            }
                        }
                    }
                    else
                    {
                        value = new object[RowNumb[ss], ColNumb[ss]];
                        for (int ii = 0; ii < RowNumb[ss]; ii++)
                        {
                            for (int jj = 0; jj < ColNumb[ss]; jj++)
                            {
                                value[ii, jj] = xx[ii + maxRow * jj + maxRow*maxCol * ss];
                            }
                        }
                        oRng.Value = value;
                    }
                }
            }

			//Exit and clean-up
			if (ReadFlag == 0) oWB.SaveAs(SaveFileName);
			oWB.Close(0);
			oXL.Quit();

			//release com objects to fully kill excel process from running in the background
			if (oRng != null) Marshal.ReleaseComObject(oRng);
			if (oSheet != null) Marshal.ReleaseComObject(oSheet);
			Marshal.ReleaseComObject(oSheets);
			Marshal.ReleaseComObject(oWB);
			Marshal.ReleaseComObject(oWBs);
			Marshal.ReleaseComObject(oXL);
		}
		catch (Exception theException)
		{
			String errorMessage;
			errorMessage = "Error: ";
			errorMessage = String.Concat(errorMessage, theException.Message);
			errorMessage = String.Concat(errorMessage, " Line: ");
			errorMessage = String.Concat(errorMessage, theException.Source);

			MessageBox.Show(errorMessage, "Error");
		}
	}

	public void CSJobFileRepn(string RootSimName, string DirName, int[] int_vec)
	{
		/* 

			ROUTINE TO REPLICATE JOB FILES FOR CLONED ANALYSIS

		*/

		//Declare COM variables
		Excel.Application oXL = null;
		Excel.Workbooks oWBs = null;
		Excel.Workbook oWB = null;
		Excel.Sheets oSheets = null;
		Excel.Worksheet oSheet = null;
		Excel.Range oRng1 = null;
		Excel.Range oRng2 = null;
		//NOTE: these variables declared to avoid double-dot-calling expressions
		// as these prevent COM clean-up, complicating process closure.

		try
		{
			//Define working variables for analysis
			string SheetName;
			object CellRef1;
			string string_val;
			string string_val1;
			string string_val2;
			string SaveFileName;

			int NoReps = int_vec[0];
			int JobNo = int_vec[1];
			int NameIndex = int_vec[2];

			//Start Excel and get Application object.
			oXL = new Excel.Application();
			oXL.Visible = false;
			oXL.DisplayAlerts = false;

			//Get a new workbook.
			string OpenFileName = DirName + "\\" + RootSimName;
			oWBs = oXL.Workbooks;
			oWB = oWBs.Open(OpenFileName, 0, false, 5, "", "", true, Excel.XlPlatform.xlWindows, "", true, false, 0, false, true, false);
			oSheets = oWB.Sheets;

			//Get target worksheet
			SheetName = "input";
			if (oSheet != null) Marshal.ReleaseComObject(oSheet);
			oSheet = (Excel.Worksheet)oSheets[SheetName];
			oSheet.Activate();

			//Store values to update
			CellRef1 = "A2";
			if (oRng1 != null) Marshal.ReleaseComObject(oRng1);
			oRng1 = oSheet.get_Range(CellRef1);
			if (oRng1.Value != null) string_val1 = (string)(oRng1.Value);
			else string_val1 = "";

			CellRef1 = "E23";
			if (oRng2 != null) Marshal.ReleaseComObject(oRng2);
			oRng2 = oSheet.get_Range(CellRef1);
			if (oRng2.Value != null) string_val2 = (string)(oRng2.Value);
			else string_val2 = "";

			//Generate new job files
			for (int ii = 0; ii < NoReps; ii++)
			{

				string_val = string_val1 + Convert.ToString(NameIndex);
				oRng1.Value = string_val;

				string_val = string_val2 + Convert.ToString(NameIndex);
				oRng2.Value = string_val;

				SaveFileName = DirName + "\\Job File" + Convert.ToString(JobNo) + ".xls";
				oWB.SaveAs(SaveFileName);
				JobNo = JobNo + 1;
				NameIndex = NameIndex + 1;
			}

			//Exit and clean-up
			oWB.Close(0);
			oXL.Quit();

			//release com objects to fully kill excel process from running in the background
			if (oRng1 != null) Marshal.ReleaseComObject(oRng1);
			if (oRng2 != null) Marshal.ReleaseComObject(oRng2);
			if (oSheet != null) Marshal.ReleaseComObject(oSheet);
			Marshal.ReleaseComObject(oSheets);
			Marshal.ReleaseComObject(oWB);
			Marshal.ReleaseComObject(oWBs);
			Marshal.ReleaseComObject(oXL);
		}
		catch (Exception theException)
		{
			String errorMessage;
			errorMessage = "Error: ";
			errorMessage = String.Concat(errorMessage, theException.Message);
			errorMessage = String.Concat(errorMessage, " Line: ");
			errorMessage = String.Concat(errorMessage, theException.Source);

			MessageBox.Show(errorMessage, "Error");
		}
	}
}
