MODULE internal_analyses


!***************************************************************
!
!	Module for analysing results
!
!***************************************************************

IMPLICIT none


CONTAINS


SUBROUTINE BespokeAnalysis(RootSimName)

!*************************************************
!
!	subroutine tailored to bespoke needs
!
!*************************************************

USE Comm_tools
USE ifport
USE dfwin

IMPLICIT none


character(256), intent(in) :: RootSimName

! local variables
character(2000) :: OutDir
character(2000) :: DirName, FileName
character(256) :: SimName, AnalFile
character(4) :: vsn1
character(1) :: vsn2
integer(4), parameter :: mm = 10, nn = 3, mm_s = 42, nn_s = 4
integer(4) :: SimNumb, SimStart
integer(4) :: ii, jj, yy, ll, vv, istat, mm_i
real(8), dimension(mm,nn) :: ben, taxes, cons, vat, hrs, prett, net, ww, pp, ww2, ngtrf
real(8) :: store(mm_s, nn_s)


!*************************************************
!	begin code
!*************************************************
OutDir = "C:\MyFiles\03 PROJECTS\2017\endog savings\01 analysis\summary stats\cross_sections"
! evaluate statistics
yy = 2016
mm_i = 0
do  while ( yy.le.2046 )
! loop over each considered year

	write(vsn1, '(i4)') yy
	DirName = "C:\MyFiles\03 PROJECTS\2017\endog savings\01 analysis\summary stats\cross_sections\" // trim(vsn1) // "\"
	do vv = 0, 2
	! loop over each analysis routine

		! copy means to matrices
		write(vsn2, '(i1)') vv
		FileName = trim(DirName) // trim(RootSimName) // trim(vsn2) // "_means.csv"
		call csv_file_read( FileName, store, mm_s, nn_s, istat )
		ben(1+2*mm_i,vv+1) = store(14,4)
		taxes(1+2*mm_i,vv+1) = store(8,4)
		vat(1+2*mm_i,vv+1) = store(25,4)
		ngtrf(1+2*mm_i,vv+1) = store(26,4)
		cons(1+2*mm_i,vv+1) = store(27,4)
		hrs(1+2*mm_i,vv+1) = store(3,4)
		prett(1+2*mm_i,vv+1) = store(10,4)
		net(1+2*mm_i,vv+1) = store(24,4)
		ww(1+2*mm_i,vv+1) = store(30,4)
		pp(1+2*mm_i,vv+1) = store(35,4)
		ww2(1+2*mm_i,vv+1) = store(36,4)

		! copy sdevs to matrices
		write(vsn2, '(i1)') vv
		FileName = trim(DirName) // trim(RootSimName) // trim(vsn2) // "_sds.csv"
		call csv_file_read( FileName, store, mm_s, nn_s, istat )
		ben(2+2*mm_i,vv+1) = store(14,4)
		taxes(2+2*mm_i,vv+1) = store(8,4)
		vat(2+2*mm_i,vv+1) = store(25,4)
		ngtrf(2+2*mm_i,vv+1) = store(26,4)
		cons(2+2*mm_i,vv+1) = store(27,4)
		hrs(2+2*mm_i,vv+1) = store(3,4)
		prett(2+2*mm_i,vv+1) = store(10,4)
		net(2+2*mm_i,vv+1) = store(24,4)
		ww(2+2*mm_i,vv+1) = store(30,4)
		pp(2+2*mm_i,vv+1) = store(35,4)
		ww2(2+2*mm_i,vv+1) = store(36,4)
	end do
	mm_i = mm_i + 1
	
	! proceed to next loop
	if ( yy.eq.2016 ) then
		
		yy = 2021
	elseif ( yy.eq.2021 ) then
		
		yy = 2026
	elseif ( yy.eq.2026 ) then
		
		yy = 2036
	elseif ( yy.eq.2036 ) then
		
		yy = 2046
	else
		
		yy = yy + 1
	end if
end do

! write output
FileName = trim(OutDir) // '\' // trim(RootSimName) // '_ben.csv'
call csv_file_write( FileName, ben, mm, nn, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // '_taxes.csv'
call csv_file_write( FileName, taxes, mm, nn, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // '_ngtrf.csv'
call csv_file_write( FileName, ngtrf, mm, nn, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // '_cons.csv'
call csv_file_write( FileName, cons, mm, nn, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // '_hrs.csv'
call csv_file_write( FileName, hrs, mm, nn, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // '_prett.csv'
call csv_file_write( FileName, prett, mm, nn, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // '_net.csv'
call csv_file_write( FileName, net, mm, nn, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // '_vat.csv'
call csv_file_write( FileName, vat, mm, nn, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // '_ww.csv'
call csv_file_write( FileName, ww, mm, nn, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // '_pp.csv'
call csv_file_write( FileName, pp, mm, nn, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // '_ww2.csv'
call csv_file_write( FileName, ww2, mm, nn, 3 )

print *, 'completed calculation of summary statistics for replication analysis'


END SUBROUTINE BespokeAnalysis


SUBROUTINE ReplicnConsol(OutDir, RootSimName, AnalFile, SimNumb, SimStart, mm, nn, nn_t)

!*************************************************
!
!	subroutine to evaluate means and variances of 
!	replicated simulated statistics
!
!*************************************************

USE Comm_tools
USE ifport
USE dfwin

IMPLICIT none


character(2000), intent(in) :: OutDir
character(256), intent(in) :: RootSimName, AnalFile
integer(4), intent(in) :: SimNumb, SimStart, mm, nn, nn_t

! local variables
character(2000) :: DirName, FileName
character(256) :: SimName
character(2) :: vsn
integer(4) :: ii, jj, ll, vv, istat
real(8) :: store(mm, nn), consol_store(mm, SimNumb)


!*************************************************
!	begin code
!*************************************************
istat = getcwd(DirName)

! evaluate statistics
consol_store = 0.0
vv = SimStart
do ll = 1, SimNumb

	if ( vv.lt.10_4 ) then

		write(vsn, '(i1)') vv
	else

		write(vsn, '(i2)') vv
	end if
	SimName = trim(RootSimName) // trim(vsn)
	FileName = trim(DirName) // '\simulations\' // trim(SimName) // '\anals\' // trim(AnalFile)
	call csv_file_read( FileName, store, mm, nn, istat )
	if ( istat.eq.0 ) then
		
		do ii = 1, mm
		
			consol_store(ii,ll) = store(ii,nn_t)
		end do
	end if
	vv = vv + 1
end do

! write output
FileName = trim(OutDir) // '\' // trim(RootSimName) // 'consol.csv'
call csv_file_write( FileName, consol_store, mm, SimNumb, 3 )

print *, 'completed calculation of summary statistics for replication analysis'


END SUBROUTINE ReplicnConsol


SUBROUTINE ReplicnSummary(OutDir, RootSimName, AnalFile, SimNumb, SimStart, mm, nn)

!*************************************************
!
!	subroutine to evaluate means and variances of 
!	replicated simulated statistics
!
!*************************************************

USE Comm_tools
USE ifport
USE dfwin

IMPLICIT none


character(2000), intent(in) :: OutDir
character(256), intent(in) :: RootSimName, AnalFile
integer(4), intent(in) :: SimNumb, SimStart, mm, nn

! local variables
character(2000) :: DirName, FileName
character(256) :: SimName
character(2) :: vsn
integer(4) :: ii, jj, ll, vv, istat
real(8) :: store(mm, nn), means(mm, nn), StdDev(mm, nn)


!*************************************************
!	begin code
!*************************************************
istat = getcwd(DirName)

! evaluate statistics
means = 0.0
StdDev = 0.0
vv = SimStart
do ll = 1, SimNumb

	if ( vv.lt.10_4 ) then

		write(vsn, '(i1)') vv
	else

		write(vsn, '(i2)') vv
	end if
	SimName = trim(RootSimName) // trim(vsn)
	FileName = trim(DirName) // '\simulations\' // trim(SimName) // '\anals\' // trim(AnalFile)
	call csv_file_read( FileName, store, mm, nn, istat )
	do ii = 1, mm
		
		do jj = 1, nn
			
			means(ii,jj) = means(ii,jj) + store(ii,jj) / real(SimNumb)
			StdDev(ii,jj) = StdDev(ii,jj) + (store(ii,jj)**2.0) / real(SimNumb)
		end do
	end do
	vv = vv + 1
end do

! complete evaluations of standard deviations
do ii = 1, mm
		
	do jj = 1, nn
			
		StdDev(ii,jj) = (StdDev(ii,jj) - (means(ii,jj)**2.0))**0.5
	end do
end do

! write output
FileName = trim(OutDir) // '\' // trim(RootSimName) // 'means.csv'
call csv_file_write( FileName, means, mm, nn, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // 'sds.csv'
call csv_file_write( FileName, StdDev, mm, nn, 3 )

print *, 'completed calculation of summary statistics for replication analysis'


END SUBROUTINE ReplicnSummary


SUBROUTINE ReplicnSummary2(OutDir, RootSimName, AnalFile, SimNumb, SimStart, mm, nn)

!*************************************************
!
!	subroutine to evaluate means and variances of 
!	replicated simulated statistics
!
!*************************************************

USE Comm_tools
USE ifport
USE dfwin

IMPLICIT none


character(2000), intent(in) :: OutDir
character(256), intent(in) :: RootSimName, AnalFile
integer(4), intent(in) :: SimNumb, SimStart, mm, nn

! local variables
character(2000) :: DirName, FileName
character(256) :: SimName
character(2) :: vsn
integer(4) :: ii, jj, ll, vv, istat
real(8) :: store(mm, nn), means(mm, nn), StdDev(mm, nn), var


!*************************************************
!	begin code
!*************************************************
istat = getcwd(DirName)

! evaluate statistics
means = 0.0
StdDev = 0.0
vv = SimStart
do ll = 1, SimNumb

	if ( vv.lt.10_4 ) then

		write(vsn, '(i1)') vv
	else

		write(vsn, '(i2)') vv
	end if
	SimName = trim(RootSimName) // trim(vsn)
	FileName = trim(DirName) // '\simulations\' // trim(SimName) // '\anals\' // trim(AnalFile)
	call csv_file_read( FileName, store, mm, nn, istat )
	do ii = 1, mm
		
		do jj = 1, nn
			
			means(ii,jj) = means(ii,jj) + store(ii,jj) / real(SimNumb)
			StdDev(ii,jj) = StdDev(ii,jj) + (store(ii,jj)**2.0) / real(SimNumb)
		end do
	end do
	do jj = 1, nn
		
		var = 0.0
		do ii = 15, 22 

			var = var + store(ii,jj)
		end do
		means(8,jj) = means(8,jj) + var / real(SimNumb)
		stdDev(8,jj) = StdDev(8,jj) + (var**2.0) / real(SimNumb)
	end do
	vv = vv + 1
end do

! complete evaluations of standard deviations
do ii = 1, mm
		
	do jj = 1, nn
			
		StdDev(ii,jj) = (StdDev(ii,jj) - (means(ii,jj)**2.0))**0.5
	end do
end do

! write output
FileName = trim(OutDir) // '\' // trim(RootSimName) // 'means.csv'
call csv_file_write( FileName, means, mm, nn, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // 'sds.csv'
call csv_file_write( FileName, StdDev, mm, nn, 3 )

print *, 'completed calculation of summary statistics for replication analysis'


END SUBROUTINE ReplicnSummary2


SUBROUTINE DecileAggregn(OutDir, RootSimName, AnalFile, SimNumb, SimStart, mm, nn)

!*************************************************
!
!	subroutine evaluates age specific population 
!	averages by aggregating over decile statistics
!	then evaluate means and variances of these
!	aggregates derived from replicated simulated 
!	statistics
!
!	OutDir:			directory to save output to
!	RootSimName:	root of replicated simulation name
!	AnalFile:		file name to analyse
!	SimNumb:		number of simulation replications
!	SimStart:		start number of replicated simulations
!	mm:				number of rows in analysis file
!	nn:				number of cols in analysis file
!
!*************************************************

USE Comm_tools
USE ifport
USE dfwin

IMPLICIT none


character(2000), intent(in) :: OutDir
character(256), intent(in) :: RootSimName, AnalFile
integer(4), intent(in) :: SimNumb, SimStart, mm, nn

! local variables
character(2000) :: DirName, FileName
character(256) :: SimName
character(2) :: vsn
integer(4) :: ii, jj, kk, ll, vv, istat
real(8) :: store(mm, nn), store2(mm,nint(real(nn-1)/10.0)), means(mm, nint(real(nn-1)/10.0)), StdDev(mm, nint(real(nn-1)/10.0))


!*************************************************
!	begin code
!*************************************************
istat = getcwd(DirName)

! evaluate statistics
means = 0.0
StdDev = 0.0
vv = SimStart
do ll = 1, SimNumb

	if ( vv.lt.10_4 ) then

		write(vsn, '(i1)') vv
	else

		write(vsn, '(i2)') vv
	end if
	SimName = trim(RootSimName) // trim(vsn)
	FileName = trim(DirName) // '\simulations\' // trim(SimName) // '\anals\' // trim(AnalFile)
	call csv_file_read( FileName, store, mm, nn, istat )
	
	! aggregate over deciles
	store2 = 0.0
	do kk = 1, 18
		
		do ii = 1, mm
		
			do jj = 1, 10
			
				store2(ii,kk) = store2(ii,kk) + store(ii,1+10*(kk-1)+jj) / 10.0
			end do
		end do
	end do
	do ii = 1, mm
		
		do jj = 1, 18
			
			means(ii,jj) = means(ii,jj) + store2(ii,jj) / real(SimNumb)
			StdDev(ii,jj) = StdDev(ii,jj) + (store2(ii,jj)**2.0) / real(SimNumb)
		end do
	end do
	vv = vv + 1
end do

! complete evaluations of standard deviations
do ii = 1, mm
		
	do jj = 1, 18
			
		StdDev(ii,jj) = (StdDev(ii,jj) - (means(ii,jj)**2.0))**0.5
	end do
end do

! write output
FileName = trim(OutDir) // '\' // trim(RootSimName) // 'AggMeans.csv'
call csv_file_write( FileName, means, mm, 18, 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // 'AggSDs.csv'
call csv_file_write( FileName, StdDev, mm, 18, 3 )

print *, 'completed calculation of summary statistics for replication'
print *, 'analysis following aggregation of decile stats'


END SUBROUTINE DecileAggregn


SUBROUTINE DecileAggregn2(OutDir, RootSimName, AnalFile, SimNumb, SimStart, StartAge, mm, nn, age_bands, nn_ab, decile_flag)

!*************************************************
!
!	subroutine evaluates age-group population 
!	averages by aggregating over decile statistics
!	then evaluates means and variances of of these
!	aggregates derived from replicated simulated 
!	statistics
!
!	subroutine similar to DecileAggregn, but 
!	can be used to evaluate averages over broad age 
!	groups, and for selected decile groups
!
!	OutDir:			directory to save output to
!	RootSimName:	root of replicated simulation name
!	AnalFile:		file name to analyse
!	SimNumb:		number of simulation replications
!	SimStart:		start number of replicated simulations
!	mm:				number of rows in analysis file
!	nn:				number of cols in analysis file
!	age_bands:		upper threshold for respective age band
!	nn_ab:			number of age bands to evaluate
!	decile_flag:	flag defining deciles included in analysis
!
!*************************************************

USE Comm_tools
USE ifport
USE dfwin

IMPLICIT none


character(2000), intent(in) :: OutDir
character(256), intent(in) :: RootSimName, AnalFile
integer(4), intent(in) :: SimNumb, SimStart, StartAge, mm, nn, nn_ab, age_bands(nn_ab), decile_flag(10)

! local variables
character(2000) :: DirName, FileName
character(256) :: SimName
character(2) :: vsn
integer(4) :: ii, jj, kk, ll, vv, hh, istat, aa0, aa1
real(8) :: no_dec
real(8) :: store(mm, nn), store2(nn_ab,nint(real(nn-1)/10)), means(nn_ab, nint(real(nn-1)/10)), StdDev(nn_ab, nint(real(nn-1)/10))


!*************************************************
!	begin code
!*************************************************
istat = getcwd(DirName)

! evaluate statistics
means = 0.0
StdDev = 0.0
vv = SimStart
no_dec = real( sum(decile_flag) )
do ll = 1, SimNumb

	if ( vv.lt.10_4 ) then

		write(vsn, '(i1)') vv
	else

		write(vsn, '(i2)') vv
	end if
	SimName = trim(RootSimName) // trim(vsn)
	FileName = trim(DirName) // '\simulations\' // trim(SimName) // '\anals\' // trim(AnalFile)
	call csv_file_read( FileName, store, mm, nn, istat )
	
	! aggregate over deciles
	store2 = 0.0
	do kk = 1, nint(real(nn-1)/10)
		
		do hh = 1, nn_ab

			if ( hh.eq.1 ) then
				
				aa0 = 1 
			else
				
				aa0 = aa1 + 1
			end if
			aa1 = age_bands(hh) - StartAge + 1
			do ii = aa0, aa1
		
				do jj = 1, 10
			
					if ( decile_flag(jj).eq.1 ) then
						
						store2(hh,kk) = store2(hh,kk) + store(ii,1+10*(kk-1)+jj) / no_dec / real(aa1 - aa0 + 1)
					end if
				end do
			end do
		end do
	end do
	do ii = 1, nn_ab
		
		do jj = 1, nint(real(nn-1)/10)
			
			means(ii,jj) = means(ii,jj) + store2(ii,jj) / real(SimNumb)
			StdDev(ii,jj) = StdDev(ii,jj) + (store2(ii,jj)**2.0) / real(SimNumb)
		end do
	end do
	vv = vv + 1
end do

! complete evaluations of standard deviations
do ii = 1, nn_ab
		
	do jj = 1, nint(real(nn-1)/10)
			
		StdDev(ii,jj) = (StdDev(ii,jj) - (means(ii,jj)**2.0))**0.5
	end do
end do

! write output
FileName = trim(OutDir) // '\' // trim(RootSimName) // 'Means.csv'
call csv_file_write( FileName, means, nn_ab, nint(real(nn-1)/10), 3 )

FileName = trim(OutDir) // '\' // trim(RootSimName) // 'SDs.csv'
call csv_file_write( FileName, StdDev, nn_ab, nint(real(nn-1)/10), 3 )


END SUBROUTINE DecileAggregn2


END MODULE internal_analyses