MODULE solve_grids


!***************************************************************
!
!	Grids and axes are created here, solved and then saved
!
!***************************************************************

IMPLICIT none


CONTAINS


SUBROUTINE solve_grid( aa_begin, aa_end, grid_outer_size_a, i_grids, aa_correct, test )

!***************************************************************
!
!	This SUBROUTINE defines grids, calls a SUBROUTINE to solve
!	utility maximising decisions, and saves the output
!
!***************************************************************

USE global_ParamStore
USE global_CommonAll
USE global_f1
USE other_tools
USE utility_maxn

! MPI DIRECTIVE
!USE mpi

IMPLICIT none


integer(4), intent(in) :: aa_begin, aa_end
integer(8), intent(in) :: grid_outer_size_a(5)
integer(4), intent(in) :: i_grids, aa_correct
integer(4) :: test

! local variables
integer(4) :: aa_max_grid, var_id, varst_dim, dv_varid, dv2_varid, file_id
integer(4) :: aa, ii, jj, i_correct(5), istat, istat1, var_i, numb_width
integer(4) :: disp_test0, disp_test1
real(8) :: n_points, var_r
character(15) :: fmat
character(50) :: banner1, banner2
character(256) :: grid_filename1, banner
character(2000) :: grid_filename
real(8), allocatable :: V_grid_a(:), dV_grid_a(:), dV2_grid_a(:), c_grid_a(:), emp1_grid_a(:), emp2_grid_a(:)
real(8), allocatable :: ri_grid_a(:), pc_grid_a(:), train_grid_a(:), prec_grid_a(:), isa_grid_a(:)


!************************************************************
!	begin code
!************************************************************
if ( grid_typ(6).lt.1.5 ) then
! use global grids matrices

	istat = 0
	allocate( V_grid_a_e(grid_outer_size_a(1)), c_grid_a_e(grid_outer_size_a(1)), STAT=istat1 )
	istat = istat1
	varst_dim = 1
	if ( grid_outer_size_a(2).gt.0 ) then

		allocate( emp1_grid_a_e(grid_outer_size_a(2)), emp2_grid_a_e(grid_outer_size_a(2)), STAT=istat1 )
		istat = istat + istat1
	end if
	if ( model_par(33).gt.0.5 ) then

		allocate( ri_grid_a_e(grid_outer_size_a(1)), STAT=istat1 )
		istat = istat + istat1
	end if
	if ( grid_outer_size_a(3).gt.0 ) then

		allocate( pc_grid_a_e(grid_outer_size_a(3)), STAT=istat1 )
		istat = istat + istat1
	end if
	if ( model_par(47).gt.0.5 ) then

		allocate( isa_grid_a_e(grid_outer_size_a(1)), STAT=istat1 )
		istat = istat + istat1
	end if
	if ( grid_outer_size_a(4).gt.0 ) then

		allocate( train_grid_a_e(grid_outer_size_a(4)), STAT=istat1 )
		istat = istat + istat1
	end if
	if ( grid_outer_size_a(5).gt.0 ) then

		allocate( prec_grid_a_e(grid_outer_size_a(5)), STAT=istat1 )
		istat = istat + istat1
	end if
	if ( model_par(49).gt.0.5 ) then
	! Quasi-hyperbolic discounting considered

		allocate( dV_grid_a_e(grid_outer_size_a(1)), dV2_grid_a_e(grid_outer_size_a(1)), STAT=istat1 )
		varst_dim = varst_dim + 2
		istat = istat + istat1
	end if
else
! use local grids matrices

	istat = 0
	allocate( V_grid_a(grid_outer_size_a(1)), c_grid_a(grid_outer_size_a(1)), STAT=istat1 )
	istat = istat1
	varst_dim = 1
	if ( grid_outer_size_a(2).gt.0 ) then

		allocate( emp1_grid_a(grid_outer_size_a(2)), emp2_grid_a(grid_outer_size_a(2)), STAT=istat1 )
		istat = istat + istat1
	end if
	if ( model_par(33).gt.0.5 ) then

		allocate( ri_grid_a(grid_outer_size_a(1)), STAT=istat1 )
		istat = istat + istat1
	end if
	if ( grid_outer_size_a(3).gt.0 ) then

		allocate( pc_grid_a(grid_outer_size_a(3)), STAT=istat1 )
		istat = istat + istat1
	end if
	if ( model_par(47).gt.0.5 ) then

		allocate( isa_grid_a(grid_outer_size_a(1)), STAT=istat1 )
		istat = istat + istat1
	end if
	if ( grid_outer_size_a(4).gt.0 ) then

		allocate( train_grid_a(grid_outer_size_a(4)), STAT=istat1 )
		istat = istat + istat1
	end if
	if ( grid_outer_size_a(5).gt.0 ) then

		allocate( prec_grid_a(grid_outer_size_a(5)), STAT=istat1 )
		istat = istat + istat1
	end if
	if ( model_par(49).gt.0.5 ) then
	! Quasi-hyperbolic discounting considered

		allocate( dV_grid_a(grid_outer_size_a(1)), dV2_grid_a(grid_outer_size_a(1)), STAT=istat1 )
		varst_dim = varst_dim + 2
		istat = istat + istat1
	end if
end if
if ( istat.ne.0 ) then

	print *, 'Allocation error in solve_grids 1'
	print *, 'Consider increasing the Virtual Memory available to the system,'
	print *, 'dividing the grids into more components,'
	print *, 'or using a 64 bit version of the model'
	pause 'press enter to exit'
	stop
end if


!******************************************************************
!	begin cycling through ages
!******************************************************************
if ( (aa_begin.lt.SimLifeSpan).or.((model_par(16).gt.0.5).and.(grid_typ(6).lt.1.5)) ) then
! need to load in data to initialise variables for period tp1

	if ( i_grids.eq.nint(grid_typ(6)) ) then
		aa_max_grid = SimLifeSpan
	else
		aa_max_grid = aa_correct * i_grids
	end if
	if ( (((test.eq.0).and.(model_par(16).lt.0.5)).or.((test.eq.1).and.((aa_begin+1).gt.aa_max_grid))).and.(nint(grid_typ(6)).lt.nint(model_par(7)-model_par(6)+1.0)) ) then
	! separate grid exists to link with period i_grids + 1

		i_correct = grid_outer_size_a
		ndim_a = 0
		do jj = 1, max_ndim

			if ( n_grid(aa_begin+1,1,jj).gt.0.1 ) then

				ndim_a = ndim_a + 1
			end if
		end do
		if ( allocated( axis_a ) ) deallocate( axis_a )
		allocate( axis_a(4,ndim_a) )
		call axis_defn(ndim_a, axis_a, aa_begin+1_4)
		msize_a = grid_size_all(aa_begin+1)

		allocate( vars_a(msize_a, varst_dim) )
		
		if ( mpi_task.eq.0 ) then
			
			if ( i_grids+1.lt.10 ) then

				write(grid_filename1, '(i1)') i_grids+1
			elseif ( i_grids+1.lt.100 ) then

				write(grid_filename1, '(i2)') i_grids+1
			else

				write(grid_filename1, '(i3)') i_grids+1
			end if
			istat = 0
			if ( grid_typ(19).lt.0.5 ) then
			! read formatted output

				grid_filename = trim(grid_dir) // '\' // trim(grid_filename1) // 'a.dat' //char(0)
				open(newunit=file_id, FILE=grid_filename, action = 'read', iostat = istat)
				if ( istat.ne.0 ) then

					print *, 'problem opening file: '
					print *, trim(grid_filename)
					pause 'press enter to exit'
					stop
				end if
				read(file_id,*,IOSTAT=istat1) vars_a(:,1)
				istat = istat + istat1
				if ( model_par(49).gt.0.5 ) then
				! Quasi-hyperbolic discounting considered

					read(file_id,*,IOSTAT=istat1) vars_a(:,2)
					istat = istat + istat1
					read(file_id,*,IOSTAT=istat1) vars_a(:,3)
					istat = istat + istat1
				end if
			else
			! read unformatted output

				grid_filename = trim(grid_dir) // '\' // trim(grid_filename1) // 'a.uft' //char(0)
				open ( newunit = file_id, file = grid_filename, form = 'unformatted', action='read', access='stream', iostat = istat )
				if ( istat.ne.0 ) then

					print *, 'problem opening file: '
					print *, trim(grid_filename)
					pause 'press enter to exit'
					stop
				end if

				read(file_id, IOSTAT=istat1) vars_a(:,1)
				istat = istat + istat1
				if ( model_par(49).lt.0.5 ) then

					read(file_id, IOSTAT=istat1) vars_a(:,2)
					istat = istat + istat1
					read(file_id, IOSTAT=istat1) vars_a(:,3)
					istat = istat + istat1
				end if
			end if
			close(file_id)
			if ( istat.gt.0 ) then

				print *, 'Error in reading stored value function link - check code'
				pause 'press enter to exit'
				stop
			elseif ( istat.lt.0 ) then

				print *, 'Insufficient data to fill value function expectations'
				pause 'press enter to exit'
				stop
			end if
			if ( mpi_numtasks.gt.1 ) then
				
				do jj = 1, varst_dim
					
					! MPI DIRECTIVE
					!call MPI_BCAST( vars_a(:,jj), msize_a, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end do
			end if
		else
			
			do jj = 1, varst_dim
				
				! MPI DIRECTIVE
				!call MPI_BCAST( vars_a(:,jj), msize_a, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
			end do
		end if
	elseif ( nint(grid_typ(6)).eq.nint(model_par(7)-model_par(6)+1.0) ) then
	! number of grid slices equal to number of discrete ages - load in from *.dat file

		i_correct = grid_outer_size_a
		ndim_a = 0
		do jj = 1, max_ndim

			if ( n_grid(aa_begin+1,1,jj).gt.0.1 ) then

				ndim_a = ndim_a + 1
			end if
		end do
		if ( allocated( axis_a ) ) deallocate( axis_a )
		allocate( axis_a(4,ndim_a) )
		call axis_defn(ndim_a, axis_a, aa_begin+1_4)
		msize_a = grid_size_all(aa_begin+1)

		allocate( vars_a(msize_a, varst_dim) )
		
		if ( mpi_task.eq.0 ) then
			
			if ( i_grids+1.lt.10 ) then

				write(grid_filename1, '(i1)') i_grids+1
			elseif ( i_grids+1.lt.100 ) then

				write(grid_filename1, '(i2)') i_grids+1
			else

				write(grid_filename1, '(i3)') i_grids+1
			end if
			if ( grid_typ(19).lt.0.5 ) then
			! read formatted output

				grid_filename = trim(grid_dir) // '\' // trim(grid_filename1) // '.dat' //char(0)
				open(newunit=file_id, FILE=grid_filename, action = 'read', iostat = istat)
				if ( istat.ne.0 ) then

					print *, 'problem opening file for reading grids auxilliary data (1)'
					pause 'press enter to exit'
					stop
				end if
				istat = 0
				read(file_id,*,IOSTAT=istat1) vars_a(:,1)
				istat = istat + istat1
				if ( varst_dim.gt.1 ) then
				! need to load in additional linking data for utility maximisation

					read(file_id,*,IOSTAT=istat1) vars_a(:,2)
					istat = istat + istat1
					if ( (emp_par(1).gt.0.5).and.((aa_begin+nint(model_par(6))).le.nint(model_par(5))) ) then
					! employment saved in t+1

						read(file_id,*,IOSTAT=istat1) vars_a(:,2)
						istat = istat + istat1
						read(file_id,*,IOSTAT=istat1) vars_a(:,2)
						istat = istat + istat1
					end if
					if ( model_par(33).gt.0.5 ) then
					! investment decision saved in t+1

						read(file_id,*,IOSTAT=istat1) vars_a(:,2)
						istat = istat + istat1
					end if
					if ( (pen_par(11).gt.0.5).and.( (aa_begin+nint(model_par(6))).lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then
					! pension contribution decision saved in t+1

						read(file_id,*,IOSTAT=istat1) vars_a(:,2)
						istat = istat + istat1
					end if
					if ( model_par(47).gt.0.5 ) then
					! ISA decisions saved in t+1

						read(file_id,*,IOSTAT=istat1) vars_a(:,2)
						istat = istat + istat1
					end if
					if ( (model_par(64).gt.0.5).and.((aa_begin+nint(model_par(6))).le.nint(edu_par(22))) ) then
					! training decision saved in t+1

						read(file_id,*,IOSTAT=istat1) vars_a(:,2)
						istat = istat + istat1
					end if
					if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.((aa_begin+nint(model_par(6))).ge.min_page).and.((aa_begin+nint(model_par(6))).lt.max_page) ) then
					! pension receipt decision saved in t+1

						read(file_id,*,IOSTAT=istat1) vars_a(:,2)
						istat = istat + istat1
					end if
					read(file_id,*,IOSTAT=istat1) vars_a(:,2)
					istat = istat + istat1
					read(file_id,*,IOSTAT=istat1) vars_a(:,3)
					istat = istat + istat1
				end if
			else
			! read unformatted output

				grid_filename = trim(grid_dir) // '\' // trim(grid_filename1) // '.uft' //char(0)
				open ( newunit = file_id, file = grid_filename, form = 'unformatted', action='read', access='stream', iostat = istat )
				if ( istat.ne.0 ) then

					print *, 'problem opening file for reading grids auxilliary data (2)'
					pause 'press enter to exit'
					stop
				end if
				read(file_id, IOSTAT=istat1) vars_a(:,1)
				istat = istat + istat1
				if ( varst_dim.gt.1 ) then
				! need to load in additional linking data for utility maximisation

					jj = 2  ! for consumption
					if ( (emp_par(1).gt.0.5).and.((aa_begin+nint(model_par(6))).le.nint(model_par(5))) ) then
					! employment saved in t+1

						jj = jj + 2
					end if
					if ( model_par(33).gt.0.5 ) then
					! investment decision saved in t+1

						jj = jj + 1
					end if
					if ( (pen_par(11).gt.0.5).and.( (aa_begin+nint(model_par(6))).lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then
					! pension contribution decision saved in t+1

						jj = jj + 1
					end if
					if ( model_par(47).gt.0.5 ) then
					! ISA decisions saved in t+1

						jj = jj + 1
					end if
					if ( (model_par(64).gt.0.5).and.((aa_begin+nint(model_par(6))).le.nint(edu_par(22))) ) then
					! training decision saved in t+1

						jj = jj + 1
					end if
					if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.((aa_begin+nint(model_par(6))).ge.min_page).and.((aa_begin+nint(model_par(6))).lt.max_page) ) then
					! pension receipt decision saved in t+1

						jj = jj + 1
					end if
					do ii = 1, jj

						read(file_id, IOSTAT=istat1) vars_a(:,2)
						istat = istat + istat1
					end do
					read(file_id, IOSTAT=istat1) vars_a(:,3)
					istat = istat + istat1
				end if
			end if
			close(file_id)
			if ( istat.gt.0 ) then

				print *, 'Error in reading stored grid results - check code'
				pause 'press enter to exit'
				stop
			elseif ( istat.lt.0 ) then

				print *, 'Insufficient data to complete grid results'
				pause 'press enter to exit'
				stop
			end if
			if ( mpi_numtasks.gt.1 ) then
				
				do jj = 1, varst_dim
					
					! MPI DIRECTIVE
					!call MPI_BCAST( vars_a(:,jj), msize_a, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end do
			end if
		else
			
			do jj = 1, varst_dim
				
				! MPI DIRECTIVE
				!call MPI_BCAST( vars_a(:,jj), msize_a, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
			end do
		end if
	else
	! first run of test internal to period i_grids

		i_correct = grid_outer_size_a
		if ( aa_begin.lt.aa_max_grid ) then
			
			do ii = aa_max_grid,aa_begin+1,-1

				i_correct(1) = i_correct(1) - grid_size_all(ii)
				if ( (emp_par(1).gt.0.5).and.((ii+nint(model_par(6))-1).le.nint(model_par(5))) ) then
				! allow for employment

					i_correct(2) = i_correct(2) - grid_size_all(ii)
				end if
				if ( (pen_par(11).gt.0.5).and.( (ii+nint(model_par(6))-1).lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then
				! allow for pension contributions

					i_correct(3) = i_correct(3) - grid_size_all(ii)
				end if
				if ( (model_par(64).gt.0.5).and.((ii+nint(model_par(6))-1).le.nint(edu_par(22))) ) then
				! allow for training decision

					i_correct(4) = i_correct(4) - grid_size_all(ii)
				end if
				if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.((ii+nint(model_par(6))-1).ge.min_page).and.((ii+nint(model_par(6))-1).lt.max_page) ) then
				! allow for pension receipt decision

					i_correct(5) = i_correct(5) - grid_size_all(ii)
				end if
			end do
			
			ndim_a = 0
			do jj = 1, max_ndim

				if ( n_grid(aa_begin+1,1,jj).gt.0.1 ) then

					ndim_a = ndim_a + 1
				end if
			end do
			if ( allocated( axis_a ) ) deallocate( axis_a )
			allocate( axis_a(4,ndim_a) )
			call axis_defn(ndim_a, axis_a, aa_begin+1_4)
			msize_a = grid_size_all(aa_begin+1)
		else
			
			msize_a = 1
			allocate( vars_a(msize_a, varst_dim) )
			vars_a = 0.0

			ndim_a = 1
			allocate( axis_a(4, ndim_a) )
			axis_a = 0.0
			i_correct = grid_outer_size_a
		end if
		if ( mpi_task.eq.0 ) then
			
			if ( i_grids.lt.10 ) then

				write(grid_filename1, '(i1)') i_grids
			elseif ( i_grids.lt.100 ) then

				write(grid_filename1, '(i2)') i_grids
			else

				write(grid_filename1, '(i3)') i_grids
			end if
			if ( grid_typ(19).lt.0.5 ) then
			! read formatted data

				grid_filename = trim(grid_dir) // '\' // trim(grid_filename1) // '.dat' //char(0)
				open(newunit = file_id, FILE=grid_filename, action = 'read', iostat = istat)
				if ( istat.ne.0 ) then

					print *, 'problem opening file to read grids data for global variables (1)'
					pause 'press enter to exit'
					stop
				end if
				if ( allocated( V_grid_a_e ) ) then

					read(file_id,*,IOSTAT=istat1) V_grid_a_e
					istat = istat + istat1
					read(file_id,*,IOSTAT=istat1) c_grid_a_e
					istat = istat + istat1
					if ( allocated( emp1_grid_a_e ) ) then

						read(file_id,*,IOSTAT=istat1) emp1_grid_a_e
						istat = istat + istat1
						read(file_id,*,IOSTAT=istat1) emp2_grid_a_e
						istat = istat + istat1
					end if
					if ( allocated( ri_grid_a_e ) ) then

						read(file_id,*,IOSTAT=istat1) ri_grid_a_e
						istat = istat + istat1
					end if
					if ( allocated( pc_grid_a_e ) ) then

						read(file_id,*,IOSTAT=istat1) pc_grid_a_e
						istat = istat + istat1
					end if
					if ( allocated( isa_grid_a_e ) ) then

						read(file_id,*,IOSTAT=istat1) isa_grid_a_e
						istat = istat + istat1
					end if
					if ( allocated( train_grid_a_e ) ) then

						read(file_id,*,IOSTAT=istat1) train_grid_a_e
						istat = istat + istat1
					end if
					if ( allocated( prec_grid_a_e ) ) then

						read(file_id,*,IOSTAT=istat1) prec_grid_a_e
						istat = istat + istat1
					end if
					if ( model_par(49).gt.0.5 ) then
					! Quasi-hyperbolic discounting considered

						read(file_id,*,IOSTAT=istat1) dV_grid_a_e
						istat = istat + istat1
						read(file_id,*,IOSTAT=istat1) dV2_grid_a_e
						istat = istat + istat1
					end if
				else

					read(file_id,*,IOSTAT=istat1) V_grid_a
					istat = istat + istat1
					read(file_id,*,IOSTAT=istat1) c_grid_a
					istat = istat + istat1
					if ( allocated( emp1_grid_a ) ) then

						read(file_id,*,IOSTAT=istat1) emp1_grid_a
						istat = istat + istat1
						read(file_id,*,IOSTAT=istat1) emp2_grid_a
						istat = istat + istat1
					end if
					if ( allocated( ri_grid_a ) ) then

						read(file_id,*,IOSTAT=istat1) ri_grid_a
						istat = istat + istat1
					end if
					if ( allocated( pc_grid_a ) ) then

						read(file_id,*,IOSTAT=istat1) pc_grid_a
						istat = istat + istat1
					end if
					if ( allocated( isa_grid_a ) ) then

						read(file_id,*,IOSTAT=istat1) isa_grid_a
						istat = istat + istat1
					end if
					if ( allocated( train_grid_a ) ) then

						read(file_id,*,IOSTAT=istat1) train_grid_a
						istat = istat + istat1
					end if
					if ( allocated( prec_grid_a ) ) then

						read(file_id,*,IOSTAT=istat1) prec_grid_a
						istat = istat + istat1
					end if
					if ( model_par(49).gt.0.5 ) then
					! Quasi-hyperbolic discounting considered

						read(file_id,*,IOSTAT=istat1) dV_grid_a
						istat = istat + istat1
						read(file_id,*,IOSTAT=istat1) dV2_grid_a
						istat = istat + istat1
					end if
				end if
			else
			! read unformatted data

				grid_filename = trim(grid_dir) // '\' // trim(grid_filename1) // '.uft' //char(0)
				open ( newunit = file_id, file = grid_filename, form = 'unformatted', action='read', access='stream', iostat = istat )
				if ( istat.ne.0 ) then

					print *, 'problem opening file to read grids data for global variables (2)'
					pause 'press enter to exit'
					stop
				end if
				if ( allocated( V_grid_a_e ) ) then

					read(file_id, iostat=istat1) V_grid_a_e
					istat = istat + istat1
					read(file_id, iostat=istat1) c_grid_a_e
					istat = istat + istat1
					if ( allocated( emp1_grid_a_e ) ) then

						read(file_id, iostat=istat1) emp1_grid_a_e
						istat = istat + istat1
						read(file_id, iostat=istat1) emp2_grid_a_e
						istat = istat + istat1
					end if
					if ( allocated( ri_grid_a_e ) ) then

						read(file_id, iostat=istat1) ri_grid_a_e
						istat = istat + istat1
					end if
					if ( allocated( pc_grid_a_e ) ) then

						read(file_id, iostat=istat1) pc_grid_a_e
						istat = istat + istat1
					end if
					if ( allocated( isa_grid_a_e ) ) then

						read(file_id, iostat=istat1) isa_grid_a_e
						istat = istat + istat1
					end if
					if ( allocated( train_grid_a_e ) ) then

						read(file_id, iostat=istat1) train_grid_a_e
						istat = istat + istat1
					end if
					if ( allocated( prec_grid_a_e ) ) then

						read(file_id, iostat=istat1) prec_grid_a_e
						istat = istat + istat1
					end if
					if ( model_par(49).gt.0.5 ) then
					! Quasi-hyperbolic discounting considered

						read(file_id, iostat=istat1) dV_grid_a_e
						istat = istat + istat1
						read(file_id, iostat=istat1) dV2_grid_a_e
						istat = istat + istat1
					end if
				else

					read(file_id, iostat=istat1) V_grid_a
					istat = istat + istat1
					read(file_id, iostat=istat1) c_grid_a
					istat = istat + istat1
					if ( allocated( emp1_grid_a ) ) then

						read(file_id, iostat=istat1) emp1_grid_a
						istat = istat + istat1
						read(file_id, iostat=istat1) emp2_grid_a
						istat = istat + istat1
					end if
					if ( allocated( ri_grid_a ) ) then

						read(file_id, iostat=istat1) ri_grid_a
						istat = istat + istat1
					end if
					if ( allocated( pc_grid_a ) ) then

						read(file_id, iostat=istat1) pc_grid_a
						istat = istat + istat1
					end if
					if ( allocated( isa_grid_a ) ) then

						read(file_id, iostat=istat1) isa_grid_a
						istat = istat + istat1
					end if
					if ( allocated( train_grid_a ) ) then

						read(file_id, iostat=istat1) train_grid_a
						istat = istat + istat1
					end if
					if ( allocated( prec_grid_a ) ) then

						read(file_id, iostat=istat1) prec_grid_a
						istat = istat + istat1
					end if
					if ( model_par(49).gt.0.5 ) then
					! Quasi-hyperbolic discounting considered

						read(file_id, iostat=istat1) dV_grid_a
						istat = istat + istat1
						read(file_id, iostat=istat1) dV2_grid_a
						istat = istat + istat1
					end if
				end if
			end if
			close(file_id)
			if ( istat.gt.0 ) then

				print *, 'Error in reading stored grid results 2 - check code'
				pause 'press enter to exit'
				stop
			elseif ( istat.lt.0 ) then

				print *, 'Insufficient data to complete grid results 2'
				pause 'press enter to exit'
				stop
			end if
			if ( mpi_numtasks.gt.1 ) then
			! MPI enabled - need to broadcast loaded values to all compute nodes
				
				if ( allocated( V_grid_a_e ) ) then
					
					! MPI DIRECTIVE - SEARCH AND REPLACE ! !call MPI_BCAST
					!call MPI_BCAST( V_grid_a_e, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					!call MPI_BCAST( c_grid_a_e, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					if ( grid_outer_size_a(2).gt.0 ) then

						!call MPI_BCAST( emp1_grid_a_e, grid_outer_size_a(2), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
						!call MPI_BCAST( emp2_grid_a_e, grid_outer_size_a(2), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
					if ( model_par(33).gt.0.5 ) then

						!call MPI_BCAST( ri_grid_a_e, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
					if ( grid_outer_size_a(3).gt.0 ) then

						!call MPI_BCAST( pc_grid_a_e, grid_outer_size_a(3), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
					if ( model_par(47).gt.0.5 ) then

						!call MPI_BCAST( isa_grid_a_e, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
					if ( grid_outer_size_a(4).gt.0 ) then

						!call MPI_BCAST( train_grid_a_e, grid_outer_size_a(4), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
					if ( grid_outer_size_a(5).gt.0 ) then

						!call MPI_BCAST( prec_grid_a_e, grid_outer_size_a(5), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
					if ( model_par(49).gt.0.5 ) then
					! Quasi-hyperbolic discounting considered

						!call MPI_BCAST( dV_grid_a_e, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
						!call MPI_BCAST( dV2_grid_a_e, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
				else
					
					!call MPI_BCAST( V_grid_a, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					!call MPI_BCAST( c_grid_a, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					if ( grid_outer_size_a(2).gt.0 ) then

						!call MPI_BCAST( emp1_grid_a, grid_outer_size_a(2), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
						!call MPI_BCAST( emp2_grid_a, grid_outer_size_a(2), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
					if ( model_par(33).gt.0.5 ) then

						!call MPI_BCAST( ri_grid_a, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
					if ( grid_outer_size_a(3).gt.0 ) then

						!call MPI_BCAST( pc_grid_a, grid_outer_size_a(3), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
					if ( model_par(47).gt.0.5 ) then

						!call MPI_BCAST( isa_grid_a, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
					if ( grid_outer_size_a(4).gt.0 ) then

						!call MPI_BCAST( train_grid_a, grid_outer_size_a(4), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
					if ( grid_outer_size_a(5).gt.0 ) then

						!call MPI_BCAST( prec_grid_a, grid_outer_size_a(5), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
					if ( model_par(49).gt.0.5 ) then
					! Quasi-hyperbolic discounting considered

						!call MPI_BCAST( dV_grid_a, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
						!call MPI_BCAST( dV2_grid_a, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					end if
				end if
			end if
		else
		! MPI compute node - need to receive loaded data from master node
			
			if ( allocated( V_grid_a_e ) ) then
					
				! MPI DIRECTIVE - SEARCH AND REPLACE ! !call MPI_BCAST
				!call MPI_BCAST( V_grid_a_e, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				!call MPI_BCAST( c_grid_a_e, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				if ( grid_outer_size_a(2).gt.0 ) then

					!call MPI_BCAST( emp1_grid_a_e, grid_outer_size_a(2), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					!call MPI_BCAST( emp2_grid_a_e, grid_outer_size_a(2), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
				if ( model_par(33).gt.0.5 ) then

					!call MPI_BCAST( ri_grid_a_e, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
				if ( grid_outer_size_a(3).gt.0 ) then

					!call MPI_BCAST( pc_grid_a_e, grid_outer_size_a(3), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
				if ( model_par(47).gt.0.5 ) then

					!call MPI_BCAST( isa_grid_a_e, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
				if ( grid_outer_size_a(4).gt.0 ) then

					!call MPI_BCAST( train_grid_a_e, grid_outer_size_a(4), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
				if ( grid_outer_size_a(5).gt.0 ) then

					!call MPI_BCAST( prec_grid_a_e, grid_outer_size_a(5), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
				if ( model_par(49).gt.0.5 ) then
				! Quasi-hyperbolic discounting considered

					!call MPI_BCAST( dV_grid_a_e, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					!call MPI_BCAST( dV2_grid_a_e, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
			else
					
				!call MPI_BCAST( V_grid_a, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				!call MPI_BCAST( c_grid_a, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				if ( grid_outer_size_a(2).gt.0 ) then

					!call MPI_BCAST( emp1_grid_a, grid_outer_size_a(2), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					!call MPI_BCAST( emp2_grid_a, grid_outer_size_a(2), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
				if ( model_par(33).gt.0.5 ) then

					!call MPI_BCAST( ri_grid_a, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
				if ( grid_outer_size_a(3).gt.0 ) then

					!call MPI_BCAST( pc_grid_a, grid_outer_size_a(3), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
				if ( model_par(47).gt.0.5 ) then

					!call MPI_BCAST( isa_grid_a, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
				if ( grid_outer_size_a(4).gt.0 ) then

					!call MPI_BCAST( train_grid_a, grid_outer_size_a(4), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
				if ( grid_outer_size_a(5).gt.0 ) then

					!call MPI_BCAST( prec_grid_a, grid_outer_size_a(5), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
				if ( model_par(49).gt.0.5 ) then
				! Quasi-hyperbolic discounting considered

					!call MPI_BCAST( dV_grid_a, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
					!call MPI_BCAST( dV2_grid_a, grid_outer_size_a(1), MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, mpi_ierr )
				end if
			end if
		end if

		if ( aa_begin.lt.aa_max_grid ) then
			
			allocate( vars_a(msize_a, varst_dim) )
			if ( grid_typ(6).lt.1.5 ) then

				vars_a(:,1) = V_grid_a_e(i_correct(1)+1:i_correct(1)+msize_a)
				if ( varst_dim.gt.1 ) then

					vars_a(:,2) = dV_grid_a_e(i_correct(1)+1:i_correct(1)+msize_a)
					vars_a(:,3) = dV2_grid_a_e(i_correct(1)+1:i_correct(1)+msize_a)
				end if
			else

				vars_a(:,1) = V_grid_a(i_correct(1)+1:i_correct(1)+msize_a)
				if ( varst_dim.gt.1 ) then

					vars_a(:,2) = dV_grid_a(i_correct(1)+1:i_correct(1)+msize_a)
					vars_a(:,3) = dV2_grid_a(i_correct(1)+1:i_correct(1)+msize_a)
				end if
			end if
		end if
	end if
else
! final period - no expectation vector required for solution

	msize_a = 1
	allocate( vars_a(msize_a, varst_dim) )
	vars_a = 0.0

	ndim_a = 1
	allocate( axis_a(4, ndim_a) )
	axis_a = 0.0
	i_correct = grid_outer_size_a
end if
if ( (aa_begin.eq.SimLifeSpan).and.(report_flag.eq.1).and.(mpi_task.eq.0) ) then

	CALL system_clock( COUNT=timet,COUNT_RATE=time_rate,COUNT_MAX=timer_max) ! end timer
	time1 = real(timet)/real(time_rate)
	if ( time1.lt.time0 ) then
	! passed timer_max and started again

		time0 = time1 + real(timer_max)/real(time_rate) - time0
	else

		time0 = time1-time0
	end if
	total_time = total_time + time0/60.0
	if ( time0.lt.999.0 ) then

		write(banner1, "(1PG11.4E2)") time0
		banner = 'Preliminaries completed in' // trim(banner1) // ' seconds.'
	else

		time0 = time0 / 60.0
		write(banner1, "(1PG11.4E2)") time0
		banner = 'Preliminaries completed in' // trim(banner1) // ' minutes.'
	end if
	print*, trim(banner)
	time0 = time1
end if
if ( aa_begin.ge.aa_end ) then

	do aa = aa_begin, aa_end, -1

		! set up new variables for t
		if ( aa.lt.nint(model_par(5)-model_par(6)+1) ) then
			
			n_points = macro_par(4)
		else
			
			n_points = 1
		end if
		call time_given_agei_cohort(aa, 1.0)
		call pension_exist()

		tax_par4(62) = s_hsg_adj(aa)
		tax_par4(63) = c_hsg_adj(aa)
		ndim_am1 = 0
		do jj = 1, max_ndim

			if ( n_grid(aa,1,jj).gt.0.1 ) then

				ndim_am1 = ndim_am1 + 1
			end if
		end do
		if ( allocated( axis_am1 ) ) deallocate( axis_am1 )
		allocate( axis_am1(4,ndim_am1) )
		call axis_defn(ndim_am1, axis_am1, aa)
		msize_am1 = grid_size_all(aa)
		i_correct(1) = i_correct(1) - msize_am1
		var_id = 2
		if ( (emp_par(1).gt.0.5).and.((aa+nint(model_par(6))-1).le.nint(model_par(5))) ) then
		! allow for employment

			var_id = var_id + 2
			i_correct(2) = i_correct(2) - msize_am1
		end if
		if ( model_par(33).gt.0.5 ) then
		! investment decision saved in t+1

			var_id = var_id + 1
		end if
		if ( (pen_par(11).gt.0.5).and.( (aa+nint(model_par(6))-1).lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then
		! allow for pension contributions

			var_id = var_id + 1
			i_correct(3) = i_correct(3) - msize_am1
		end if
		if ( model_par(47).gt.0.5 ) then
		! ISA decision saved in t+1

			var_id = var_id + 1
		end if
		if ( (model_par(64).gt.0.5).and.((aa+nint(model_par(6))-1).le.nint(edu_par(22))) ) then
		! allow for training decision

			var_id = var_id + 1
			i_correct(4) = i_correct(4) - msize_am1
		end if
		if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.((aa+nint(model_par(6))-1).ge.min_page).and.((aa+nint(model_par(6))-1).lt.max_page) ) then
		! allow for pension receipt decision

			var_id = var_id + 1
			i_correct(5) = i_correct(5) - msize_am1
		end if
		if ( varst_dim.gt.1 ) then

			var_id = var_id + 2
		end if

		if ( allocated(vars_am1) ) deallocate ( vars_am1 )
		allocate ( vars_am1(msize_am1,var_id) )
		vars_am1 = 0.0_8
		if ( model_par(16).gt.0.5 ) then

			if ( grid_typ(6).lt.1.5 ) then
				
				vars_am1(:, 2) = c_grid_a_e(i_correct(1)+1:i_correct(1) + msize_am1)
				var_id = 2
				if ( (emp_par(1).gt.0.5).and.((aa+nint(model_par(6))-1).le.nint(model_par(5))) ) then
				! allow for employment

					var_id = var_id + 1
					var_id = var_id + 1
				end if
				if ( model_par(33).gt.0.5 ) then
				! investment decision saved in t+1

					var_id = var_id + 1
				end if
				if ( (pen_par(11).gt.0.5).and.( (aa+nint(model_par(6))-1).lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then
				! allow for pension contributions

					var_id = var_id + 1
					vars_am1(:, var_id) = pc_grid_a_e(i_correct(3)+1:i_correct(3) + msize_am1)
				end if
			else
				
				if ( i_grids.lt.10 ) then

					write(grid_filename1, '(i1)') i_grids
				elseif ( i_grids.lt.100 ) then

					write(grid_filename1, '(i2)') i_grids
				else

					write(grid_filename1, '(i3)') i_grids
				end if
				if ( grid_typ(19).lt.0.5 ) then
				! read formatted output

					grid_filename = trim(grid_dir) // '\' // trim(grid_filename1) // '.dat' //char(0)
					open(newunit=file_id, FILE=grid_filename, action = 'read', iostat = istat)
					if ( istat.ne.0 ) then

						print *, 'problem opening file for reading grids auxilliary data (1)'
						pause 'press enter to exit'
						stop
					end if
					istat = 0
					read(file_id,*,IOSTAT=istat1) vars_am1(:,2)
					istat = istat + istat1
					read(file_id,*,IOSTAT=istat1) vars_am1(:,2)
					istat = istat + istat1
					var_id = 2
					if ( (emp_par(1).gt.0.5).and.((aa+nint(model_par(6))-1).le.nint(model_par(5))) ) then
					! allow for employment

						var_id = var_id + 1
						read(file_id,*,IOSTAT=istat1) vars_am1(:,var_id)
						istat = istat + istat1
						var_id = var_id + 1
						read(file_id,*,IOSTAT=istat1) vars_am1(:,var_id)
						istat = istat + istat1
					end if
					if ( model_par(33).gt.0.5 ) then
					! investment decision saved in t+1

						var_id = var_id + 1
						read(file_id,*,IOSTAT=istat1) vars_am1(:,var_id)
						istat = istat + istat1
					end if
					if ( (pen_par(11).gt.0.5).and.( (aa+nint(model_par(6))-1).lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then
					! allow for pension contributions

						var_id = var_id + 1
						read(file_id,*,IOSTAT=istat1) vars_am1(:,var_id)
						istat = istat + istat1
					end if
				else
				! read unformatted output

					grid_filename = trim(grid_dir) // '\' // trim(grid_filename1) // '.uft' //char(0)
					open ( newunit = file_id, file = grid_filename, form = 'unformatted', action='read', access='stream', iostat = istat )
					if ( istat.ne.0 ) then

						print *, 'problem opening file for reading grids auxilliary data (2)'
						pause 'press enter to exit'
						stop
					end if
					read(file_id, IOSTAT=istat1) vars_am1(:,2)
					istat = istat + istat1
					read(file_id, IOSTAT=istat1) vars_am1(:,2)
					istat = istat + istat1
					var_id = 2
					if ( (emp_par(1).gt.0.5).and.((aa+nint(model_par(6))-1).le.nint(model_par(5))) ) then
					! allow for employment

						var_id = var_id + 1
						read(file_id, IOSTAT=istat1) vars_am1(:,var_id)
						istat = istat + istat1
						var_id = var_id + 1
						read(file_id, IOSTAT=istat1) vars_am1(:,var_id)
						istat = istat + istat1
					end if
					if ( model_par(33).gt.0.5 ) then
					! investment decision saved in t+1

						var_id = var_id + 1
						read(file_id, IOSTAT=istat1) vars_am1(:,var_id)
						istat = istat + istat1
					end if
					if ( (pen_par(11).gt.0.5).and.( (aa+nint(model_par(6))-1).lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then
					! allow for pension contributions

						var_id = var_id + 1
						read(file_id, IOSTAT=istat1) vars_am1(:,var_id)
						istat = istat + istat1
					end if
				end if
				close(file_id)
				if ( istat.gt.0 ) then

					print *, 'Error in reading stored grid results - check code'
					pause 'press enter to exit'
					stop
				elseif ( istat.lt.0 ) then

					print *, 'Insufficient data to complete grid results'
					pause 'press enter to exit'
					stop
				end if
			end if
		end if

		
		!**********************************************************
		!
		!	STEP IN TO SOLVE FOR UTILITY MAXIMISING DECISIONS
		!
		!**********************************************************
		if ( mpi_pnts_per_task(mpi_task,aa).gt.0 ) then
			
			call tm_grid_cal( mpi_outer_start(mpi_task,aa), mpi_outer_end(mpi_task,aa), disp_test0, disp_test1 )
			if ( (disp_test0.ne.mpi_disp_per_task(mpi_task,aa)).or.(disp_test1.ne.(mpi_disp_per_task(mpi_task,aa)+mpi_pnts_per_task(mpi_task,aa))) ) then
			
				print *, 'Error in co-ordinating MPI nodes'
				pause 'press enter to exit'
				! MPI DIRECTIVE
				!call MPI_ABORT(MPI_COMM_WORLD, mpi_rc, mpi_ierr)
				stop
			end if
		end if
		if ( mpi_numtasks.gt.1 ) then
			
			do ii = 1, var_id
				
				! MPI DIRECTIVE
				!call MPI_ALLGATHERV( MPI_IN_PLACE, mpi_pnts_per_task(mpi_task,aa), MPI_DOUBLE_PRECISION, vars_am1(:,ii), mpi_pnts_per_task(:,aa), mpi_disp_per_task(:,aa), MPI_DOUBLE_PRECISION, MPI_COMM_WORLD, mpi_ierr )
			end do
		end if
		if ( grid_typ(6).lt.1.5 ) then
		! use global grid store

			var_id = 1
			V_grid_a_e(i_correct(1)+1:i_correct(1) + msize_am1) = vars_am1(:, var_id)
			var_id = var_id + 1
			c_grid_a_e(i_correct(1)+1:i_correct(1) + msize_am1) = vars_am1(:, var_id)
			if ( (emp_par(1).gt.0.5).and.((aa+nint(model_par(6))-1).le.nint(model_par(5))) ) then
			! allow for employment

				var_id = var_id + 1
				emp1_grid_a_e(i_correct(2)+1:i_correct(2) + msize_am1) = vars_am1(:, var_id)
				var_id = var_id + 1
				emp2_grid_a_e(i_correct(2)+1:i_correct(2) + msize_am1) = vars_am1(:, var_id)
			end if
			if ( model_par(33).gt.0.5 ) then
			! investment decision saved in t+1

				var_id = var_id + 1
				ri_grid_a_e(i_correct(1)+1:i_correct(1) + msize_am1) = vars_am1(:, var_id)
			end if
			if ( (pen_par(11).gt.0.5).and.( (aa+nint(model_par(6))-1).lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then
			! allow for pension contributions

				var_id = var_id + 1
				pc_grid_a_e(i_correct(3)+1:i_correct(3) + msize_am1) = vars_am1(:, var_id)
			end if
			if ( model_par(47).gt.0.5 ) then
			! ISA decision saved in t+1

				var_id = var_id + 1
				isa_grid_a_e(i_correct(1)+1:i_correct(1) + msize_am1) = vars_am1(:, var_id)
			end if
			if ( (model_par(64).gt.0.5).and.((aa+nint(model_par(6))-1).le.nint(edu_par(22))) ) then
			! allow for training decision

				var_id = var_id + 1
				train_grid_a_e(i_correct(4)+1:i_correct(4) + msize_am1) = vars_am1(:, var_id)
			end if
			if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.((aa+nint(model_par(6))-1).ge.min_page).and.((aa+nint(model_par(6))-1).lt.max_page) ) then
			! allow for pension receipt decision

				var_id = var_id + 1
				prec_grid_a_e(i_correct(5)+1:i_correct(5) + msize_am1) = vars_am1(:, var_id)
			end if
			if ( varst_dim.gt.1 ) then

				var_id = var_id + 1
				dV_grid_a_e(i_correct(1)+1:i_correct(1) + msize_am1) = vars_am1(:, var_id)
				dv_varid = var_id
				var_id = var_id + 1
				dV2_grid_a_e(i_correct(1)+1:i_correct(1) + msize_am1) = vars_am1(:, var_id)
				dv2_varid = var_id
			end if
		else
		! use local grid store

			var_id = 1
			V_grid_a(i_correct(1)+1:i_correct(1) + msize_am1) = vars_am1(:, var_id)
			var_id = var_id + 1
			c_grid_a(i_correct(1)+1:i_correct(1) + msize_am1) = vars_am1(:, var_id)
			if ( (emp_par(1).gt.0.5).and.((aa+nint(model_par(6))-1).le.nint(model_par(5))) ) then
			! allow for employment

				var_id = var_id + 1
				emp1_grid_a(i_correct(2)+1:i_correct(2) + msize_am1) = vars_am1(:, var_id)
				var_id = var_id + 1
				emp2_grid_a(i_correct(2)+1:i_correct(2) + msize_am1) = vars_am1(:, var_id)
			end if
			if ( model_par(33).gt.0.5 ) then
			! investment decision saved in t+1

				var_id = var_id + 1
				ri_grid_a(i_correct(1)+1:i_correct(1) + msize_am1) = vars_am1(:, var_id)
			end if
			if ( (pen_par(11).gt.0.5).and.( (aa+nint(model_par(6))-1).lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then
			! allow for pension contributions

				var_id = var_id + 1
				pc_grid_a(i_correct(3)+1:i_correct(3) + msize_am1) = vars_am1(:, var_id)
			end if
			if ( model_par(47).gt.0.5 ) then
			! ISA decision saved in t+1

				var_id = var_id + 1
				isa_grid_a(i_correct(1)+1:i_correct(1) + msize_am1) = vars_am1(:, var_id)
			end if
			if ( (model_par(64).gt.0.5).and.((aa+nint(model_par(6))-1).le.nint(edu_par(22))) ) then
			! allow for training decision

				var_id = var_id + 1
				train_grid_a(i_correct(4)+1:i_correct(4) + msize_am1) = vars_am1(:, var_id)
			end if
			if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.((aa+nint(model_par(6))-1).ge.min_page).and.((aa+nint(model_par(6))-1).lt.max_page) ) then
			! allow for pension receipt decision

				var_id = var_id + 1
				prec_grid_a(i_correct(5)+1:i_correct(5) + msize_am1) = vars_am1(:, var_id)
			end if
			if ( varst_dim.gt.1 ) then

				var_id = var_id + 1
				dV_grid_a(i_correct(1)+1:i_correct(1) + msize_am1) = vars_am1(:, var_id)
				dv_varid = var_id
				var_id = var_id + 1
				dV2_grid_a(i_correct(1)+1:i_correct(1) + msize_am1) = vars_am1(:, var_id)
				dv2_varid = var_id
			end if
		end if

		! refresh t variables for next run
		if ( ndim_a.eq.ndim_am1 ) then

			axis_a = axis_am1
		else

			ndim_a = ndim_am1
			deallocate( axis_a )
			allocate( axis_a(4, ndim_a) )
			axis_a = axis_am1
		end if
		msize_a = msize_am1
		if ( allocated(vars_a) ) deallocate( vars_a )
		allocate( vars_a(msize_a, varst_dim) )
		vars_a(:,1) = vars_am1(:,1)
		if ( model_par(49).gt.0.5 ) then
		! Quasi-hyperbolic discounting considered

			vars_a(:,2) = vars_am1(:,dv_varid)
			vars_a(:,3) = vars_am1(:,dv2_varid)
		end if
		if ( (report_flag.eq.1).and.(mpi_task.eq.0) ) then

			CALL system_clock( COUNT=timet,COUNT_RATE=time_rate,COUNT_MAX=timer_max) ! end timer
			time1 = real(timet)/real(time_rate)
			if ( time1.lt.time0 ) then
			! passed timer_max and started again

				time0 = time1 + real(timer_max)/real(time_rate) - time0
			else

				time0 = time1-time0
			end if
			total_time = total_time + time0/60.0
			write(banner2, "(I3)") (nint(model_par(6) + aa - 1))
			if ( time0.lt.999.0 ) then

				write(banner1, "(1PG11.4E2)") time0
				banner = 'completed calculating solution for age ' // trim(banner2) // ' in' // trim(banner1) // ' seconds.'
			else

				time0 = time0 / 60.0
				write(banner1, "(1PG11.4E2)") time0
				banner = 'completed calculating solution for age ' // trim(banner2) // ' in' // trim(banner1) // ' minutes.'
			end if
			print*, trim(banner)
			time0 = time1
		end if
	end do
end if
test = 0

if ( (grid_save.eq.1).and.(mpi_task.eq.0) ) then
! save grid output to files

	if ( i_grids.lt.10 ) then

		write(grid_filename1, '(i1)') i_grids
	elseif ( i_grids.lt.100 ) then

		write(grid_filename1, '(i2)') i_grids
	else

		write(grid_filename1, '(i3)') i_grids
	end if
	if ( grid_typ(19).lt.0.5 ) then
	! write formatted data

		grid_filename = trim(grid_dir) // '\' // trim(grid_filename1) // '.dat' //char(0)
		open(newunit = file_id, FILE=grid_filename, status = 'replace', action = 'write', iostat = istat )
		if ( istat.ne.0 ) then

			print *, 'problem opening file to save grids data (1)'
			pause 'press enter to exit'
			stop
		end if
		if ( grid_typ(6).lt.1.5 ) then
		! global storage vectors


			!********************************************
			!	write V grid
			!********************************************
			do ii = 1, grid_outer_size_a(1)

				var_r = V_grid_a_e(ii)
				var_i = nint(var_r)
				if ( var_r.eq.real(var_i) ) then
				! write as integer

					call i4_width( numb_width, var_i )
					write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
					write (file_id, fmat) var_i
				else

					fmat = '(1PG24.15E3)'
					write (file_id, fmat) var_r
				end if
			end do


			!********************************************
			!	write c grid
			!********************************************
			do ii = 1, grid_outer_size_a(1)

				var_r = c_grid_a_e(ii)
				var_i = nint(var_r)
				if ( var_r.eq.real(var_i) ) then
				! write as integer

					call i4_width( numb_width, var_i )
					write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
					write (file_id, fmat) var_i
				else

					fmat = '(g13.5)'
					write (file_id, fmat) var_r
				end if
			end do


			!********************************************
			!	write employment grid
			!********************************************
			if ( allocated( emp1_grid_a_e ) ) then

				do ii = 1, grid_outer_size_a(2)

					var_r = emp1_grid_a_e(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
					! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write (file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write (file_id, fmat) var_r
					end if
				end do

				do ii = 1, grid_outer_size_a(2)

					var_r = emp2_grid_a_e(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do
			end if


			!********************************************
			!	write investment grid
			!********************************************
			if ( allocated( ri_grid_a_e ) ) then

				do ii = 1, grid_outer_size_a(1)

					var_r = ri_grid_a_e(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do
			end if


			!********************************************
			!	write pension contribution grid
			!********************************************
			if ( allocated( pc_grid_a_e ) ) then

				do ii = 1, grid_outer_size_a(3)

					var_r = pc_grid_a_e(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do
			end if


			!********************************************
			!	write ISA grid
			!********************************************
			if ( allocated( isa_grid_a_e ) ) then

				do ii = 1, grid_outer_size_a(1)

					var_r = isa_grid_a_e(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do
			end if


			!********************************************
			!	write training grid
			!********************************************
			if ( allocated( train_grid_a_e ) ) then

				do ii = 1, grid_outer_size_a(4)

					var_r = train_grid_a_e(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do
			end if


			!********************************************
			!	write pension receipt grid
			!********************************************
			if ( allocated( prec_grid_a_e ) ) then

				do ii = 1, grid_outer_size_a(5)

					var_r = prec_grid_a_e(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do
			end if


			!********************************************
			!	write V components for QH discounting grid
			!********************************************
			if ( allocated( dV_grid_a_e ) ) then

				do ii = 1, grid_outer_size_a(1)

					var_r = dV_grid_a_e(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(1PG24.15E3)'
						write(file_id, fmat) var_r
					end if
				end do

				do ii = 1, grid_outer_size_a(1)

					var_r = dV2_grid_a_e(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(1PG24.15E3)'
						write(file_id, fmat) var_r
					end if
				end do
			end if
		else
		! local storage vectors


			!********************************************
			!	write V grid
			!********************************************
			do ii = 1, grid_outer_size_a(1)

				var_r = V_grid_a(ii)
				var_i = nint(var_r)
				if ( var_r.eq.real(var_i) ) then
					! write as integer

					call i4_width( numb_width, var_i )
					write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
					write(file_id, fmat) var_i
				else

					fmat = '(1PG24.15E3)'
					write(file_id, fmat) var_r
				end if
			end do


			!********************************************
			!	write c grid
			!********************************************
			do ii = 1, grid_outer_size_a(1)

				var_r = c_grid_a(ii)
				var_i = nint(var_r)
				if ( var_r.eq.real(var_i) ) then
					! write as integer

					call i4_width( numb_width, var_i )
					write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
					write(file_id, fmat) var_i
				else

					fmat = '(g13.5)'
					write(file_id, fmat) var_r
				end if
			end do


			!********************************************
			!	write employment grid
			!********************************************
			if ( allocated( emp1_grid_a ) ) then

				do ii = 1, grid_outer_size_a(2)

					var_r = emp1_grid_a(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do

				do ii = 1, grid_outer_size_a(2)

					var_r = emp2_grid_a(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do
			end if


			!********************************************
			!	write investment grid
			!********************************************
			if ( allocated( ri_grid_a ) ) then

				do ii = 1, grid_outer_size_a(1)

					var_r = ri_grid_a(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do
			end if


			!********************************************
			!	write pension contribution grid
			!********************************************
			if ( allocated( pc_grid_a ) ) then

				do ii = 1, grid_outer_size_a(3)

					var_r = pc_grid_a(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do
			end if


			!********************************************
			!	write ISA grid
			!********************************************
			if ( allocated( isa_grid_a ) ) then

				do ii = 1, grid_outer_size_a(1)

					var_r = isa_grid_a(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do
			end if


			!********************************************
			!	write training grid
			!********************************************
			if ( allocated( train_grid_a ) ) then

				do ii = 1, grid_outer_size_a(4)

					var_r = train_grid_a(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do
			end if


			!********************************************
			!	write pension receipt grid
			!********************************************
			if ( allocated( prec_grid_a ) ) then

				do ii = 1, grid_outer_size_a(5)

					var_r = prec_grid_a(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(g13.5)'
						write(file_id, fmat) var_r
					end if
				end do
			end if


			!********************************************
			!	write V components for QH discounting grid
			!********************************************
			if ( allocated( dV_grid_a ) ) then

				do ii = 1, grid_outer_size_a(1)

					var_r = dV_grid_a(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(1PG24.15E3)'
						write(file_id, fmat) var_r
					end if
				end do

				do ii = 1, grid_outer_size_a(1)

					var_r = dV2_grid_a(ii)
					var_i = nint(var_r)
					if ( var_r.eq.real(var_i) ) then
						! write as integer

						call i4_width( numb_width, var_i )
						write ( fmat, '(a,i2,a)' ) '(i', numb_width, ')'
						write(file_id, fmat) var_i
					else

						fmat = '(1PG24.15E3)'
						write(file_id, fmat) var_r
					end if
				end do
			end if
		end if
	else
	! write unformatted output
	
		grid_filename = trim(grid_dir) // '\' // trim(grid_filename1) // '.uft' //char(0)
		open ( newunit = file_id, file = grid_filename, form = 'unformatted', action='write', access='stream', iostat = istat )
		if ( istat.ne.0 ) then

			print *, 'problem opening file to save grids data (2)'
			pause 'press enter to exit'
			stop
		end if
		if ( grid_typ(6).lt.1.5 ) then
		! global storage vectors


			!********************************************
			!	write V grid
			!********************************************
			write( file_id, iostat=istat1 ) V_grid_a_e
			istat = istat + istat1


			!********************************************
			!	write c grid
			!********************************************
			write( file_id, iostat=istat1 ) c_grid_a_e
			istat = istat + istat1


			!********************************************
			!	write employment grid
			!********************************************
			if ( allocated( emp1_grid_a_e ) ) then

				write( file_id, iostat=istat1 ) emp1_grid_a_e
				istat = istat + istat1
				write( file_id, iostat=istat1 ) emp2_grid_a_e
				istat = istat + istat1
			end if


			!********************************************
			!	write investment grid
			!********************************************
			if ( allocated( ri_grid_a_e ) ) then

				write( file_id, iostat=istat1 ) ri_grid_a_e
				istat = istat + istat1
			end if


			!********************************************
			!	write pension contribution grid
			!********************************************
			if ( allocated( pc_grid_a_e ) ) then

				write( file_id, iostat=istat1 ) pc_grid_a_e
				istat = istat + istat1
			end if


			!********************************************
			!	write ISA grid
			!********************************************
			if ( allocated( isa_grid_a_e ) ) then

				write( file_id, iostat=istat1 ) isa_grid_a_e
				istat = istat + istat1
			end if


			!********************************************
			!	write training grid
			!********************************************
			if ( allocated( train_grid_a_e ) ) then

				write( file_id, iostat=istat1 ) train_grid_a_e
				istat = istat + istat1
			end if


			!********************************************
			!	write pension receipt grid
			!********************************************
			if ( allocated( prec_grid_a_e ) ) then

				write( file_id, iostat=istat1 ) prec_grid_a_e
				istat = istat + istat1
			end if


			!********************************************
			!	write V components for QH discounting grid
			!********************************************
			if ( allocated( dV_grid_a_e ) ) then

				write( file_id, iostat=istat1 ) dV_grid_a_e
				istat = istat + istat1
				write( file_id, iostat=istat1 ) dV2_grid_a_e
				istat = istat + istat1
			end if
		else
		! local storage vectors


			!********************************************
			!	write V grid
			!********************************************
			write( file_id, iostat=istat1 ) V_grid_a
			istat = istat + istat1


			!********************************************
			!	write c grid
			!********************************************
			write( file_id, iostat=istat1 ) c_grid_a
			istat = istat + istat1


			!********************************************
			!	write employment grid
			!********************************************
			if ( allocated( emp1_grid_a ) ) then

				write( file_id, iostat=istat1 ) emp1_grid_a
				istat = istat + istat1
				write( file_id, iostat=istat1 ) emp2_grid_a
				istat = istat + istat1
			end if


			!********************************************
			!	write investment grid
			!********************************************
			if ( allocated( ri_grid_a ) ) then

				write( file_id, iostat=istat1 ) ri_grid_a
				istat = istat + istat1
			end if


			!********************************************
			!	write pension contribution grid
			!********************************************
			if ( allocated( pc_grid_a ) ) then

				write( file_id, iostat=istat1 ) pc_grid_a
				istat = istat + istat1
			end if


			!********************************************
			!	write ISA grid
			!********************************************
			if ( allocated( isa_grid_a ) ) then

				write( file_id, iostat=istat1 ) isa_grid_a
				istat = istat + istat1
			end if


			!********************************************
			!	write training grid
			!********************************************
			if ( allocated( train_grid_a ) ) then

				write( file_id, iostat=istat1 ) train_grid_a
				istat = istat + istat1
			end if


			!********************************************
			!	write pension receipt grid
			!********************************************
			if ( allocated( prec_grid_a ) ) then

				write( file_id, iostat=istat1 ) prec_grid_a
				istat = istat + istat1
			end if


			!********************************************
			!	write V components for QH discounting grid
			!********************************************
			if ( allocated( dV_grid_a ) ) then

				write( file_id, iostat=istat1 ) dV_grid_a
				istat = istat + istat1
				write( file_id, iostat=istat1 ) dV2_grid_a
				istat = istat + istat1
			end if
		end if
		if ( istat.ne.0 ) then

			print *, 'problem writing grid output'
			pause 'press enter to exit'
			stop
		end if
	end if
	close(file_id)

	if ( (nint(grid_typ(6)).lt.nint(model_par(7)-model_par(6)+1.0)).and.(grid_typ(6).gt.1.5) ) then

		if (grid_typ(19).lt.0.5 ) then
		! write formatted output

			grid_filename = trim(grid_dir) // '\' // trim(grid_filename1) // 'a.dat' //char(0)
			open(newunit = file_id, FILE=grid_filename, status = 'replace', action = 'write', iostat = istat)
			if ( istat.ne.0 ) then

				print *, 'problem opening auxilliary file for saving grids data'
				pause 'press enter to exit'
				stop
			end if
			write(file_id, "(1PG24.15E3)") V_grid_a(1:msize_am1)
			if ( model_par(49).gt.0.5 ) then
			! Quasi-hyperbolic discounting considered

				write(file_id, "(1PG24.15E3)") dV_grid_a(1:msize_am1)
				write(file_id, "(1PG24.15E3)") dV2_grid_a(1:msize_am1)
			end if
		else
		! write unformatted output

			grid_filename = trim(grid_dir) // '\' // trim(grid_filename1) // 'a.uft' //char(0)
			open ( newunit = file_id, file = grid_filename, form = 'unformatted', action='write', access='stream', iostat = istat )
			if ( istat.ne.0 ) then

				print *, 'problem opening file to save grids data (3)'
				pause 'press enter to exit'
				stop
			end if
			write( file_id, iostat=istat1 ) V_grid_a(1:msize_am1)
			istat = istat + istat1
			if ( model_par(49).gt.0.5 ) then
			! Quasi-hyperbolic discounting considered

				write(file_id, iostat=istat1 ) dV_grid_a(1:msize_am1)
				istat = istat + istat1
				write(file_id, iostat=istat1 ) dV2_grid_a(1:msize_am1)
				istat = istat + istat1
			end if
			if ( istat.ne.0 ) then

				print *, 'error in writing temporary grid file'
				pause 'press enter to exit'
				stop
			end if
		end if
		close(file_id)
	end if
end if

! clean up variables
if ( allocated( V_grid_a ) ) deallocate( V_grid_a, c_grid_a )
if ( allocated( emp1_grid_a ) ) deallocate( emp1_grid_a, emp2_grid_a )
if ( allocated( dV_grid_a ) ) deallocate( dV_grid_a, dV2_grid_a )
if ( allocated( ri_grid_a ) ) deallocate( ri_grid_a )
if ( allocated( pc_grid_a ) ) deallocate( pc_grid_a )
if ( allocated( isa_grid_a ) ) deallocate( isa_grid_a )
if ( allocated( train_grid_a ) ) deallocate( train_grid_a )
if ( allocated( prec_grid_a ) ) deallocate( prec_grid_a )

deallocate( vars_a, vars_am1, axis_a, axis_am1 )
if ( allocated(dV_grid_a_e) ) deallocate( dV_grid_a_e )
if ( allocated(dV2_grid_a_e) ) deallocate( dV2_grid_a_e )


END SUBROUTINE solve_grid


END MODULE solve_grids