MODULE utility_maxn


!*************************************************************
!
!	module to manage solution of grids
!
!*************************************************************

IMPLICIT none


CONTAINS


SUBROUTINE tm_grid_cal(outer_start_h, outer_end_h, disp_test0, disp_test1)

!***********************************************************************
!
!   Grid Calculation Routine
!
!***********************************************************************

USE Exp_Prelim
USE global
USE global_ParamStore
USE global_CommonAll
USE global_CommonCohort
USE global_f1
USE solve_continuous
USE other_tools

IMPLICIT none

integer(4), intent(in) :: outer_start_h, outer_end_h
integer(4), intent(out) :: disp_test0, disp_test1

! local variables
integer(4) :: ndim_inner, ndim_outer, ndim_outer_total, n_iuser, n_ruser
integer(4) :: ii, jj, i_outer

! looping variables
integer(4) :: m_inner, m_inner_nw, m_inner_w, m_outer, wo_no, start_here, emp_states, nwo1_states, nwo2_states, counter, istat, istat1
integer(4) :: dims_inner(ndim_am1), dims_outer(ndim_am1), xind_outer
real(8) :: step_inner(ndim_am1), step_outer(ndim_am1), x_outer(ndim_am1), x_init_outer(ndim_am1)
real(8) :: emp_cost

! variables inside OpenMP loop
integer(4) :: i_inner, i_inner_nw, i_inner_w, training_options, prec_options, kk, ll, mm, nwos1, nwos2, flag_skip, indx, indxnwo, indxnwo1, indxnwo2, pen_regime, var_id
integer(4) :: pens_takeup, emp1, emp2
real(8) :: xx(100), training_state1, training_state, labour
real(8) :: prwage1, prwage2, leisure, training_gain
real(8) :: ri_am1(150), rio_am1, pc_am1(150), pco_am1, pcemp_am1, isa_am1(150), isao_am1
real(8) :: u, val_1, dv_1, dv2_1
real(8) :: uo, valo_1, dvo_1, dv2o_1
real(8) :: u_nwo , val_1_nwo , dv_1_nwo , dv2_1_nwo , emp1_nwo , emp2_nwo , ri_nwo , pc_nwo , isa_nwo,  train_nwo , prec_nwo
real(8) :: u_nwo1, val_1_nwo1, dv_1_nwo1, dv2_1_nwo1, emp1_nwo1, emp2_nwo1, ri_nwo1, pc_nwo1, isa_nwo1, train_nwo1, prec_nwo1
real(8) :: u_nwo2, val_1_nwo2, dv_1_nwo2, dv2_1_nwo2, emp1_nwo2, emp2_nwo2, ri_nwo2, pc_nwo2, isa_nwo2,	train_nwo2, prec_nwo2
real(8) :: u_temp, val_1_temp, dv_1_temp, dv2_1_temp, emp1_temp, emp2_temp, ri_temp, pc_temp, isa_temp, train_temp, prec_temp
real(8) :: u_temp_s, u_nwo_s, u_nwo1_s, u_nwo2_s
real(8) :: pc_temp_s, pc_nwo_s, pc_nwo1_s, pc_nwo2_s


!***********************************************************************
!	begin code
!***********************************************************************
disp_test0 = 0
disp_test1 = 0
default_interp = nint(sln_par(1))
ii = 2


!***********************************************************************
!	Initialise statistics for integrations over continuous state variables
!***********************************************************************

call cont_prelim()

!***********************************************************************
!	Initialise statistics for expectations over discrete state variables
!***********************************************************************

call disc_prelim()
training_state1 = 0.0

!***********************************************************************
!	Initialise remaining variables for optimisation loops
!***********************************************************************

call axes_prelim()
ndim_outer_total = demog_axes_am1 + edu_axis_am1 + prec_axis_am1 + pcr_axis_am1 + dppart_axis_am1 + &
	stud_axis_am1 + ret_axis_am1 + semp_axis_am1 + pub_axis_am1 + hlth_axis_am1 + dla_axis_am1 + &
	care_axis_am1 + nkd_axes_am1 + offer_axis2_am1 + offer_axis_am1 + bc_axis_am1
if ( (model_par(64).gt.0.5).and.(current_age.gt.nint(edu_par(22))) ) then
	! training is a discrete axis (if it exists) and the state is certain

	ndim_outer_total = ndim_outer_total + train_axis_am1
end if
ndim_inner = ndim_am1 - ndim_outer_total
ndim_outer = ndim_outer_total - offer_axis_am1 - offer_axis2_am1

! nr_a is the number of points on grids with continuous states
if ( ndim_a.gt.1 ) then

	nr_a = nint( product( axis_a(3,1:ndim_a-disc_axes_a) ) )
else

	nr_a = axis_a(3,1)
end if
do ii = 1, ndim_inner

	dims_inner(ii)=nint(axis_am1(3,ii))
end do
m_inner_w = dims_inner(1)
if ( ndim_inner.gt.1 ) then

	m_inner_nw = nint( product( real(dims_inner(2:ndim_inner)) ) )
else

	m_inner_nw = 1
end if
m_inner = m_inner_w * m_inner_nw
do ii = 1, ndim_inner

	if ( dims_inner(ii).gt.1_4 ) then

		step_inner(ii) = ( axis_am1(2,ii) - axis_am1(1,ii) ) / real( dims_inner(ii) - 1_4 )
	else

		step_inner(ii) = 0.0
	end if
end do

wo_no = m_inner
if ( ndim_outer.gt.0 ) then

	flag_ad = 0
	jj = 1
	kk = ndim_inner + 1
	if ( (model_par(64).gt.0.5).and.(current_age.le.nint(model_par(5))).and.(current_age.gt.nint(edu_par(22))) ) then
	! training included as a discrete axis - outer grids

		! allocate training
		flag_ad = 1_4
		dims_outer(jj) = nint(axis_am1(3,kk))
		wo_no = wo_no * dims_outer(jj)
		step_outer(jj) = (axis_am1(2,kk) - axis_am1(1,kk)) / max( 1.0, real(dims_outer(jj)-1.0) )
		x_init_outer(jj) = axis_am1(1,kk)
		jj = jj + 1
		kk = kk + 1
	end if
	if ( spop_par(12).gt.1.5 ) then
	! allow for birth cohort state

		dims_outer(jj) = nint( axis_am1(3,kk) )
		wo_no = wo_no * dims_outer(jj)
		step_outer(jj) = ( axis_am1(2,kk) - axis_am1(1,kk) ) / max( 1.0, real(dims_outer(jj)-1.0) )
		x_init_outer(jj) = axis_am1(1,kk)
		jj = jj + 1
		kk = kk + 1
	end if
	if ( offer_axis_am1.eq.1 ) then

		kk = kk + 1
		if ( offer_axis2_am1.eq.1 ) then

			kk = kk + 1
		end if
	end if
	ii = kk
	do kk = ii, ndim_am1

		dims_outer(jj) = nint( axis_am1(3,kk) )
		step_outer(jj) = ( axis_am1(2,kk) - axis_am1(1,kk) ) / max( 1.0, real(dims_outer(jj)-1.0) )
		x_init_outer(jj) = axis_am1(1,kk)
		jj = jj + 1
	end do
	m_outer = nint( product( real(dims_outer(1:ndim_outer)) ) )
else

	m_outer = 1_4
end if


!************************************************************
!	begin looping structure
!************************************************************
do i_outer = outer_start_h, outer_end_h

	indx = 1
	xind_outer = mod( (i_outer-1), dims_outer(indx) )
	x_outer(indx) = x_init_outer(indx) + step_outer(indx) * real(xind_outer)
	if ( ndim_outer.gt.1 ) then

		do indx = 2, ndim_outer

			counter = 1 + int( (real(i_outer)-0.01) / real(product(dims_outer(1:indx-1))) )
			xind_outer = mod( (counter-1), dims_outer(indx) )
			x_outer(indx) = x_init_outer(indx) + step_outer(indx) * real(xind_outer)
		end do
	end if

	! allocate discrete states (values at time t-1)
	call disc_states(training_state1, start_here, x_outer(1:ndim_outer), ndim_outer)
	if ( i_outer.eq.outer_start_h ) then
		
		disp_test0 = start_here
	elseif ( i_outer.eq.outer_end_h ) then
		
		disp_test1 = start_here + m_inner_w * m_inner_nw
		if ( offer_axis_am1.gt.0_4 ) then

			if ( offer_axis2_am1.eq.0_4 ) then
			! no separate wage axes for each spouse
				
				disp_test1 = disp_test1 + wo_no
			else
				
				disp_test1 = disp_test1 + wo_no * 3
			end if
		end if
	end if
	if ( spop_par(12).gt.1.5 ) then
	! birth cohorts explicit

		call pension_exist()
	end if
	
	if ( current_age.lt.nint(model_par(7)) ) then
	! take expectaions over discrete states (for time t)

		call disc_exp(training_state1, n_iuser, n_ruser)
	else

		nexp_d1 = nexp_d0
		if ( current_age.le.nint(model_par(5)) ) then
		! working lifetime

			max_prob = nexp_d1 * max_probc(1)
		else
		! mandatory retirement

			max_prob = nexp_d1 * max_probc(2)
		end if
		n_iuser = 300 + nexp_d1
		n_ruser = 1500 + max_prob*(8+ndim_a)

		allocate( V_str(nr_a,nexp_d1), STAT= istat1 )
		istat = istat1
		if ( model_par(49).gt.0.5 ) then
		! Quasi-hyperbolic discounting considered

			allocate( dV_str(nr_a,nexp_d1), dV2_str(nr_a,nexp_d1), STAT= istat1 )
			istat = istat + istat1
		end if
		if (istat.ne.0 ) then

			print *, 'Allocation error in dV2_str'
			pause 'press enter to exit'
			stop
		end if
	end if
	if ( (model_par(18).lt.0.5).or.(na_g.lt.1.5) ) then
	! model as single

		emp_states = lab_alt
	else
	! model as couple

		emp_states = lab_alt * lab_alt
	end if
	if ( (emp_par(3).gt.0.5).and.(lpr_par(1).gt.0.5) ) then
	! separate treatment of non-employment and low wage offer
	! this is necessary here because the logit equation for generating
	! transitions between labour states allows for an auto-regressive
	! form in the low-wage offer

		nwo1_states = 2
		if ( (emp_par(4).gt.0.5).and.(na_g.gt.1.5) ) then

			nwo2_states = 2
		else

			nwo2_states = 1
		end if
	else

		nwo1_states = 1
		nwo2_states = 1
	end if
	if ( estw2_par(1).lt.0.5 ) then
	! emp_cost defines ratio of time spent in employment - alternative used for wage equation 2

		if ( current_age.lt.spa_bc ) then

			if ( nint(na_g*model_par(18)).eq.2 ) then
				emp_cost = dyn_parc(8)
			else
				emp_cost = dyn_pars(8)
			end if
		else

			if ( nint(na_g*model_par(18)).eq.2 ) then
				emp_cost = dyn_parc(15)
			else
				emp_cost = dyn_pars(9)
			end if
		end if
	else

		emp_cost = 0.0
	end if
	if ( (tax_par4(44).gt.0.1).and. &	! pension receipt may be a decision variable
	( (current_age.ge.page1_bc).and.(current_age.lt.pagemax_bc) ).and. &	! pension decision may occur at current age
	(prec_state.eq.0).and.(emp_par(12).lt.0.5) ) then ! haven't received pension already

		prec_options = 2_4
	else

		prec_options = 1_4
	end if

	!********************************************************************
	!
	!	Start paralellised loop using OpenMP directives
	!
	!	Paralellisation of all dimensions other than liquid wealth
	!	this makes possible some serially structured code
	!
	!	All scalar variables that are modified within the parallel do loop are scoped
	!	as private. All arrays that have been initialised before the parallel do loop
	!	are scoped as firstprivate. This means that they are private AND the values they
	!	contain before the parallel do loop are copied to each thread's instance of the
	!	array. Any array that is to be modified within the parallel do loop, is scoped
	!	as private and passed as an argument to the function modifying it.
	!	The exception is the array "xx" - we have been forced to use a critical region
	!	to get this to work.
	!
	!********************************************************************

	!$OMP PARALLEL DO DEFAULT(SHARED) SCHEDULE(DYNAMIC) &
	!$OMP PRIVATE(i_inner_w, i_inner_nw, i_inner, training_options, ll, kk, mm, nwos1, nwos2, flag_skip, indx, indxnwo1, indxnwo2, indxnwo, pens_takeup, pen_regime, var_id) &
	!$OMP PRIVATE(xx, training_state, labour) &
	!$OMP PRIVATE(prwage1, prwage2, leisure, training_gain) &
	!$OMP PRIVATE(emp1, emp2) &
	!$OMP PRIVATE(ri_am1, rio_am1, pc_am1, pco_am1, pcemp_am1, isa_am1, isao_am1) &
	!$OMP PRIVATE(u, val_1, dv_1, dv2_1) &
	!$OMP PRIVATE(uo, valo_1, dvo_1, dv2o_1) &
	!$OMP PRIVATE(u_nwo , val_1_nwo , dv_1_nwo , dv2_1_nwo , emp1_nwo , emp2_nwo , ri_nwo , pc_nwo , isa_nwo , train_nwo , prec_nwo) &
	!$OMP PRIVATE(u_nwo1, val_1_nwo1, dv_1_nwo1, dv2_1_nwo1, emp1_nwo1, emp2_nwo1, ri_nwo1, pc_nwo1, isa_nwo1, train_nwo1, prec_nwo1) &
	!$OMP PRIVATE(u_nwo2, val_1_nwo2, dv_1_nwo2, dv2_1_nwo2, emp1_nwo2, emp2_nwo2, ri_nwo2, pc_nwo2, isa_nwo2, train_nwo2, prec_nwo2) &
	!$OMP PRIVATE(u_temp, val_1_temp, dv_1_temp, dv2_1_temp, emp1_temp, emp2_temp, ri_temp, pc_temp, isa_temp, train_temp, prec_temp) &
	!$OMP PRIVATE(u_temp_s, u_nwo_s, u_nwo1_s, u_nwo2_s, pc_temp_s, pc_nwo_s, pc_nwo1_s, pc_nwo2_s)

	do i_inner_nw = 1, m_inner_nw	! parallelised loop

	! initialise ri_am1 and pc_am1 once for each new thread
	! thereafter, initialisation is based upon preceding optimised values of ri and pc
	ri_am1 = 0.01_8
	if ( semp_state.eq.0 ) then

		pc_am1 = pen_par(27) + 0.05*(pen_par(28)-pen_par(27))
	else

		pc_am1 = semp_par(24) + 0.05*(pen_par(28)-semp_par(24))
	end if
	pco_am1 = 0.0_8
	isa_am1 = 0.0

	do i_inner_w = 1, m_inner_w		! non-parallised loop over liquid wealth

		i_inner = i_inner_w + m_inner_w * (i_inner_nw - 1)
		call inner_state(xx(1:ndim_inner), training_state, training_state1, step_inner(1:ndim_inner), dims_inner(1:ndim_inner), ndim_inner, axis_am1, ndim_am1, i_inner_w, i_inner_nw, flag_skip)
		if ( (prec_state.eq.1).and.(current_age.le.page1_bc) ) then

			flag_skip = 1
		end if
		if ( model_par(45).gt.0.5 ) then

			if ( na_g.lt.1.5 ) then

				if ( health2_state.gt.1 ) then

					flag_skip = 1
				end if
				if ( (care_state.eq.1).and.(health1_state.gt.nint(hlth_par(25))) ) then

					flag_skip = 1
				end if
			else

				if ( (care_state.eq.1).and.(min(health1_state,health2_state).gt.nint(hlth_par(25))) ) then

					flag_skip = 1
				end if
			end if
			if ( hlth_par(6).gt.0.5 ) then

				do kk = 1, brth_ages

					if ( (nk_allg(kk).lt.0.5).and.(nkd_all(kk).gt.0.5) ) then

						flag_skip = 1
					end if
				end do
			end if
		end if

		! initialise storage values
		u_nwo = -999.0
		val_1_nwo = -999.0
		emp1_nwo = -999.0
		emp2_nwo = -999.0
		dv_1_nwo = -999.0
		dv2_1_nwo = -999.0
		ri_nwo = -999.0
		pc_nwo = -999.0
		isa_nwo = -999.0
		train_nwo = -999.0
		prec_nwo = -999.0

		u_nwo1 = -999.0
		val_1_nwo1 = -999.0
		emp1_nwo1 = -999.0
		emp2_nwo1 = -999.0
		dv_1_nwo1 = -999.0
		dv2_1_nwo1 = -999.0
		ri_nwo1 = -999.0
		pc_nwo1 = -999.0
		isa_nwo1 = -999.0
		train_nwo1 = -999.0
		prec_nwo1 = -999.0

		u_nwo2 = -999.0
		val_1_nwo2 = -999.0
		emp1_nwo2 = -999.0
		emp2_nwo2 = -999.0
		dv_1_nwo2 = -999.0
		dv2_1_nwo2 = -999.0
		ri_nwo2 = -999.0
		pc_nwo2 = -999.0
		isa_nwo2 = -999.0
		train_nwo2 = -999.0
		prec_nwo2 = -999.0

		u_temp = -999.0
		val_1_temp = -999.0
		emp1_temp = -999.0
		emp2_temp = -999.0
		dv_1_temp = -999.0
		dv2_1_temp = -999.0
		ri_temp = -999.0
		pc_temp = -999.0
		isa_temp = -999.0
		train_temp = -999.0
		prec_temp = -999.0
		if ( model_par(16).gt.0.5 ) then
			
			!$OMP CRITICAL(write_output1)
			if ( (offer_axis_am1.gt.0_4).and.(current_age.le.nint(model_par(5))) ) then

				if ( offer_axis2_am1.eq.0_4 ) then
				! no separate wage axes for each spouse

					! no wage offer received
					u_nwo_s = vars_am1(start_here + i_inner, 2)
					var_id = 2
					if ( emp_par(1).gt.0.5 ) then

						var_id = var_id + 1
						var_id = var_id + 1
					end if
					if ( model_par(33).gt.0.5 ) then

						var_id = var_id + 1
					end if
					if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

						var_id = var_id + 1
						pc_nwo_s = vars_am1(start_here + i_inner, var_id)
					end if

					! wage offer received
					u_temp_s = vars_am1(start_here + wo_no + i_inner, 2)
					var_id = 2
					if ( emp_par(1).gt.0.5 ) then

						var_id = var_id + 1
						var_id = var_id + 1
					end if
					if ( model_par(33).gt.0.5 ) then

						var_id = var_id + 1
					end if
					if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

						var_id = var_id + 1
						pc_temp_s = vars_am1(start_here + wo_no + i_inner, 5)
					end if
				else
				! separate wage axes for each spouse

					! two 0 wage offers received
					u_nwo_s = vars_am1(start_here + wo_no*0 + i_inner, 2)
					var_id = 2
					if ( emp_par(1).gt.0.5 ) then

						var_id = var_id + 1
						var_id = var_id + 1
					end if
					if ( model_par(33).gt.0.5 ) then

						var_id = var_id + 1
					end if
					if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

						var_id = var_id + 1
						pc_nwo_s = vars_am1(start_here + wo_no*0 + i_inner, var_id)
					end if

					! wage offer for cohort member, no wage offer for spouse
					u_nwo2_s = vars_am1(start_here + wo_no*1 + i_inner, 2)
					var_id = 2
					if ( emp_par(1).gt.0.5 ) then

						var_id = var_id + 1
						var_id = var_id + 1
					end if
					if ( model_par(33).gt.0.5 ) then

						var_id = var_id + 1
					end if
					if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

						var_id = var_id + 1
						pc_nwo2_s = vars_am1(start_here + wo_no*1 + i_inner, var_id)
					end if

					! no wage offer for cohort member, wage offer for spouse
					u_nwo1_s = vars_am1(start_here + wo_no*2 + i_inner, 2)
					var_id = 2
					if ( emp_par(1).gt.0.5 ) then

						var_id = var_id + 1
						var_id = var_id + 1
					end if
					if ( model_par(33).gt.0.5 ) then

						var_id = var_id + 1
					end if
					if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

						var_id = var_id + 1
						pc_nwo1_s = vars_am1(start_here + wo_no*2 + i_inner, var_id)
					end if

					! wage offers for both cohort member and spouse
					u_temp_s = vars_am1(start_here + wo_no*3 + i_inner, 2)
					var_id = 2
					if ( emp_par(1).gt.0.5 ) then

						var_id = var_id + 1
						var_id = var_id + 1
					end if
					if ( model_par(33).gt.0.5 ) then

						var_id = var_id + 1
					end if
					if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

						var_id = var_id + 1
						pc_temp_s = vars_am1(start_here + wo_no*3 + i_inner, var_id)
					end if
				end if
			else
			! wage offers not included in analysis

				u_temp_s = vars_am1(i_inner+m_inner*(i_outer-1), 2)
				var_id = 2
				if ( model_par(33).gt.0.5 ) then

					var_id = var_id + 1
				end if
				if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

					var_id = var_id + 1
					pc_temp_s = vars_am1(i_inner+m_inner*(i_outer-1), var_id)
				end if
			end if
			!$OMP END CRITICAL(write_output1)
		end if
		if ( current_age.le.nint(model_par(5)) ) then
		! employment possible

			!**********************************************************************
			!
			!	Up to six decisions need to be made at each possible state-space
			!	combination during the working lifetime:
			!		- consumption / saving (required)
			!		- employment
			!		- training
			!		- pension receipt
			!		- division of liquid assets into safe and risky investments
			!			- only if xx(1) > wmin(r) = macro_par(15)
			!		- proportion of labour income contributed to private pension
			!			- only if lab_inc > ymin(p) = pen_par(32) and
			!					  lab_inc < ymax(p) = pen_par(33) and
			!					  age < SPA
			!
			!	The solution is organised as follows:
			!		1)	for any given combination of xx(1) and xx(2), a specific training, employment &
			!			pension receipt option is tested
			!		2)	given lab_inc and xx(1), the thresholds required for
			!			porfolio decisions over liquid assets and private pension
			!			contributions are checked
			!		3)	starting values for porfolio allocations and pension
			!			contributions are chosen
			!		4)	consumption is solved
			!		5)	portfolio allocations and pension contributions are adjusted
			!		6)	return to (4) until a local optimum is identified
			!		7)	store employment & training specific result and select new employment/training/pension receipt option
			!		8)	return to (2) and loop until all discrete options are tested
			!		9)	select decision combination associated with the highest V
			!
			!**********************************************************************

			if ( flag_skip.eq.0 ) then
			! consider this grid slice

				!**********************************************************************
				!	initialise variables to control cycle through training states
				!**********************************************************************

				if ( (model_par(64).gt.0.5).and.(flag_ad.eq.0) ) then
				! training taken into account and not discrete

					if ( abs(1.0-training_state).gt.0.01 ) then
					! training is possible

						training_options = nint(edu_par(19)+edu_par(20))
					else
					! training no longer possible as already upgraded

						training_options = 0
					end if
				else

					training_options = 0
				end if
				indx = 1
				indxnwo = 1
				indxnwo1 = 1
				indxnwo2 = 1
				do ll = 1, emp_states
				! loop up through employment states

					do kk = 0, training_options
					! loop up through training alternatives

						call emp_training_id(ll, kk, emp_cost, prwage1, prwage2, emp1, emp2, training_gain, leisure, labour, xx(2), flag_skip)
						if ( flag_skip.eq.0 ) then
						! consider this employment/training option

							do nwos1 = 1, nwo1_states

								flag_skip = 0
								if ( (nwos1.eq.2).and.((emp1.ne.0).or.(semp_state.eq.1)) ) then
								! no wage offer for individual 1 - must be employee with emp1 = 0

									flag_skip = 1
								elseif ( (nwos1.eq.1).and.(nwos1.ne.nwo1_states).and.(emp_par(1).lt.0.5).and.(emp1.eq.0).and.(current_age.lt.spa_bc) ) then
								! wage offer for individual 1, no employment decision - must be employed

									flag_skip = 1
								elseif ( (emp_par(1).lt.0.5).and.(current_age.ge.spa_bc).and.(emp1.gt.0) ) then
								! no employment decision and over state pension age - must be not employed

									flag_skip = 1
								end if
								if ( flag_skip.eq.0 ) then
								! consider this nwo/emp combination

									do nwos2 = 1, nwo2_states

										flag_skip = 0
										if ( (nwos2.eq.2).and.((emp2.ne.0).or.(semp_state.eq.1)) ) then
										! no wage offer for individual 2 - must be employee with emp2 = 0

											flag_skip = 1
										elseif ( (emp_par(1).lt.0.5).and.(na_g.eq.2).and.(current_age.lt.spa_bc).and.(emp_par(4).lt.0.5).and.(emp2.eq.0) ) then
										! individual 2 must be employed

											flag_skip = 1
										elseif ( (emp_par(1).lt.0.5).and.(na_g.eq.2).and.(current_age.lt.spa_bc).and.(nwos2.eq.1).and.(nwos2.ne.nwo2_states).and.(emp2.eq.0) ) then
										! wage offer for individual 2, no employment decision - must be employed

											flag_skip = 1
										elseif ( (emp_par(1).lt.0.5).and.(current_age.ge.spa_bc).and.(emp2.gt.0) ) then
										! no employment decision and over state pension age - must be not employed

											flag_skip = 1
										end if
										if ( flag_skip.eq.0 ) then
										! consider this nwo2/emp2 option

											do mm = 1, prec_options
											!loop through pension receipt options

												flag_skip = 0
												if ( mm.eq.2 ) then
												! test of voluntary pension take-up

													pens_takeup = 1
													if ( current_age.lt.page1_bc ) then
													! test only due to varying age thresholds

														flag_skip = 1
													elseif ( current_age.lt.page2_bc ) then
													! age regime 1

														if ( tax_par4(45).gt.0.5 ) then
															! must be retired

															if ( (emp1+emp2).gt.0 ) then

																flag_skip = 1
															end if
														end if
													elseif ( current_age.lt.page3_bc ) then
													! age regime 2

														if ( tax_par4(50).gt.0.5 ) then
														! must be retired

															if ( (emp1+emp2).gt.0 ) then

																flag_skip = 1
															end if
														end if
													else
													! age regime 3

														if ( tax_par4(55).gt.0.5 ) then
														! must be retired

															if ( (emp1+emp2).gt.0 ) then

																flag_skip = 1
															end if
														end if
													end if
												else
												! not voluntary takeup

													if ( prec_state.eq.0 ) then

														if ( (flag_ret_inner.eq.1).and.(emp_par(12).gt.0.5).and.(emp1.eq.0).and.(current_age.ge.page1_bc) ) then
														! timing of pension coincides with wage penalty switch

															pens_takeup = 1
														elseif ( (tax_par4(44).gt.0.5).and.(current_age.eq.pagemax_bc) ) then
														! endogenouse timing of pension receipt and at maximum age of pension take-up

															pens_takeup = 1
														elseif ( (tax_par4(44).lt.0.5).and.(current_age.eq.spa_bc) ) then

															pens_takeup = 1
														else

															pens_takeup = 0
														end if
													else
													! have already taken pension up

														pens_takeup = 0
													end if
													if ( current_age.lt.page2_bc ) then
													! age regime 1

														if ( ((tax_par4(45).gt.0.5).and.(pens_takeup.eq.1)).or.((tax_par4(46).gt.0.5).and.(prec_state.eq.1)) ) then
														! must be retired

															if ( (emp1+emp2).gt.0 ) then

																flag_skip = 1
															end if
														end if
													elseif ( current_age.lt.page3_bc ) then
													! age regime 2

														if ( ((tax_par4(50).gt.0.5).and.(pens_takeup.eq.1)).or.((tax_par4(51).gt.0.5).and.(prec_state.eq.1)) ) then
															! must be retired

															if ( (emp1+emp2).gt.0 ) then

																flag_skip = 1
															end if
														end if
													else
													! age regime 3

														if ( ((tax_par4(55).gt.0.5).and.(pens_takeup.eq.1)).or.((tax_par4(56).gt.0.5).and.(prec_state.eq.1)) ) then
														! must be retired

															if ( (emp1+emp2).gt.0 ) then

																flag_skip = 1
															end if
														end if
													end if
												end if
												if ( flag_skip.eq.0 ) then
												! undertake calculation

													if ( model_par(33).lt.0.5 ) then
													! no asset allocation between safe and risky assets

														ri_am1(indx) = 0.0_8
													end if
													if ( model_par(47).lt.0.5 ) then
													! no asset allocation to ISAs

														isa_am1(indx) = 0.0
													end if
													pen_regime = 1
													if ( model_par(16).lt.0.5 ) then
														
														if ( semp_state.eq.0 ) then

															if ( pen_par(14).gt.0.5 ) then
															! discrete state for pension contribution rate

																pc_am1(indx) = pen_par(51+pcr_state)
																pcemp_am1 = pen_par(56+pcr_state)
															else

																pc_am1(indx) = pen_par(27)
																pcemp_am1 = pen_par(31)
															end if
														else

															pcemp_am1 = 0.0
															if ( semp_par(2).gt.0.5 ) then

																pc_am1(indx) = semp_par(24)
															else

																pc_am1(indx) = 0.0
															end if
														end if
													else
														
														if ( (emp_par(3).gt.0.5).and.(semp_state.eq.0) ) then
														! need to track 0 wage offers for cohort member

															if ( emp_par(4).lt.0.5 ) then
															! only consider wage offers for cohort member

																if ( emp1.eq.0 ) then
																! cohort member not employed
			
																	u = u_nwo_s
																	if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

																		pc_am1(indx) = pc_nwo_s
																	else
																		
																		pc_am1(indx) = 0.0
																	end if
																else
			
																	u = u_temp_s
																	if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

																		pc_am1(indx) = pc_temp_s
																	else
																		
																		pc_am1(indx) = 0.0
																	end if
																end if
															else
															! consider wage offers for both spouses
		
																if ( (emp1.eq.0).and.(emp2.eq.0) ) then
			
																	u = u_nwo_s
																	if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

																		pc_am1(indx) = pc_nwo_s
																	else
																		
																		pc_am1(indx) = 0.0
																	end if
																elseif ( emp1.eq.0 ) then
			
																	u = u_nwo1_s
																	if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

																		pc_am1(indx) = pc_nwo1_s
																	else
																		
																		pc_am1(indx) = 0.0
																	end if
																elseif ( emp1.eq.0 ) then
			
																	u = u_nwo2_s
																	if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

																		pc_am1(indx) = pc_nwo2_s
																	else
																		
																		pc_am1(indx) = 0.0
																	end if
																else
			
																	u = u_temp_s
																	if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

																		pc_am1(indx) = pc_temp_s
																	else
																		
																		pc_am1(indx) = 0.0
																	end if
																end if
															end if
														else
														! no low wage offers
		
															u = u_temp_s
															pc_am1(indx) = pc_temp_s
														end if ! wage offers considered for analysis	
														if ( pc_am1(indx).gt.1.0E-5 ) then
														
															if ( pen_par(14).gt.0.5 ) then
															! discrete state for pension contribution rate

																pcemp_am1 = pen_par(56+pcr_state)
															else

																pcemp_am1 = pen_par(31)
															end if
														else
														
															pcemp_am1 = 0.0
														end if
													end if
													call continuous_choice(xx(1:ndim_inner), ndim_inner, n_iuser, n_ruser, training_state, leisure, labour, prwage1, prwage2, emp1, emp2, nwos1, kk, training_gain, pen_regime, pens_takeup, ri_am1(indx), pc_am1(indx), pcemp_am1, isa_am1(indx), u, val_1, dv_1, dv2_1)
													if ( (pen_par(144).gt.0.1).and.(pc_am1(indx).gt.1.0E-5).and.(semp_state.eq.0) ) then
													! need to consider employer matching contribution

														! update inputs for optimisation
														pen_regime = 2
														if ( pen_par(14).gt.0.5 ) then
														! discrete state for pension contribution rate

															if ( abs(pen_par(51+pcr_state)-pen_par(144+pcr_state)).lt.1.0E-5 ) then

																flag_skip = 1
															else

																flag_skip = 0
																pco_am1 = pen_par(144+pcr_state)
																pcemp_am1 = pen_par(149+pcr_state)
															end if
														else

															pco_am1 = pen_par(145)
															pcemp_am1 = pen_par(150)
														end if
														rio_am1 = ri_am1(indx)
														isao_am1 = isa_am1(indx)
														if ( flag_skip.eq.0 ) then

															if ( model_par(16).gt.0.5 ) then
														
																if ( (emp_par(3).gt.0.5).and.(semp_state.eq.0) ) then
																! need to track 0 wage offers for cohort member

																	if ( emp_par(4).lt.0.5 ) then
																	! only consider wage offers for cohort member

																		if ( emp1.eq.0 ) then
																		! cohort member not employed
			
																			uo = u_nwo_s
																		else
			
																			uo = u_temp_s
																		end if
																	else
																	! consider wage offers for both spouses
		
																		if ( (emp1.eq.0).and.(emp2.eq.0) ) then
			
																			uo = u_nwo_s
																		elseif ( emp1.eq.0 ) then
			
																			uo = u_nwo1_s
																		elseif ( emp1.eq.0 ) then
			
																			uo = u_nwo2_s
																		else
			
																			uo = u_temp_s
																		end if
																	end if
																else
																! no low wage offers
		
																	uo = u_temp_s
																end if ! wage offers considered for analysis	
															end if
															! solve optimisation
															call continuous_choice(xx(1:ndim_inner), ndim_inner, n_iuser, n_ruser, training_state, leisure, labour, prwage1, prwage2, emp1, emp2, nwos1, kk, training_gain, pen_regime, pens_takeup, rio_am1, pco_am1, pcemp_am1, isao_am1, uo, valo_1, dvo_1, dv2o_1)

															! update solutions
															if ( valo_1.gt.val_1 ) then

																u = uo
																val_1 = valo_1
																dv_1 = dvo_1
																dv2_1 = dv2o_1
																if ( model_par(33).gt.0.5 ) then

																	ri_am1(indx) = rio_am1
																end if
																pc_am1(indx) = pco_am1
																if ( model_par(47).gt.0.5 ) then

																	if ( abs(isao_am1+999.0).lt.0.01 ) then

																		isa_am1(indx) = -998.0
																	else

																		isa_am1(indx) = isao_am1
																	end if
																end if
															end if
														end if
													end if
													if ( (pen_par(12)*pen_par(11).gt.0.1).and.(pc_am1(indx).gt.1.0E-5).and.(semp_state.eq.0).and.(model_par(16).lt.0.5) ) then
													! need to consider opt-out condition for PP

														! update inputs for optimisation
														pen_regime = 0
														pco_am1 = 0.0
														pcemp_am1 = 0.0
														rio_am1 = ri_am1(indx)
														isao_am1 = isa_am1(indx)
														if ( model_par(16).gt.0.5 ) then
														
															if ( (emp_par(3).gt.0.5).and.(semp_state.eq.0) ) then
															! need to track 0 wage offers for cohort member

																if ( emp_par(4).lt.0.5 ) then
																! only consider wage offers for cohort member

																	if ( emp1.eq.0 ) then
																	! cohort member not employed
			
																		uo = u_nwo_s
																	else
			
																		uo = u_temp_s
																	end if
																else
																! consider wage offers for both spouses
		
																	if ( (emp1.eq.0).and.(emp2.eq.0) ) then
			
																		uo = u_nwo_s
																	elseif ( emp1.eq.0 ) then
			
																		uo = u_nwo1_s
																	elseif ( emp1.eq.0 ) then
			
																		uo = u_nwo2_s
																	else
			
																		uo = u_temp_s
																	end if
																end if
															else
															! no low wage offers
		
																uo = u_temp_s
															end if ! wage offers considered for analysis	
														end if

														! solve optimisation
														call continuous_choice(xx(1:ndim_inner), ndim_inner, n_iuser, n_ruser, training_state, leisure, labour, prwage1, prwage2, emp1, emp2, nwos1, kk, training_gain, pen_regime, pens_takeup, rio_am1, pco_am1, pcemp_am1, isao_am1, uo, valo_1, dvo_1, dv2o_1)

														! update solutions
														if ( valo_1.ge.val_1 ) then

															u = uo
															val_1 = valo_1
															dv_1 = dvo_1
															dv2_1 = dv2o_1
															if ( model_par(33).gt.0.5 ) then

																ri_am1(indx) = rio_am1
															end if
															pc_am1(indx) = pco_am1
															if ( model_par(47).gt.0.5 ) then

																if ( abs(isao_am1+999.0).lt.0.01 ) then

																	isa_am1(indx) = -998.0
																else

																	isa_am1(indx) = isao_am1
																end if
															end if
														end if
													end if


													!**********************************************************************************
													!
													!	record the running maximum of val_1
													!
													!**********************************************************************************

													if ( ((emp_par(1).gt.0.5).and.((indx.eq.1).or.(val_1.ge.val_1_temp))).or. &
														( (emp_par(1).lt.0.5).and.(val_1.ge.val_1_temp).and. &
														( ((current_age.lt.spa_bc).and.(emp1.gt.0)).or. &
														((current_age.ge.spa_bc).and.(emp1.eq.0)) ) ) ) then
													! initialise results or track maximum expected lifetime utility

													u_temp = u
													val_1_temp = val_1
													dv_1_temp = dv_1
													dv2_1_temp = dv2_1
													emp1_temp = real(emp1)
													emp2_temp = real(emp2)
													if ( model_par(33).gt.0.5 ) then

														ri_temp = ri_am1(indx)
													end if
													if ( pen_par(11).gt.0.5 ) then

														pc_temp = pc_am1(indx)
													end if
													if ( model_par(47).gt.0.5 ) then

														if ( abs(isa_am1(indx)+999.0).lt.0.01 ) then

															isa_temp = -998.0
														else

															isa_temp = isa_am1(indx)
														end if
													end if
													if ( model_par(64).gt.0.5 ) then

														train_temp = real(kk)
													end if
													if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5) ) then

														prec_temp = pens_takeup
													end if
													end if
													if ( emp_par(3).gt.0.5 ) then
													! need to track 0 wage offers for cohort member

														if ( semp_state.eq.1 ) then
														! self-employed, low wage offers not possible for both cohort member and spouse

															if ( indx.eq.1 ) then
															! initialise values

																u_nwo = -999.0
																val_1_nwo = -999.0
																dv_1_nwo = -999.0
																dv2_1_nwo = -999.0
																emp1_nwo = -999.0
																emp2_nwo = -999.0
																if ( model_par(33).gt.0.5 ) then

																	ri_nwo = -999.0
																end if
																if ( pen_par(11).gt.0.5 ) then

																	pc_nwo = -999.0
																end if
																if ( model_par(47).gt.0.5 ) then

																	isa_nwo = -999.0
																end if
																if ( model_par(64).gt.0.5 ) then

																	train_nwo = -999.0
																end if
																if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5) ) then

																	prec_nwo = -999.0
																end if
																indxnwo = indxnwo + 1
																if ( emp_par(4).gt.0.5 ) then

																	u_nwo1 = -999.0
																	val_1_nwo1 = -999.0
																	dv_1_nwo1 = -999.0
																	dv2_1_nwo1 = -999.0
																	emp1_nwo1 = -999.0
																	emp2_nwo1 = -999.0
																	if ( model_par(33).gt.0.5 ) then

																		ri_nwo1 = -999.0
																	end if
																	if ( pen_par(11).gt.0.5 ) then

																		pc_nwo1 = -999.0
																	end if
																	if ( model_par(47).gt.0.5 ) then

																		isa_nwo1 = -999.0
																	end if
																	if ( model_par(64).gt.0.5 ) then

																		train_nwo1 = -999.0
																	end if
																	if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5) ) then

																		prec_nwo1 = -999.0
																	end if
																	indxnwo1 = indxnwo1 + 1

																	u_nwo2 = -999.0
																	val_1_nwo2 = -999.0
																	dv_1_nwo2 = -999.0
																	dv2_1_nwo2 = -999.0
																	emp1_nwo2 = -999.0
																	emp2_nwo2 = -999.0
																	if ( model_par(33).gt.0.5 ) then

																		ri_nwo2 = -999.0
																	end if
																	if ( pen_par(11).gt.0.5 ) then

																		pc_nwo2 = -999.0
																	end if
																	if ( model_par(47).gt.0.5 ) then

																		isa_nwo2 = -999.0
																	end if
																	if ( model_par(64).gt.0.5 ) then

																		train_nwo2 = -999.0
																	end if
																	if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5) ) then

																		prec_nwo2 = -999.0
																	end if
																	indxnwo2 = indxnwo2 + 1
																end if
															end if
														else
														! not self employed - 0 wage offers possible for cohort member and spouse

															if ( (emp1.eq.0).and.(nwos1.eq.nwo1_states).and.((edu_par(5).gt.0.5).or.(kk.eq.0)).and. &
																((model_par(45).lt.0.5).or.(health1_state.le.nint(hlth_par(12)))).and. &
																((hlth_par(5).lt.0.5).or.(hlth_par(26).gt.0.5)) ) then
															! viable alternative for cohort member

															if ( emp_par(4).gt.0.5 ) then
															! separate wage offers for both spouses

																if ( (indxnwo1.eq.1).or.(val_1.ge.val_1_nwo1) ) then
																! initialise results or track maximum expected lifetime utility

																	u_nwo1 = u
																	val_1_nwo1 = val_1
																	dv_1_nwo1 = dv_1
																	dv2_1_nwo1 = dv2_1
																	emp1_nwo1 = real(emp1)
																	emp2_nwo1 = real(emp2)
																	if ( model_par(33).gt.0.5 ) then

																		ri_nwo1 = ri_am1(indx)
																	end if
																	if ( pen_par(11).gt.0.5 ) then

																		pc_nwo1 = pc_am1(indx)
																	end if
																	if ( model_par(47).gt.0.5 ) then

																		if ( abs(isa_am1(indx)+999.0).lt.0.01 ) then

																			isa_nwo1 = -998.0
																		else

																			isa_nwo1 = isa_am1(indx)
																		end if
																	end if
																	if ( model_par(64).gt.0.5 ) then

																		train_nwo1 = real(kk)
																	end if
																	if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5) ) then

																		prec_nwo1 = pens_takeup
																	end if
																	indxnwo1 = indxnwo1 + 1
																	if ( na_g.lt.1.5 ) then
																	! initialise spouse values

																		u_nwo2 = -999.0
																		val_1_nwo2 = -999.0
																		dv_1_nwo2 = -999.0
																		dv2_1_nwo2 = -999.0
																		emp1_nwo2 = -999.0
																		emp2_nwo2 = -999.0
																		if ( model_par(33).gt.0.5 ) then

																			ri_nwo2 = -999.0
																		end if
																		if ( pen_par(11).gt.0.5 ) then

																			pc_nwo2 = -999.0
																		end if
																		if ( model_par(47).gt.0.5 ) then

																			isa_nwo2 = -999.0
																		end if
																		if ( model_par(64).gt.0.5 ) then

																			train_nwo2 = -999.0
																		end if
																		if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5) ) then

																			prec_nwo2 = -999.0
																		end if
																		indxnwo2 = indxnwo2 + 1

																		u_nwo = -999.0
																		val_1_nwo = -999.0
																		dv_1_nwo = -999.0
																		dv2_1_nwo = -999.0
																		emp1_nwo = -999.0
																		emp2_nwo = -999.0
																		if ( model_par(33).gt.0.5 ) then

																			ri_nwo = -999.0
																		end if
																		if ( pen_par(11).gt.0.5 ) then

																			pc_nwo = -999.0
																		end if
																		if ( model_par(47).gt.0.5 ) then

																			isa_nwo = -999.0
																		end if
																		if ( model_par(64).gt.0.5 ) then

																			train_nwo = -999.0
																		end if
																		if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5) ) then

																			prec_nwo = -999.0
																		end if
																		indxnwo = indxnwo + 1
																	end if
																end if ! track maximum
															else
															! only one low wage offer considered

																if ( (indxnwo.eq.1).or.(val_1.ge.val_1_nwo) ) then
																! initialise results or track maximum expected lifetime utility

																	u_nwo = u
																	val_1_nwo = val_1
																	dv_1_nwo = dv_1
																	dv2_1_nwo = dv2_1
																	emp1_nwo = real(emp1)
																	emp2_nwo = real(emp2)
																	if ( model_par(33).gt.0.5 ) then

																		ri_nwo = ri_am1(indx)
																	end if
																	if ( pen_par(11).gt.0.5 ) then

																		pc_nwo = pc_am1(indx)
																	end if
																	if ( model_par(47).gt.0.5 ) then

																		if ( abs(isa_am1(indx)+999.0).lt.0.01 ) then

																			isa_nwo = -998.0
																		else

																			isa_nwo = isa_am1(indx)
																		end if
																	end if
																	if ( model_par(64).gt.0.5 ) then

																		train_nwo = real(kk)
																	end if
																	if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5) ) then

																		prec_nwo = pens_takeup
																	end if
																	indxnwo = indxnwo + 1
																end if ! track maximum wage offer
															end if ! offers for both spouses / one spouse
															end if ! viable low wage offer for 1
															if ( (emp_par(4).gt.0.5).and.(na_g.gt.1.5) ) then
															! low wage offers considered for each spouse

																if ( (emp2.eq.0).and.(nwos2.eq.nwo2_states).and.((model_par(45).lt.0.5).or.(health2_state.le.nint(hlth_par(12)))) ) then
																! viable alternative for spouse 2

																	if ( (indxnwo2.eq.1).or.(val_1.ge.val_1_nwo2) ) then
																	! initialise results or track maximum expected lifetime utility

																		u_nwo2 = u
																		val_1_nwo2 = val_1
																		dv_1_nwo2 = dv_1
																		dv2_1_nwo2 = dv2_1
																		emp1_nwo2 = real(emp1)
																		emp2_nwo2 = real(emp2)
																		if ( model_par(33).gt.0.5 ) then

																			ri_nwo2 = ri_am1(indx)
																		end if
																		if ( pen_par(11).gt.0.5 ) then

																			pc_nwo2 = pc_am1(indx)
																		end if
																		if ( model_par(47).gt.0.5 ) then

																			if ( abs(isa_am1(indx)+999.0).lt.0.01 ) then

																				isa_nwo2 = -998.0
																			else

																				isa_nwo2 = isa_am1(indx)
																			end if
																		end if
																		if ( model_par(64).gt.0.5 ) then

																			train_nwo2 = real(kk)
																		end if
																		if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5) ) then

																			prec_nwo2 = pens_takeup
																		end if
																		indxnwo2 = indxnwo2 + 1
																	end if
																	if ( (emp1.eq.0).and.(nwos1.eq.nwo1_states).and.((edu_par(5).gt.0.5).or.(kk.eq.0)).and. &
																		((model_par(45).lt.0.5).or.(health1_state.le.nint(hlth_par(12)))) ) then
																	! viable alternative for both spouses

																	if ( (indxnwo.eq.1).or.(val_1.ge.val_1_nwo) ) then
																	! initialise results or track maximum expected lifetime utility

																		u_nwo = u
																		val_1_nwo = val_1
																		dv_1_nwo = dv_1
																		dv2_1_nwo = dv2_1
																		emp1_nwo = real(emp1)
																		emp2_nwo = real(emp2)
																		if ( model_par(33).gt.0.5 ) then

																			ri_nwo = ri_am1(indx)
																		end if
																		if ( pen_par(11).gt.0.5 ) then

																			pc_nwo = pc_am1(indx)
																		end if
																		if ( model_par(47).gt.0.5 ) then

																			if ( abs(isa_am1(indx)+999.0).lt.0.01 ) then

																				isa_nwo = -998.0
																			else

																				isa_nwo = isa_am1(indx)
																			end if
																		end if
																		if ( model_par(64).gt.0.5 ) then

																			train_nwo = real(kk)
																		end if
																		if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5) ) then

																			prec_nwo = pens_takeup
																		end if
																		indxnwo = indxnwo + 1
																	end if ! update low wage offer at household level
																	end if ! viable low wage offer at household level
																end if ! viable low wage offer for spouse 2
															end if ! low wage offers considered for spouse
														end if ! not self employed
													end if ! wage offers considered for analysis
													indx = indx + 1
												end if ! end flag_skip (inner)
											end do ! end pension receipt loop
										end if ! end flag_skip
									end do ! end wage offer2 loop
								end if	! end flag_skip
							end do ! end wage offer1 loop
						end if	! end flag_skip
					end do ! end training loop
				end do ! end employment loop
			end if ! end flag_skip (outer)
			! store results

			!$OMP CRITICAL(write_output1)
			if ( offer_axis_am1.gt.0_4 ) then

				if ( offer_axis2_am1.eq.0_4 ) then
				! no separate wage axes for each spouse

					var_id = 1
					vars_am1(start_here + i_inner, var_id) = val_1_nwo
					var_id = var_id + 1
					vars_am1(start_here + i_inner, var_id) = u_nwo
					if ( emp_par(1).gt.0.5 ) then

						var_id = var_id + 1
						vars_am1(start_here + i_inner, var_id) = emp1_nwo
						var_id = var_id + 1
						vars_am1(start_here + i_inner, var_id) = emp2_nwo
					end if
					if ( model_par(33).gt.0.5 ) then

						var_id = var_id + 1
						vars_am1(start_here + i_inner, var_id) = ri_nwo
					end if
					if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

						var_id = var_id + 1
						vars_am1(start_here + i_inner, var_id) = pc_nwo
					end if
					if ( model_par(47).gt.0.5 ) then
					! isa decision saved

						var_id = var_id + 1
						vars_am1(start_here + i_inner, var_id) = isa_nwo
					end if
					if ( (model_par(64).gt.0.5).and.(current_age.le.nint(edu_par(22))) ) then
					! training decision saved here

						var_id = var_id + 1
						vars_am1(start_here + i_inner, var_id) = train_nwo
					end if
					if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.(current_age.ge.min_page).and.(current_age.lt.max_page) ) then
					! pension receipt decision saved here

						var_id = var_id + 1
						vars_am1(start_here + i_inner, var_id) = prec_nwo
					end if
					if ( model_par(49).gt.0.5 ) then
					! Quasi-hyperbolic discounting considered

						var_id = var_id + 1
						vars_am1(start_here + i_inner, var_id) = dv_1_nwo
						var_id = var_id + 1
						vars_am1(start_here + i_inner, var_id) = dv2_1_nwo
					end if

					var_id = 1
					vars_am1(start_here + wo_no + i_inner, var_id) = val_1_temp
					var_id = var_id + 1
					vars_am1(start_here + wo_no + i_inner, var_id) = u_temp
					if ( emp_par(1).gt.0.5 ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no + i_inner, var_id) = emp1_temp
						var_id = var_id + 1
						vars_am1(start_here + wo_no + i_inner, var_id) = emp2_temp
					end if
					if ( model_par(33).gt.0.5 ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no + i_inner, var_id) = ri_temp
					end if
					if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no + i_inner, var_id) = pc_temp
					end if
					if ( model_par(47).gt.0.5 ) then
					! isa decision saved

						var_id = var_id + 1
						vars_am1(start_here + wo_no + i_inner, var_id) = isa_temp
					end if
					if ( (model_par(64).gt.0.5).and.(current_age.le.nint(edu_par(22))) ) then
					! training decision saved here

						var_id = var_id + 1
						vars_am1(start_here + wo_no + i_inner, var_id) = train_temp
					end if
					if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.(current_age.ge.min_page).and.(current_age.lt.max_page) ) then
					! pension receipt decision saved here

						var_id = var_id + 1
						vars_am1(start_here + wo_no + i_inner, var_id) = prec_temp
					end if
					if ( model_par(49).gt.0.5 ) then
					! Quasi-hyperbolic discounting considered

						var_id = var_id + 1
						vars_am1(start_here + wo_no + i_inner, var_id) = dv_1_temp
						var_id = var_id + 1
						vars_am1(start_here + wo_no + i_inner, var_id) = dv2_1_temp
					end if
				else
				! separate wage axes for each spouse

					! two 0 wage offers received
					var_id = 1
					vars_am1(start_here + wo_no*0 + i_inner, var_id) = val_1_nwo
					var_id = var_id + 1
					vars_am1(start_here + wo_no*0 + i_inner, var_id) = u_nwo
					if ( emp_par(1).gt.0.5 ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no*0 + i_inner, var_id) = emp1_nwo
						var_id = var_id + 1
						vars_am1(start_here + wo_no*0 + i_inner, var_id) = emp2_nwo
					end if
					if ( model_par(33).gt.0.5 ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no*0 + i_inner, var_id) = ri_nwo
					end if
					if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no*0 + i_inner, var_id) = pc_nwo
					end if
					if ( model_par(47).gt.0.5 ) then
					! isa decision saved

						var_id = var_id + 1
						vars_am1(start_here + wo_no*0 + i_inner, var_id) = isa_nwo
					end if
					if ( (model_par(64).gt.0.5).and.(current_age.le.nint(edu_par(22))) ) then
					! training decision saved here

						var_id = var_id + 1
						vars_am1(start_here + wo_no*0 + i_inner, var_id) = train_nwo
					end if
					if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.(current_age.ge.min_page).and.(current_age.lt.max_page) ) then
					! pension receipt decision saved here

						var_id = var_id + 1
						vars_am1(start_here + wo_no*0 + i_inner, var_id) = prec_nwo
					end if
					if ( model_par(49).gt.0.5 ) then
					! Quasi-hyperbolic discounting considered

						var_id = var_id + 1
						vars_am1(start_here + wo_no*0 + i_inner, var_id) = dv_1_nwo
						var_id = var_id + 1
						vars_am1(start_here + wo_no*0 + i_inner, var_id) = dv2_1_nwo
					end if

					! wage offer for cohort member, no wage offer for spouse
					var_id = 1
					vars_am1(start_here + wo_no*1 + i_inner, var_id) = val_1_nwo2
					var_id = var_id + 1
					vars_am1(start_here + wo_no*1 + i_inner, var_id) = u_nwo2
					if ( emp_par(1).gt.0.5 ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no*1 + i_inner, var_id) = emp1_nwo2
						var_id = var_id + 1
						vars_am1(start_here + wo_no*1 + i_inner, var_id) = emp2_nwo2
					end if
					if ( model_par(33).gt.0.5 ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no*1 + i_inner, var_id) = ri_nwo2
					end if
					if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no*1 + i_inner, var_id) = pc_nwo2
					end if
					if ( model_par(47).gt.0.5 ) then
					! isa decision saved

						var_id = var_id + 1
						vars_am1(start_here + wo_no*1 + i_inner, var_id) = isa_nwo2
					end if
					if ( (model_par(64).gt.0.5).and.(current_age.le.nint(edu_par(22))) ) then
					! training decision saved here

						var_id = var_id + 1
						vars_am1(start_here + wo_no*1 + i_inner, var_id) = train_nwo2
					end if
					if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.(current_age.ge.min_page).and.(current_age.lt.max_page) ) then
					! pension receipt decision saved here

						var_id = var_id + 1
						vars_am1(start_here + wo_no*1 + i_inner, var_id) = prec_nwo2
					end if
					if ( model_par(49).gt.0.5 ) then
					! Quasi-hyperbolic discounting considered

						var_id = var_id + 1
						vars_am1(start_here + wo_no*1 + i_inner, var_id) = dv_1_nwo2
						var_id = var_id + 1
						vars_am1(start_here + wo_no*1 + i_inner, var_id) = dv2_1_nwo2
					end if

					! no wage offer for cohort member, wage offer for spouse
					var_id = 1
					vars_am1(start_here + wo_no*2 + i_inner, var_id) = val_1_nwo1
					var_id = var_id + 1
					vars_am1(start_here + wo_no*2 + i_inner, var_id) = u_nwo1
					if ( emp_par(1).gt.0.5 ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no*2 + i_inner, var_id) = emp1_nwo1
						var_id = var_id + 1
						vars_am1(start_here + wo_no*2 + i_inner, var_id) = emp2_nwo1
					end if
					if ( model_par(33).gt.0.5 ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no*2 + i_inner, var_id) = ri_nwo1
					end if
					if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no*2 + i_inner, var_id) = pc_nwo1
					end if
					if ( model_par(47).gt.0.5 ) then
					! isa decision saved

						var_id = var_id + 1
						vars_am1(start_here + wo_no*2 + i_inner, var_id) = isa_nwo1
					end if
					if ( (model_par(64).gt.0.5).and.(current_age.le.nint(edu_par(22))) ) then
					! training decision saved here

						var_id = var_id + 1
						vars_am1(start_here + wo_no*2 + i_inner, var_id) = train_nwo1
					end if
					if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.(current_age.ge.min_page).and.(current_age.lt.max_page) ) then
					! pension receipt decision saved here

						var_id = var_id + 1
						vars_am1(start_here + wo_no*2 + i_inner, var_id) = prec_nwo1
					end if
					if ( model_par(49).gt.0.5 ) then
					! Quasi-hyperbolic discounting considered

						var_id = var_id + 1
						vars_am1(start_here + wo_no*2 + i_inner, var_id) = dv_1_nwo1
						var_id = var_id + 1
						vars_am1(start_here + wo_no*2 + i_inner, var_id) = dv2_1_nwo1
					end if

					! wage offers for both cohort member and spouse
					var_id = 1
					vars_am1(start_here + wo_no*3 + i_inner, var_id) = val_1_temp
					var_id = var_id + 1
					vars_am1(start_here + wo_no*3 + i_inner, var_id) = u_temp
					if ( emp_par(1).gt.0.5 ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no*3 + i_inner, var_id) = emp1_temp
						var_id = var_id + 1
						vars_am1(start_here + wo_no*3 + i_inner, var_id) = emp2_temp
					end if
					if ( model_par(33).gt.0.5 ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no*3 + i_inner, var_id) = ri_temp
					end if
					if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

						var_id = var_id + 1
						vars_am1(start_here + wo_no*3 + i_inner, var_id) = pc_temp
					end if
					if ( model_par(47).gt.0.5 ) then
					! isa decision saved

						var_id = var_id + 1
						vars_am1(start_here + wo_no*3 + i_inner, var_id) = isa_temp
					end if
					if ( (model_par(64).gt.0.5).and.(current_age.le.nint(edu_par(22))) ) then
					! training decision saved here

						var_id = var_id + 1
						vars_am1(start_here + wo_no*3 + i_inner, var_id) = train_temp
					end if
					if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.(current_age.ge.min_page).and.(current_age.lt.max_page) ) then
					! pension receipt decision saved here

						var_id = var_id + 1
						vars_am1(start_here + wo_no*3 + i_inner, var_id) = prec_temp
					end if
					if ( model_par(49).gt.0.5 ) then
					! Quasi-hyperbolic discounting considered

						var_id = var_id + 1
						vars_am1(start_here + wo_no*3 + i_inner, var_id) = dv_1_temp
						var_id = var_id + 1
						vars_am1(start_here + wo_no*3 + i_inner, var_id) = dv2_1_temp
					end if
				end if
			else
			! wage offers not included in analysis

				var_id = 1
				vars_am1(i_inner+m_inner*(i_outer-1), var_id) = val_1_temp
				var_id = var_id + 1
				vars_am1(i_inner+m_inner*(i_outer-1), var_id) = u_temp
				if ( emp_par(1).gt.0.5 ) then

					var_id = var_id + 1
					vars_am1(i_inner+m_inner*(i_outer-1), var_id) = emp1_temp
					var_id = var_id + 1
					vars_am1(i_inner+m_inner*(i_outer-1), var_id) = emp2_temp
				end if
				if ( model_par(33).gt.0.5 ) then

					var_id = var_id + 1
					vars_am1(i_inner+m_inner*(i_outer-1), var_id) = ri_temp
				end if
				if ( (pen_par(11).gt.0.5).and.( current_age.lt.max( max_page, nint( tax_par4(59)*(model_par(5)+1.0) ) ) ) ) then

					var_id = var_id + 1
					vars_am1(i_inner+m_inner*(i_outer-1), var_id) = pc_temp
				end if
				if ( model_par(47).gt.0.5 ) then
				! isa decision saved

					var_id = var_id + 1
					vars_am1(i_inner+m_inner*(i_outer-1), var_id) = isa_temp
				end if
				if ( (model_par(64).gt.0.5).and.(current_age.le.nint(edu_par(22))) ) then
				! training decisions saved here

					var_id = var_id + 1
					vars_am1(i_inner+m_inner*(i_outer-1), var_id) = train_temp
				end if
				if ( (tax_par4(44).gt.0.5).and.(emp_par(12).lt.0.5).and.(current_age.ge.min_page).and.(current_age.lt.max_page) ) then
				! pension receipt decision saved here

					var_id = var_id + 1
					vars_am1(i_inner+m_inner*(i_outer-1), var_id) = prec_temp
				end if
				if ( model_par(49).gt.0.5 ) then
				! Quasi-hyperbolic discounting considered

					var_id = var_id + 1
					vars_am1(i_inner+m_inner*(i_outer-1), var_id) = dv_1_temp
					var_id = var_id + 1
					vars_am1(i_inner+m_inner*(i_outer-1), var_id) = dv2_1_temp
				end if
			end if
			!$OMP END CRITICAL(write_output1)
		else
		! employment no longer possible - pension receipt (if considered) is assumed

			!**********************************************************************
			!
			!	Up to two decisions need to be made at each possible state-space
			!	combination during mandatory retirement:
			!		- consumption / saving (required)
			!		- division of liquid assets into safe and risky investments
			!			- only if xx(1) > wmin(r)
			!
			!	The solution is organised as follows:
			!		1)	given xx(1), the threshold required for porfolio
			!			decisions over liquid assets is checked
			!		2)	a starting value for the porfolio allocations is chosen
			!		3)	consumption is solved
			!		4)	the portfolio allocation is adjusted
			!		5)	return to (3) until a local optimum is identified
			!
			!**********************************************************************

			if ( flag_skip.eq.0 ) then
			! undertake calculations

				pc_am1(1) = 0.0
				if ( (model_par(33).lt.0.5).or.(current_age.eq.nint(model_par(7))) ) then
				! no asset allocation between safe and risky assets

					ri_am1(1) = 0.0
				end if
				if ( model_par(47).lt.0.5 ) then
				! no isa investments

					isa_am1(1) = 0.0
				end if
				training_state = 0.0
				leisure = 1.0
				if ( hlth_par(5).gt.0.5 ) then

					if ( care_state.eq.1 ) then

						leisure = leisure - hlth_par(27) / na_g
					end if
				end if
				labour = 0.0
				emp1 = 0
				emp2 = 0
				prwage1 = 0.0
				prwage2 = 0.0
				training_gain = 0.0
				pcemp_am1 = 0.0
				kk = 0
				nwos1 = 1
				nwos2 = 1
				if ( prec_state.eq.0 ) then
				! forced pension take-up

					pens_takeup = 1
				else

					pens_takeup = 0
				end if
				pen_regime = 0
				pcemp_am1 = 0.0
				if ( model_par(16).gt.0.5 ) then
					
					u = u_temp_s
				end if
				call continuous_choice(xx(1:ndim_inner), ndim_inner, n_iuser, n_ruser, training_state, leisure, labour, prwage1, prwage2, emp1, emp2, nwos1, kk, training_gain, pen_regime, pens_takeup, ri_am1(1), pc_am1(1), pcemp_am1, isa_am1(1), u, val_1, dv_1, dv2_1)
				if ( abs(isa_am1(1)+999.0).lt.0.01 ) then

					isa_am1(1) = -998.0
				end if
			else
			! skip grid slice

				u = -999.0
				val_1 = -999.0
				leisure = -999.0
				dv_1 = -999.0
				dv2_1 = -999.0
				ri_am1(1) = -999.0
				isa_am1(1) = -999.0
			end if

			!$OMP CRITICAL(write_output2)
			var_id = 1
			vars_am1(i_inner+m_inner*(i_outer-1), var_id) = val_1
			var_id = var_id + 1
			vars_am1(i_inner+m_inner*(i_outer-1), var_id) = u
			if ( model_par(33).gt.0.5 ) then

				var_id = var_id + 1
				vars_am1(i_inner+m_inner*(i_outer-1),var_id) = ri_am1(1)
			end if
			if ( model_par(47).gt.0.5 ) then

				var_id = var_id + 1
				vars_am1(i_inner+m_inner*(i_outer-1),var_id) = isa_am1(1)
			end if
			if ( model_par(49).gt.0.5 ) then
				! Quasi-hyperbolic discounting considered

				var_id = var_id + 1
				vars_am1(i_inner+m_inner*(i_outer-1), var_id) = dv_1
				var_id = var_id + 1
				vars_am1(i_inner+m_inner*(i_outer-1), var_id) = dv2_1
			end if
			!$OMP END CRITICAL(write_output2)
		end if
	end do	! end of non-parallelised inner loop
end do	! end of parallelised inner loop
!$OMP END PARALLEL DO
deallocate( V_str )
if ( allocated ( dV_str ) ) deallocate ( dV_str, dV2_str )
end do	! end of outer loop

if ( abs(model_par(1)-1.0).gt.1.0E-5 ) then

	do ii = 1, size(vars_am1(:,1))

		if ( vars_am1(ii,1).lt.0.0 ) then

			if ( (abs(vars_am1(ii,1)+999.0).lt.1.0E-5).and. &
				 ( (current_age.le.nint(model_par(5))).and. &
				   ( (semp_par(1).gt.0.5).and. &
				     ( (UI_state.eq.1).or.(semp_par(3).gt.0.5).or.(emp_par(7).gt.0.5).or.(emp_par(3).gt.0.5) ) ) ) ) then
			! this is possible, where -999 defines grid slice that was skipped

				continue
			elseif ( pen_par(17).gt.0.5 ) then
			! this is possible, where -999 defines domain in the pension grid not applicable to singles

				continue
			elseif ( model_par(45).gt.0.5 ) then
			! this is possible, where -999 defines health/carer states that are not applicable 
				
				continue
			else
			! this is not possible

				print *, 'problem in evaluation of value function'
				pause 'press enter to exit'
				stop
			end if
		elseif ( vars_am1(ii,1).lt.1.0E-9 ) then

			flag_smallv = 3
		elseif ( vars_am1(ii,1).lt.1.0E-8 ) then

			flag_smallv = max(flag_smallv, 2)
		elseif ( vars_am1(ii,1).lt.1.0E-7 ) then

			flag_smallv = max(flag_smallv, 1)
		end if
	end do
end if


deallocate ( prob_d, xexp_d, xexp_d2, hl_wf_ap1, nk_allg, nkd_all, child_ages )
if ( allocated ( prob_g0 ) ) deallocate ( prob_g0, abscissae0 )
if ( allocated ( prob_g1 ) ) deallocate ( prob_g1, abscissae1 )
if ( allocated ( prob_g2 ) ) deallocate ( prob_g2, abscissae2 )


END SUBROUTINE tm_grid_cal


SUBROUTINE inner_state(xx, training_state, training_state1, step_inner, dims_inner, ndim_inner, axis_am1, ndim_am1, i_inner_w, i_inner_nw, flag_skip)

!*************************************************************
!
!	subroutine to evaluate values of xx for given loop index
!
!*************************************************************

USE global
USE global_ParamStore
USE global_CommonCohort

IMPLICIT none


integer(4) :: ndim_inner, ndim_am1, i_inner_w, i_inner_nw, dims_inner(ndim_inner), flag_skip
real(8) :: xx(ndim_inner), training_state1, training_state
real(8) :: step_inner(ndim_inner), axis_am1(4, ndim_am1)

! local variables
integer(4) :: indx, no, counter, test_flag


!*********************************************************
!	begin code
!*********************************************************
test_flag = 1
flag_skip = 0
indx = 1
xx(indx) = axis_am1(1,indx) + step_inner(indx) * (i_inner_w - 1_4)
if ( grid_typ(2).gt.0.5 ) then
	! first dimension is always wealth

	xx(indx) = dexp(xx(indx)) - grid_typ(4)
end if
if ( abs(xx(indx)).lt.1E-6 ) then

	xx(indx) = 0.0
end if
if ( ((i_inner_w.eq.1).and.(ndim_inner.gt.1)).or.(test_flag.eq.1) ) then
	! need to initialise other inner dimensions for thread

	if ( current_age.le.nint(model_par(5)) ) then
		! next dimension is wage

		indx = indx + 1
		if ( estw2_par(1).gt.0.5 ) then
			! wages modelled on estimated earnings equation 2

			! persistent wage innovations
			no = mod( (i_inner_nw-1), dims_inner(indx) )
			xx(indx) = axis_am1(1,indx) + step_inner(indx) * real(no)
			if ( estw2_par(4).gt.0.5 ) then
				! temporary wage innovations

				indx = indx + 1
				counter = 1 + int( (real(i_inner_nw)-0.01) / real(product(dims_inner(2:indx-1))) )
				no = mod( (counter-1), dims_inner(indx) )
				xx(indx) = axis_am1(1,indx) + step_inner(indx) * real(no)
			end if
			if ( estw2_par(2).gt.0.5 ) then
				! individual specific effects

				indx = indx + 1
				counter = 1 + int( (real(i_inner_nw)-0.01) / real(product(dims_inner(2:indx-1))) )
				no = mod( (counter-1), dims_inner(indx) )
				xx(indx) = axis_am1(1,indx) + step_inner(indx) * real(no)
			end if
		else
			! wages not modelled on earnings equation 2

			no = mod( (i_inner_nw-1), dims_inner(indx) )
			xx(indx) = axis_am1(1,indx) + step_inner(indx) * real(no)
			if ( grid_typ(1).gt.0.5 ) then

				xx(indx) = dexp(xx(indx)) - grid_typ(3)
			end if
			if ( abs(xx(indx)).lt.(grid_par(7)+1E-6) ) then

				xx(indx) = grid_par(7)
			end if
		end if
	end if
	if ( (model_par(11).gt.0.1).or.( (current_age.gt.min_page).and.(pen_par(156).gt.0.5) ) ) then
		! next dimension is OP pension annuity

		indx = indx + 1
		counter = 1 + int( (real(i_inner_nw)-0.01) / real(product(dims_inner(2:indx-1))) )
		no = mod( (counter-1), dims_inner(indx) )
		xx(indx) = axis_am1(1,indx) + step_inner(indx) * real(no)
		if ( grid_typ(14).gt.0.5 ) then

			xx(indx) = dexp(xx(indx)) - grid_typ(16)
			if ( (pen_par(17).gt.0.5).and.(na_g.lt.1.5).and.((no+1).gt.nint(grid_typ(15))).and.(current_age.le.nint(model_par(5))) ) then
				! outside bounds for single adult

				flag_skip = 1
			end if
		else

			if ( (pen_par(17).gt.0.5).and.(na_g.lt.1.5).and.((no+1).gt.nint(1.0+grid_par(23)/2.0)).and.(current_age.le.nint(model_par(5))) ) then
				! outside bounds for single adult

				flag_skip = 1
			end if
		end if
		if ( abs(xx(indx)).lt.1E-6 ) then

			xx(indx) = 0.0
		end if
	end if
	if ( (model_par(26).gt.0.1).and.(PPOP_state.eq.0).and.( (current_age.le.min_page).or.( (model_par(11).gt.0.1).and.(current_age.le.max_page) ) ) ) then
		! next dimension is PP pension annuity

		indx = indx + 1
		counter = 1 + int( (real(i_inner_nw)-0.01) / real(product(dims_inner(2:indx-1))) )
		no = mod( (counter-1), dims_inner(indx) )
		xx(indx) = axis_am1(1,indx) + step_inner(indx) * real(no)
		if ( (grid_typ(11).gt.0.5).and.(nopen_a.eq.0) ) then

			xx(indx) = dexp(xx(indx)) - grid_typ(13)
			if ( (pen_par(17).gt.0.5).and.(na_g.lt.1.5).and.((no+1).gt.nint(grid_typ(12))).and.(current_age.le.nint(model_par(5))) ) then
				! outside bounds for single adult

				flag_skip = 1
			end if
		else

			if ( (pen_par(17).gt.0.5).and.(na_g.lt.1.5).and.((no+1).gt.nint(1.0+grid_par(21)/2.0)).and.(current_age.le.nint(model_par(5))) ) then
				! outside bounds for single adult

				flag_skip = 1
			end if
		end if
		if ( abs(xx(indx)).lt.1E-6 ) then

			xx(indx) = 0.0
		end if
	end if
	if ( ((semp_par(1)*semp_par(3)).gt.0.5).and.(current_age.le.nint(model_par(5))) ) then
		! next dimension is own-business assets

		indx = indx + 1
		counter = 1 + int( (real(i_inner_nw)-0.01) / real(product(dims_inner(2:indx-1))) )
		no = mod( (counter-1), dims_inner(indx) )
		xx(indx) = axis_am1(1,indx) + step_inner(indx) * real(no)
		if ( grid_typ(9).gt.0.5 ) then
			! first dimension is always wealth

			xx(indx) = dexp(xx(indx)) - grid_typ(10)
		end if
		if ( abs(xx(indx)).lt.1E-6 ) then

			xx(indx) = 0.0
		end if
		if ( (xx(indx).gt.0.1).and.(semp_state.eq.0) ) then
			! can only have own business assets if self-employed

			flag_skip = 1
		end if
	end if
	if ( pen_par(21).gt.0.5 ) then
		! next dimension is contributory pension 1

		indx = indx + 1
		counter = 1 + int( (real(i_inner_nw)-0.01) / real(product(dims_inner(2:indx-1))) )
		no = mod( (counter-1), dims_inner(indx) )
		xx(indx) = axis_am1(1,indx) + step_inner(indx) * real(no)
		if ( abs(xx(indx)).lt.1E-6 ) then

			xx(indx) = 0.0
		end if
		if ( (xx(indx).gt.(grid_par(6)/2.0+0.000001)).and.(na_g.lt.1.00001) ) then

			flag_skip = 1
		end if
	end if
	if ( (cp2_state.eq.1).or.((cp2_state.eq.2).and.(current_age.le.max_page)) ) then
		! next dimension is contributory pension 2

		indx = indx + 1
		counter = 1 + int( (real(i_inner_nw)-0.01) / real(product(dims_inner(2:indx-1))) )
		no = mod( (counter-1), dims_inner(indx) )
		xx(indx) = axis_am1(1,indx) + step_inner(indx) * real(no)
		if ( abs(xx(indx)).lt.1E-6 ) then

			xx(indx) = 0.0
		end if
	end if
	if ( model_par(47).gt.0.5 ) then
		! next dimension is savings in ISAs

		indx = indx + 1
		counter = 1 + int( (real(i_inner_nw)-0.01) / real(product(dims_inner(2:indx-1))) )
		no = mod( (counter-1), dims_inner(indx) )
		xx(indx) = axis_am1(1,indx) + step_inner(indx) * real(no)
		if ( grid_typ(7).gt.0.5 ) then
			! first dimension is always wealth

			xx(indx) = dexp(xx(indx)) - grid_typ(8)
		else

			xx(indx) = xx(indx)
		end if
		if ( abs(xx(indx)).lt.1E-6 ) then

			xx(indx) = 0.0
		end if
	end if
	if ( (UI_state.eq.1).and.(current_age.le.nint(model_par(5))) ) then
		! next dimension is unemployment insurance

		indx = indx + 1
		counter = 1 + int( (real(i_inner_nw)-0.01) / real(product(dims_inner(2:indx-1))) )
		no = mod( (counter-1), dims_inner(indx) )
		xx(indx) = axis_am1(1,indx) + step_inner(indx) * real(no)
		if ( grid_typ(17).gt.0.5 ) then

			xx(indx) = dexp(xx(indx)) - grid_typ(18)
		else

			xx(indx) = xx(indx)
		end if
		if ( abs(xx(indx)).lt.1E-6 ) then

			xx(indx) = 0.0
		end if
		if ( (xx(indx).gt.0.1).and.(semp_state.eq.1) ) then
			! can only have UI if not self-employed

			flag_skip = 1
		end if
	end if
	if ( (current_age.le.nint(model_par(5))).and.(model_par(64).gt.0.5) ) then
		! next dimension is training

		if ( flag_ad.eq.0 ) then

			indx = indx + 1
			counter = 1 + int( (real(i_inner_nw)-0.01) / real(product(dims_inner(2:indx-1))) )
			no = mod( (counter-1), dims_inner(indx) )
			xx(indx) = axis_am1(1,indx) + step_inner(indx) * real(no)
			training_state = xx(indx)
		else

			training_state = training_state1
		end if
	end if
elseif ( ndim_inner.gt.1 ) then
	! run tests to consider whether slice should be omitted on basis of state space

	if ( current_age.le.nint(model_par(5)) ) then
		! next dimension is wage

		indx = indx + 1
		if ( estw2_par(1).gt.0.5 ) then
			! wages modelled on estimated earnings equation 2

			! persistent wage innovations
			if ( estw2_par(4).gt.0.5 ) then
				! temporary wage innovations

				indx = indx + 1
			end if
			if ( estw2_par(2).gt.0.5 ) then
				! individual specific effects

				indx = indx + 1
			end if
		else
			! wages not modelled on earnings equation 2

			continue
		end if
	end if
	if ( (model_par(11).gt.0.1).or.( (current_age.gt.min_page).and.(pen_par(156).gt.0.5) ) ) then
		! next dimension is OP pension annuity

		indx = indx + 1
	end if
	if ( (model_par(26).gt.0.1).and.(PPOP_state.eq.0).and.( (current_age.le.min_page).or.( (model_par(11).gt.0.1).and.(current_age.le.max_page) ) ) ) then
		! next dimension is PP pension annuity

		indx = indx + 1
	end if
	if ( ((semp_par(1)*semp_par(3)).gt.0.5).and.(current_age.le.nint(model_par(5))) ) then
		! next dimension is own-business assets

		indx = indx + 1
		if ( (xx(indx).gt.0.1).and.(semp_state.eq.0) ) then
			! can only have own business assets if self-employed

			flag_skip = 1
		end if
	end if
	if ( pen_par(21).gt.0.5 ) then
		! next dimension is contributory pension 1

		indx = indx + 1
		if ( (xx(indx).gt.(grid_par(6)/2.0+0.000001)).and.(na_g.lt.1.00001) ) then

			flag_skip = 1
		end if
	end if
	if ( (cp2_state.eq.1).or.((cp2_state.eq.2).and.(current_age.le.max_page)) ) then
		! next dimension is contributory pension 2

		indx = indx + 1
	end if
	if ( model_par(47).gt.0.5 ) then
		! next dimension is ISA

		indx = indx + 1
	end if
	if ( (UI_state.eq.1).and.(current_age.le.nint(model_par(5))) ) then
		! next dimension is unemployment insurance

		indx = indx + 1
		if ( (xx(indx).gt.0.1).and.(semp_state.eq.1) ) then
			! can only have unemployment insurance if employee

			flag_skip = 1
		end if
	end if
	if ( (current_age.le.nint(model_par(5))).and.(model_par(64).gt.0.5).and.(flag_ad.eq.0) ) then
		! next dimension is training

		indx = indx + 1
	end if
end if


END SUBROUTINE inner_state


SUBROUTINE emp_training_id(emp_counter, training_counter, emp_cost, prwage1, prwage2, emp1, emp2, training_gain, leisure, labour, ft_wage, flag_skip)

!*************************************************************************
!
!	subroutine to identify employment and training decisions to pass
!	to solution routines
!
!	OUTPUTS:
!	prwage1, prwage2 = proportion of value of human capital individual wage when estimatable wage equation 2 is used
!					   proportion of household wage otherwise
!	emp1, emp2 = 0 for not employed
!				 1 for part-time employed
!				 2 for full-time employed
!
!*************************************************************************

USE global
USE global_ParamStore
USE global_CommonAll
USE global_CommonCohort

IMPLICIT none


integer(4), intent(in) :: emp_counter, training_counter
integer(4), intent(out) :: flag_skip, emp1, emp2
real(8), intent(in) :: emp_cost, ft_wage
real(8), intent(out) :: prwage1, prwage2, training_gain, leisure, labour

! local variables
integer(4) :: train_ft
real(8) :: training_time, ftw1, ftw2, ptw1, ptw2, ptl, ftl1, ftl2, hwr, labour1, labour2, commute, nmwr, care_time


!*************************************************************************
! begin code
!*************************************************************************
flag_skip = 0
labour = 0.0
commute = 0.0
care_time = 0.0


!*************************************************************************
! determine employment status
!*************************************************************************
! identify employment state
if ( (model_par(18).lt.0.5).or.(na_g.lt.1.5) ) then
! model as single

	emp1 = emp_counter - 1
	emp2 = 0
else
! model as a couple

	emp1 = mod(emp_counter-1,lab_alt)
	emp2 = int(real(emp_counter-1)/real(lab_alt))
end if


!*************************************************************************
! determine training status
!*************************************************************************
train_ft = 0
if ( training_counter.eq.0 ) then
! no training

	training_gain = 0.0
	training_time = 0.0
elseif ( (training_counter.eq.1).and.(edu_par(20).gt.0.5) ) then
! training done part-time

	training_gain = 1.0 / edu_par(24)
	if ( (na_g.lt.1.5).or.(model_par(18).lt.0.5) ) then
	! treat as single

		if ( model_par(62).gt.0.5 ) then
		! education variable

			if ( edu_state.eq.1 ) then

				training_time = edu_par(85)
			elseif ( edu_state.eq.2 ) then

				training_time = edu_par(86)
			elseif ( edu_state.eq.3 ) then

				training_time = edu_par(87)
			elseif ( edu_state.eq.4 ) then

				training_time = edu_par(88)
			elseif ( edu_state.eq.5 ) then

				training_time = edu_par(89)
			end if
		else
		! education not variable

			training_time = edu_par(85)
		end if
	else
	! treat as couple

		if ( model_par(62).gt.0.5 ) then
		! education variable

			if ( edu_state.eq.1 ) then

				training_time = edu_par(95)
			elseif ( edu_state.eq.2 ) then

				training_time = edu_par(96)
			elseif ( edu_state.eq.3 ) then

				training_time = edu_par(97)
			elseif ( edu_state.eq.4 ) then

				training_time = edu_par(98)
			elseif ( edu_state.eq.5 ) then

				training_time = edu_par(99)
			end if
		else
		! education not variable

			training_time = edu_par(95)
		end if
	end if
else
! training done full-time

	train_ft = 1
	training_gain = 1.0 / edu_par(23)
	if ( (na_g.lt.1.5).or.(model_par(18).lt.0.5) ) then
	! treat as single

		if ( model_par(62).gt.0.5 ) then
		! education variable

			if ( edu_state.eq.1 ) then

				training_time = edu_par(80)
			elseif ( edu_state.eq.2 ) then

				training_time = edu_par(81)
			elseif ( edu_state.eq.3 ) then

				training_time = edu_par(82)
			elseif ( edu_state.eq.4 ) then

				training_time = edu_par(83)
			elseif ( edu_state.eq.5 ) then

				training_time = edu_par(84)
			end if
		else
		! education not variable

			training_time = edu_par(80)
		end if
	else
	! treat as couple

		if ( model_par(62).gt.0.5 ) then
		! education variable

			if ( edu_state.eq.1 ) then

				training_time = edu_par(90)
			elseif ( edu_state.eq.2 ) then

				training_time = edu_par(91)
			elseif ( edu_state.eq.3 ) then

				training_time = edu_par(92)
			elseif ( edu_state.eq.4 ) then

				training_time = edu_par(93)
			elseif ( edu_state.eq.5 ) then

				training_time = edu_par(94)
			end if
		else
		! education not variable

			training_time = edu_par(90)
		end if
	end if
end if


!*************************************************************************
! now check whether time use option is feasible
!*************************************************************************
if ( (train_ft.eq.1).and.(emp1.eq.(lab_alt-1)) ) then
! training full-time and working full-time

	if ( edu_par(16).lt.0.5 ) then
	! this option not permitted

		flag_skip = 1
	end if
elseif ( emp2.gt.emp1 ) then
! then we are testing a case in which the low wage earner works more than the high wage earner
! need to consider if there is a good reason for this

	if ( (training_gain.gt.0.01) ) then
	! training undertaken - allow this option

		continue
	elseif ( (emp_par(3).gt.0.5).and.(emp1.eq.0) ) then
	! low wage offers taken into account and highest wage earner not working - allow this option

		continue
	else
	! no other good reason for this - discard this option

		flag_skip = 1
	end if
end if
if ( stud_state.gt.0 ) then
! student in prevailing period

	if ( (edu_par(8).gt.0.5).and.(emp1.eq.(lab_alt-1)) ) then
	! cannot work full-time when student

		flag_skip = 1
	elseif ( (edu_par(7).gt.0.5).and.(emp1.gt.0) ) then
	! cannot work at all when student

		flag_skip = 1
	end if
end if
if ( hlth_axis_am1.eq.1 ) then

	if ( ((emp1.gt.0).and.(health1_state.gt.nint(hlth_par(12)))).or.((emp2.gt.0).and.(health2_state.gt.nint(hlth_par(12)))) ) then
	! testing employment when health state prohibits working

		flag_skip = 1
	end if
	if ( emp1.gt.nint(hlth_par(14+health1_state)) ) then

		flag_skip = 1
	end if
	if ( (emp2.gt.nint(hlth_par(14+health2_state))).and.(na_g.gt.1.5).and.(model_par(18).gt.0.5) ) then

		flag_skip = 1
	end if
end if
if ( care_axis_am1.eq.1 ) then

	if ( care_state.eq.1 ) then

		care_time = hlth_par(27) / na_g
		if ( na_g.lt.1.5 ) then
		! treat as single adult

			if ( emp1.gt.nint(hlth_par(26)) ) then

				flag_skip = 1
			end if
		else
		! treat as a couple

			if ( min(emp1,emp2).gt.nint(hlth_par(26)) ) then

				flag_skip = 1
			end if
		end if
	end if
end if


!*************************************************************************
! compute remaining statistics if this option is viable
!*************************************************************************
if ( flag_skip.eq.0 ) then


	!**************************************************
	! first calculate wages and labour hours
	!**************************************************
	if ( (na_g.lt.1.5).or.(model_par(18).lt.0.5) ) then
	! treat as single adult

		prwage2 = 0.0
		if ( (emp_par(1).lt.0.5).and.(emp_par(3).lt.0.5).and.(current_age.lt.spa_bc) ) then
		! can only work

			emp1 = 1
			prwage1 = 1.0
			labour = emp_cost
			commute = dyn_pars(5)
		elseif ( emp1.eq.0 ) then
		! don't work

			prwage1 = 0.0
			labour = 0.0
			commute = 0.0
		elseif ( emp1.eq.(lab_alt-1) ) then
		! cohort member full-time employed

			if ( estw_par(1).gt.0.5 ) then
			! estimatable wage equation 1

				prwage1 = 1.0
				labour = emp_cost
				commute = dyn_pars(5)
			elseif ( estw2_par(1).gt.0.5 ) then
			! estimatable wage equation 2

				prwage1 = 1.0
				labour = estw2_par(26) / emp_par(6)
				commute = dyn_pars(5)
			else
			! not estimatable wage equation

				prwage1 = 1.0
				labour = emp_cost
				commute = dyn_pars(5)
			end if
		elseif ( (emp_par(1).gt.0.5).and.(emp_par(2).gt.0.5) ) then
		! cohort member part-time employed

			if ( estw_par(1).gt.0.5 ) then
			! estimatable wage equation 1

				prwage1 = dexp( estw_par(7) + estw_par(8)*real(current_age) )
				labour = emp_cost * dyn_pars(6)
				commute = dyn_pars(5)
			elseif ( estw2_par(1).gt.0.5 ) then
			! estimatable wage equation 2

				prwage1 = dexp( estw2_par(12) )
				labour = estw2_par(27) / emp_par(6)
				commute = dyn_pars(5)
			else
			! not estimatable wage equation

				prwage1 = s_ptftw(age_index)
				labour = emp_cost * dyn_pars(6)
				commute = dyn_pars(5)
			end if
		elseif ( emp_par(22).gt.0.5 ) then
		! dynamic employment options

			prwage1 = real(emp1) / real(lab_alt-1)
			labour = emp_cost * prwage1
			commute = dyn_pars(5)
		else
		! option not recognised

			!$OMP CRITICAL(error_1)
			print *, 'employment option not recognised'
			pause 'press enter to exit'
			stop
			continue
			!$OMP END CRITICAL(error_1)
		end if
		if ( (emp_par(5).gt.0.5).and.(emp1.gt.0).and.(semp_state.eq.0) ) then
		! test for minimum wage offer

			if ( current_age.lt.nint(emp_par(38)) ) then

				nmwr = emp_par(43)*model_index(current_year-year0_param+1,1)
			elseif ( current_age.lt.nint(emp_par(39)) ) then

				nmwr = emp_par(44)*model_index(current_year-year0_param+1,2)
			elseif ( current_age.lt.nint(emp_par(40)) ) then

				nmwr = emp_par(45)*model_index(current_year-year0_param+1,3)
			else

				nmwr = emp_par(46)*model_index(current_year-year0_param+1,4)
			end if
			hwr = (prwage1 * ft_wage * (1.0 - wage_penalty) * ng_wf * hl_wf * tm_wf) / (labour/emp_cost * emp_par(36))
			if ( hwr.lt.nmwr ) then

				flag_skip = 1
			end if
		end if
	else
	! treat as a couple

		! set up employment parameters
		if ( estw_par(1).gt.0.5 ) then
		! estimatable wage equation 1

			ftw1 = dexp( estw_par(19) + estw_par(22)*real(current_age) )
			ptw1 = dexp( estw_par(20) + estw_par(23)*real(current_age) )
			ftl1 = emp_cost * dyn_parc(11)
			ftl2 = emp_cost * (1.0 - dyn_parc(11))
			ptl = emp_cost * dyn_parc(13)
			ftw2 = 1.0 - ftw1
			ptw2 = dexp( estw_par(18) + estw_par(21)*real(current_age) ) - ftw1
		elseif ( estw2_par(1).gt.0.5 ) then
		! estimatable wage equation 2

			ftw1 = 1.0
			ptw1 = dexp( estw2_par(12) )
			ftl1 = estw2_par(26) / (emp_par(6) * 2.0)
			ftl2 = ftl1
			ptl = estw2_par(27) / (emp_par(6) * 2.0)
			ftw2 = 1.0
			ptw2 = dexp( estw2_par(34) )
		else
		! not estimatable wage equation

			ftw1 = dyn_parc(12)
			ptw1 = c_ptftw(age_index)
			ftl1 = emp_cost * dyn_parc(11)
			ftl2 = emp_cost * (1.0 - dyn_parc(11))
			ptl = emp_cost * dyn_parc(13)
			ftw2 = 1.0 - ftw1
			ptw2 = dyn_parc(10) - ftw1
		end if
		! now evaluate prwage and labour
		if ( (emp_par(1).lt.0.5).and.(emp_par(3).lt.0.5).and.(current_age.lt.spa_bc) ) then
		! can only work

			emp1 = 1
			prwage1 = ftw1
			labour1 = ftl1
			emp2 = 1
			prwage2 = ftw2
			labour2 = ftl1
			commute = dyn_parc(5)
		elseif ( emp1.eq.0 ) then
		! cohort member not employed

			prwage1 = 0.0
			labour1 = 0.0
			commute = 0.0
			if ( emp2.eq.0 ) then
			! spouse not employed

				prwage2 = 0.0
				labour2 = 0.0
			elseif ( emp2.eq.(lab_alt-1) ) then
			! spouse full-time employed

				prwage2 = ftw2
				labour2 = ftl1
				commute = commute + dyn_parc(5) * 0.5
			elseif ( (emp_par(1).gt.0.5).and.(emp_par(2).gt.0.5) ) then
			! spouse part-time employed

				prwage2 = ptw2
				labour2 = ptl
				commute = commute + dyn_parc(5) * 0.5
			elseif ( emp_par(22).gt.0.5 ) then
			! dynamic employment options

				prwage2 = ftw2 * real(emp2) / real(lab_alt-1)
				labour2 = ftl1 * real(emp2) / real(lab_alt-1)
				commute = commute + dyn_parc(5) * 0.5
			else
			! option not recognised

				!$OMP CRITICAL(error_2)
				print *, 'employment option not recognised'
				pause 'press enter to exit'
				stop
				continue
				!$OMP END CRITICAL(error_2)
			end if
		elseif ( emp1.eq.(lab_alt-1) ) then
		! cohort member full-time employed

			prwage1 = ftw1
			labour1 = ftl1
			commute = dyn_parc(5) * 0.5
			if ( emp2.eq.0 ) then
			! spouse not employed

				prwage2 = 0.0
				labour2 = 0.0
			elseif ( emp2.eq.(lab_alt-1) ) then
			! spouse full-time employed

				prwage2 = ftw2
				labour2 = ftl2
				commute = commute + dyn_parc(5) * 0.5
			elseif ( (emp_par(1).gt.0.5).and.(emp_par(2).gt.0.5) ) then
			! spouse part-time employed

				prwage2 = ptw2
				labour2 = ptl
				commute = commute + dyn_parc(5) * 0.5
			elseif ( emp_par(22).gt.0.5 ) then
			! dynamic employment options

				prwage2 = ftw2 * real(emp2) / real(lab_alt-1)
				labour2 = ftl2 * real(emp2) / real(lab_alt-1)
				commute = commute + dyn_parc(5) * 0.5
			else
			! option not recognised

				!$OMP CRITICAL(error_3)
				print *, 'employment option not recognised'
				pause 'press enter to exit'
				stop
				continue
				!$OMP END CRITICAL(error_3)
			end if
		elseif ( (emp_par(1).gt.0.5).and.(emp_par(2).gt.0.5) ) then
		! cohort member part-time employed

			prwage1 = ptw1
			labour1 = ptl
			commute = dyn_parc(5) * 0.5
			if ( emp2.eq.0 ) then
			! spouse not employed

				prwage2 = 0.0
				labour2 = 0.0
			elseif ( emp2.eq.(lab_alt-1) ) then
			! spouse full-time employed

				prwage2 = ftw2
				labour2 = ftl1
				commute = commute + dyn_parc(5) * 0.5
			else
			! spouse part-time employed

				prwage2 = ptw2
				labour2 = ptl
				commute = commute + dyn_parc(5) * 0.5
			end if
		elseif ( emp_par(22).gt.0.5 ) then
		! dynamic employment options

			prwage1 = ftw1 * real(emp1) / real(lab_alt-1)
			if ( emp1.ge.emp2 ) then

				labour1 = ftl1 * real(emp1) / real(lab_alt-1)
			else

				labour1 = ftl2 * real(emp1) / real(lab_alt-1)
			end if
			commute = dyn_parc(5) * 0.5
			if ( emp2.eq.0 ) then
			! spouse not employed

				prwage2 = 0.0
				labour2 = 0.0
			else

				prwage2 = ftw2 * real(emp2) / real(lab_alt-1)
				if ( emp1.ge.emp2 ) then

					labour2 = ftl2 * real(emp2) / real(lab_alt-1)
				else

					labour2 = ftl1 * real(emp2) / real(lab_alt-1)
				end if
				commute = commute + dyn_parc(5) * 0.5
			end if
		else
		! option not recognised

			!$OMP CRITICAL(error_4)
			print *, 'employment option not recognised'
			pause 'press enter to exit'
			stop
			continue
			!$OMP END CRITICAL(error_4)
		end if
		labour = labour1 + labour2
		if ( current_age.lt.nint(emp_par(38)) ) then

			nmwr = emp_par(43)*model_index(current_year-year0_param+1,1)
		elseif ( current_age.lt.nint(emp_par(39)) ) then

			nmwr = emp_par(44)*model_index(current_year-year0_param+1,2)
		elseif ( current_age.lt.nint(emp_par(40)) ) then

			nmwr = emp_par(45)*model_index(current_year-year0_param+1,3)
		else

			nmwr = emp_par(46)*model_index(current_year-year0_param+1,4)
		end if
		if ( (emp_par(5).gt.0.5).and.(emp1.gt.0).and.(semp_state.eq.0) ) then
		! test for minimum wage offer

			hwr = (prwage1 * ft_wage * (1.0 - wage_penalty) * ng_wf * hl_wf * tm_wf) / (labour1*2.0/emp_cost * emp_par(37))
			if ( hwr.lt.nmwr ) then

				flag_skip = 1
			end if
		end if
		if ( (emp_par(5).gt.0.5).and.(emp2.gt.0).and.(semp_state.eq.0) ) then
		! test for minimum wage offer

			hwr = (prwage2 * ft_wage * (1.0 - wage_penalty) * ng_wf * hl_wf * tm_wf) / (labour2*2.0/emp_cost * emp_par(37))
			if ( hwr.lt.nmwr ) then

				flag_skip = 1
			end if
		end if
	end if	! singles/couples (wages)
end if	! skip / don't skip

leisure = 1.0 - labour - training_time - commute - care_time


END SUBROUTINE emp_training_id


END MODULE utility_maxn